searchData={"items":[{"type":"module","title":"Money","doc":"Money implements a set of functions to store, retrieve, convert and perform\narithmetic on a `t:Money.t/0` type that is composed of a currency code and\na decimal currency amount.\n\nMoney is very opinionated in the interests of serving as a dependable library\nthat can underpin accounting and financial applications.\n\nThis opinion expressed by ensuring that:\n\n1. Money must always have both a amount and a currency code.\n\n2. The currency code must always be valid.\n\n3. Money arithmetic can only be performed when both operands are of the\n   same currency.\n\n4. Money amounts are represented as a `Decimal`.\n\n5. Money is serialised to the database as a custom Postgres composite type\n   that includes both the amount and the currency. Therefore for Ecto\n   serialization Postgres is assumed as the data store. Serialization is\n   entirely optional and Ecto is not a package dependency.\n\n6. All arithmetic functions work in fixed point decimal.  No rounding\n   occurs automatically (unless expressly called out for a function).\n\n7. Explicit rounding obeys the rounding rules for a given currency.  The\n   rounding rules are defined by the Unicode consortium in its CLDR\n   repository as implemented by the hex package `ex_cldr`.  These rules\n   define the number of fractional digits for a currency and the rounding\n   increment where appropriate.","ref":"Money.html"},{"type":"function","title":"Money.abs/1","doc":"The absolute value of a `t:Money.t/0` amount.\nReturns a `t:Money.t/0` type with a positive sign for the amount.","ref":"Money.html#abs/1"},{"type":"function","title":"Arguments - Money.abs/1","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`.","ref":"Money.html#abs/1-arguments"},{"type":"function","title":"Returns - Money.abs/1","doc":"* a `t:Money.t/0`.","ref":"Money.html#abs/1-returns"},{"type":"function","title":"Example - Money.abs/1","doc":"iex> m = Money.new(\"USD\", -100)\n    iex> Money.abs(m)\n    Money.new(:USD, \"100\")","ref":"Money.html#abs/1-example"},{"type":"function","title":"Money.add/2","doc":"Add two `Money` values.","ref":"Money.html#add/2"},{"type":"function","title":"Arguments - Money.add/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`.","ref":"Money.html#add/2-arguments"},{"type":"function","title":"Returns - Money.add/2","doc":"* `{:ok, money}` or\n\n* `{:error, reason}`.","ref":"Money.html#add/2-returns"},{"type":"function","title":"Example - Money.add/2","doc":"iex> Money.add Money.new(:USD, 200), Money.new(:USD, 100)\n    {:ok, Money.new(:USD, 300)}\n\n    iex> Money.add Money.new(:USD, 200), Money.new(:AUD, 100)\n    {:error, {ArgumentError, \"Cannot add monies with different currencies. \" <>\n      \"Received :USD and :AUD.\"}}","ref":"Money.html#add/2-example"},{"type":"function","title":"Money.add!/2","doc":"Add two `t:Money.t/0` values or raise on error.","ref":"Money.html#add!/2"},{"type":"function","title":"Arguments - Money.add!/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`.","ref":"Money.html#add!/2-arguments"},{"type":"function","title":"Returns - Money.add!/2","doc":"* a `t:Money.t/0` struct or\n\n* raises an exception","ref":"Money.html#add!/2-returns"},{"type":"function","title":"Examples - Money.add!/2","doc":"iex> Money.add! Money.new(:USD, 200), Money.new(:USD, 100)\n    Money.new(:USD, \"300\")\n\n    Money.add! Money.new(:USD, 200), Money.new(:CAD, 500)\n    ** (ArgumentError) Cannot add two %Money{} with different currencies. Received :USD and :CAD.","ref":"Money.html#add!/2-examples"},{"type":"type","title":"Money.amount/0","doc":"An amount can be expressed as a float, an integer,\na Decimal or a string (which is converted to a Decimal)","ref":"Money.html#t:amount/0"},{"type":"function","title":"Money.clamp/3","doc":"Clamps a `t:Money.t/0` to be in the range of `minimum`\nto `maximum`.\n\n#### Arguments\n\n* `money`, `minimum` and `maximum` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`. They should be of the same currency.\n\n#### Returns\n\n* `{:ok, money]` where `money` is clamped to the `minimum` or `maximum` if required.\n    * If `money` is within the range `minimum..maximum` then `money` is returned unchanged.\n    * If `money` is less than `minimum` then `minimum` is returned.\n    * If `money` is greater than `maximum` then `maximum` is returned.\n\n* or `{:error, reason}`.\n\n#### Examples\n\n    iex> Money.clamp(Money.new(:USD, 100), Money.new(:USD, 50), Money.new(:USD, 200))\n    {:ok, Money.new(:USD, 100)}\n\n    iex> Money.clamp(Money.new(:USD, 300), Money.new(:USD, 50), Money.new(:USD, 200))\n    {:ok, Money.new(:USD, 200)}\n\n    iex> Money.clamp(Money.new(:USD, 10), Money.new(:USD, 50), Money.new(:USD, 200))\n    {:ok, Money.new(:USD, 50)}\n\n    iex> Money.clamp(Money.new(:USD, 10), Money.new(:USD, 300), Money.new(:USD, 200))\n    {:error,\n      {ArgumentError,\n        \"Minimum must be less than maximum. Found Money.new(:USD, \\\"300\\\") and Money.new(:USD, \\\"200\\\")\"}}\n\n    iex> Money.clamp(Money.new(:USD, 10), Money.new(:AUD, 300), Money.new(:EUR, 200))\n    {:error, {ArgumentError, \"Cannot compare monies with different currencies. Received :USD, :AUD and :EUR\"}}","ref":"Money.html#clamp/3"},{"type":"function","title":"Money.clamp!/3","doc":"Clamps a `t:Money.t/0` to be in the range of `minimum`\nto `maximum` or raises an exception.\n\n#### Arguments\n\n* `money`, `minimum` and `maximum` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`. They should be of the same currency.\n\n#### Returns\n\n* `money` where `money` is clamped to the `minimum` or `maximum` if required.\n    * If `money` is within the range `minimum..maximum` then `money` is returned unchanged.\n    * If `money` is less than `minimum` then `minimum` is returned.\n    * If `money` is greater than `maximum` then `maximum` is returned.\n\n* or `{:error, reason}`.\n\n#### Examples\n\n    iex> Money.clamp!(Money.new(:USD, 100), Money.new(:USD, 50), Money.new(:USD, 200))\n    Money.new(:USD, 100)\n\n    iex> Money.clamp!(Money.new(:USD, 300), Money.new(:USD, 50), Money.new(:USD, 200))\n    Money.new(:USD, 200)\n\n    iex> Money.clamp!(Money.new(:USD, 10), Money.new(:USD, 50), Money.new(:USD, 200))\n    Money.new(:USD, 50)\n\n    iex> Money.clamp!(Money.new(:USD, 10), Money.new(:USD, 300), Money.new(:USD, 200))\n    ** (ArgumentError) Minimum must be less than maximum. Found Money.new(:USD, \"300\") and Money.new(:USD, \"200\")\n\n    iex> Money.clamp!(Money.new(:USD, 10), Money.new(:AUD, 300), Money.new(:EUR, 200))\n    ** (ArgumentError) Cannot compare monies with different currencies. Received :USD, :AUD and :EUR","ref":"Money.html#clamp!/3"},{"type":"function","title":"Money.cmp/2","doc":"Compares two `Money` values numerically. If the first number is greater\nthan the second #Integer<1> is returned, if less than Integer<-1> is\nreturned. Otherwise, if both numbers are equal Integer<0> is returned.","ref":"Money.html#cmp/2"},{"type":"function","title":"Arguments - Money.cmp/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`.","ref":"Money.html#cmp/2-arguments"},{"type":"function","title":"Returns - Money.cmp/2","doc":"*  `-1` | `0` | `1` or\n\n* `{:error, {module(), String.t}}`","ref":"Money.html#cmp/2-returns"},{"type":"function","title":"Examples - Money.cmp/2","doc":"iex> Money.cmp Money.new(:USD, 200), Money.new(:USD, 100)\n    1\n\n    iex> Money.cmp Money.new(:USD, 200), Money.new(:USD, 200)\n    0\n\n    iex> Money.cmp Money.new(:USD, 200), Money.new(:USD, 500)\n    -1\n\n    iex> Money.cmp Money.new(:USD, 200), Money.new(:CAD, 500)\n    {:error,\n     {ArgumentError,\n      \"Cannot compare monies with different currencies. Received :USD and :CAD.\"}}","ref":"Money.html#cmp/2-examples"},{"type":"function","title":"Money.cmp!/2","doc":"Compares two `Money` values numerically and raises on error.","ref":"Money.html#cmp!/2"},{"type":"function","title":"Arguments - Money.cmp!/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`","ref":"Money.html#cmp!/2-arguments"},{"type":"function","title":"Returns - Money.cmp!/2","doc":"*  `-1` | `0` | `1` or\n\n* raises an exception","ref":"Money.html#cmp!/2-returns"},{"type":"function","title":"Examples - Money.cmp!/2","doc":"Money.cmp! Money.new(:USD, 200), Money.new(:CAD, 500)\n    ** (ArgumentError) Cannot compare monies with different currencies. Received :USD and :CAD.","ref":"Money.html#cmp!/2-examples"},{"type":"function","title":"Money.compare/2","doc":"Compares two `Money` values numerically. If the first number is greater\nthan the second :gt is returned, if less than :lt is returned, if both\nnumbers are equal :eq is returned.","ref":"Money.html#compare/2"},{"type":"function","title":"Arguments - Money.compare/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`","ref":"Money.html#compare/2-arguments"},{"type":"function","title":"Returns - Money.compare/2","doc":"*  `:gt` | `:eq` | `:lt` or\n\n* `{:error, {module(), String.t}}`","ref":"Money.html#compare/2-returns"},{"type":"function","title":"Examples - Money.compare/2","doc":"iex> Money.compare Money.new(:USD, 200), Money.new(:USD, 100)\n    :gt\n\n    iex> Money.compare Money.new(:USD, 200), Money.new(:USD, 200)\n    :eq\n\n    iex> Money.compare Money.new(:USD, 200), Money.new(:USD, 500)\n    :lt\n\n    iex> Money.compare Money.new(:USD, 200), Money.new(:CAD, 500)\n    {:error,\n     {ArgumentError,\n      \"Cannot compare monies with different currencies. Received :USD and :CAD.\"}}","ref":"Money.html#compare/2-examples"},{"type":"function","title":"Money.compare!/2","doc":"Compares two `Money` values numerically and raises on error.","ref":"Money.html#compare!/2"},{"type":"function","title":"Arguments - Money.compare!/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`.","ref":"Money.html#compare!/2-arguments"},{"type":"function","title":"Returns - Money.compare!/2","doc":"*  `:gt` | `:eq` | `:lt` or\n\n* raises an exception","ref":"Money.html#compare!/2-returns"},{"type":"function","title":"Examples - Money.compare!/2","doc":"Money.compare! Money.new(:USD, 200), Money.new(:CAD, 500)\n    ** (ArgumentError) Cannot compare monies with different currencies. Received :USD and :CAD.","ref":"Money.html#compare!/2-examples"},{"type":"function","title":"Money.cross_rate/3","doc":"Returns the effective cross-rate to convert from one currency\nto another.","ref":"Money.html#cross_rate/3"},{"type":"function","title":"Arguments - Money.cross_rate/3","doc":"* `from` is any `t:Money.t/0` struct returned by `Cldr.Currency.new/2` or a valid\n   currency code.\n\n* `to_currency` is a valid currency code into which the `money` is converted.\n\n* `rates` is a `Map` of currency rates where the map key is an upcased\n  atom or string and the value is a Decimal conversion factor.  The default is the\n  latest available exchange rates returned from `Money.ExchangeRates.latest_rates/0`.","ref":"Money.html#cross_rate/3-arguments"},{"type":"function","title":"Examples - Money.cross_rate/3","doc":"Money.cross_rate(Money.new(:USD, 100), :AUD, %{USD: Decimal.new(1), AUD: Decimal.new(\"0.7345\")})\n    {:ok, Decimal.new(\"0.7345\")}\n\n    Money.cross_rate Money.new(:USD, 100), :ZZZ, %{USD: Decimal.new(1), AUD: Decimal.new(0.7345)}\n    ** (Cldr.UnknownCurrencyError) Currency :ZZZ is not known","ref":"Money.html#cross_rate/3-examples"},{"type":"function","title":"Money.cross_rate!/3","doc":"Returns the effective cross-rate to convert from one currency\nto another.","ref":"Money.html#cross_rate!/3"},{"type":"function","title":"Arguments - Money.cross_rate!/3","doc":"* `from` is any `t:Money.t/0` struct returned by `Cldr.Currency.new/2` or a valid\n   currency code.\n\n* `to_currency` is a valid currency code into which the `money` is converted.\n\n* `rates` is a `Map` of currency rates where the map key is an upcased\n  atom or string and the value is a Decimal conversion factor.  The default is the\n  latest available exchange rates returned from `Money.ExchangeRates.latest_rates/0`.","ref":"Money.html#cross_rate!/3-arguments"},{"type":"function","title":"Examples - Money.cross_rate!/3","doc":"iex> Money.cross_rate!(Money.new(:USD, 100), :AUD, %{USD: Decimal.new(1), AUD: Decimal.new(\"0.7345\")})\n    Decimal.new(\"0.7345\")\n\n    iex> Money.cross_rate!(:USD, :AUD, %{USD: Decimal.new(1), AUD: Decimal.new(\"0.7345\")})\n    Decimal.new(\"0.7345\")\n\n    Money.cross_rate Money.new(:USD, 100), :ZZZ, %{USD: Decimal.new(1), AUD: Decimal.new(\"0.7345\")}\n    ** (Cldr.UnknownCurrencyError) Currency :ZZZ is not known","ref":"Money.html#cross_rate!/3-examples"},{"type":"type","title":"Money.currency_code/0","doc":"A currency code is an ISO 4217 code expressed\nas an atom or binary or an ISO 24165 Digital\nToken ID or Digital Token short name.","ref":"Money.html#t:currency_code/0"},{"type":"function","title":"Money.default_backend/0","doc":"","ref":"Money.html#default_backend/0"},{"type":"function","title":"Money.default_backend!/0","doc":"Returns the default `ex_cldr` backend configured\nfor `Money`, if any. If no default backing is\nconfigured, an exception is raised.","ref":"Money.html#default_backend!/0"},{"type":"function","title":"Money.default_rounding_mode/0","doc":"Returns the default rounding mode.","ref":"Money.html#default_rounding_mode/0"},{"type":"function","title":"Money.div/2","doc":"Divide a `Money` value by a number.","ref":"Money.html#div/2"},{"type":"function","title":"Arguments - Money.div/2","doc":"* `money` is any valid `t:Money.t/0` types returned\n  by `Money.new/2`\n\n* `number` is an integer, float or `Decimal.t`\n\n> Note that dividing one %Money{} by another is not supported.","ref":"Money.html#div/2-arguments"},{"type":"function","title":"Returns - Money.div/2","doc":"* `{:ok, money}` or\n\n* `{:error, reason}`","ref":"Money.html#div/2-returns"},{"type":"function","title":"Example - Money.div/2","doc":"iex> Money.div Money.new(:USD, 200), 2\n    {:ok, Money.new(:USD, 100)}\n\n    iex> Money.div(Money.new(:USD, 200), \"xx\")\n    {:error, {ArgumentError, \"Cannot divide money by \\\"xx\\\"\"}}","ref":"Money.html#div/2-example"},{"type":"function","title":"Money.div!/2","doc":"Divide a `Money` value by a number and raise on error.","ref":"Money.html#div!/2"},{"type":"function","title":"Arguments - Money.div!/2","doc":"* `money` is any valid `t:Money.t/0` types returned\n  by `Money.new/2`\n\n* `number` is an integer, float or `Decimal.t`","ref":"Money.html#div!/2-arguments"},{"type":"function","title":"Returns - Money.div!/2","doc":"* a `t:Money.t/0` struct or\n\n* raises an exception","ref":"Money.html#div!/2-returns"},{"type":"function","title":"Examples - Money.div!/2","doc":"iex> Money.div!(Money.new(:USD, 200), 2)\n    Money.new(:USD, \"100\")\n\n    iex> Money.div!(Money.new(:USD, 200), \"xx\")\n    ** (ArgumentError) Cannot divide money by \"xx\"","ref":"Money.html#div!/2-examples"},{"type":"function","title":"Money.equal?/2","doc":"Returns a boolean indicating if two `Money` values are equal","ref":"Money.html#equal?/2"},{"type":"function","title":"Arguments - Money.equal?/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`","ref":"Money.html#equal?/2-arguments"},{"type":"function","title":"Returns - Money.equal?/2","doc":"* `true` or `false`","ref":"Money.html#equal?/2-returns"},{"type":"function","title":"Example - Money.equal?/2","doc":"iex> Money.equal?(Money.new(:USD, 200), Money.new(:USD, 200))\n    true\n\n    iex> Money.equal?(Money.new(:USD, 200), Money.new(:USD, 100))\n    false","ref":"Money.html#equal?/2-example"},{"type":"function","title":"Money.from_float/3","doc":"Returns a %Money{} struct from a currency code and a float amount, or\nan error tuple of the form `{:error, {exception, message}}`.\n\nFloats are fraught with danger in computer arithmetic due to the\nunexpected loss of precision during rounding. The IEEE754 standard\nindicates that a number with a precision of 16 digits should\nround-trip convert without loss of fidelity. This function supports\nnumbers with a precision up to 15 digits and will error if the\nprovided amount is outside that range.\n\n**Note** that `Money` cannot detect lack of precision or rounding errors\nintroduced upstream. This function therefore should be used with\ngreat care and its use should be considered potentially harmful.","ref":"Money.html#from_float/3"},{"type":"function","title":"Arguments - Money.from_float/3","doc":"* `currency_code` is an [ISO4217](https://en.wikipedia.org/wiki/ISO_4217)\n  binary or atom currency code or an\n  [ISO 24165](https://www.iso.org/standard/80601.html) token identifier or shortname.\n\n* `amount` is a float\n\n* `options` is a keyword list of options passed\n  to `Money.new/3`. The default is `[]`.","ref":"Money.html#from_float/3-arguments"},{"type":"function","title":"Examples - Money.from_float/3","doc":"iex> Money.from_float 1.23456, :USD\n    Money.new(:USD, \"1.23456\")\n\n    iex> Money.from_float 1.234567890987656, :USD\n    {:error,\n      {Money.InvalidAmountError,\n        \"The precision of the float 1.234567890987656 is \" <>\n        \"greater than 15 which could lead to unexpected results. \" <>\n        \"Reduce the precision or call Money.new/2 with a Decimal or String amount\"}}","ref":"Money.html#from_float/3-examples"},{"type":"function","title":"Money.from_float!/3","doc":"Returns a %Money{} struct from a currency code and a float amount, or\nraises an exception if the currency code is invalid.\n\nSee `Money.from_float/2` for further information.\n\n**Note** that `Money` cannot detect lack of precision or rounding errors\nintroduced upstream. This function therefore should be used with\ngreat care and its use should be considered potentially harmful.","ref":"Money.html#from_float!/3"},{"type":"function","title":"Arguments - Money.from_float!/3","doc":"* `currency_code` is an [ISO4217](https://en.wikipedia.org/wiki/ISO_4217)\n  binary or atom currency code or an\n  [ISO 24165](https://www.iso.org/standard/80601.html) token identifier or shortname.\n\n* `amount` is a float.\n\n* `options` is a keyword list of options passed\n  to `Money.new/3`. The default is `[]`.","ref":"Money.html#from_float!/3-arguments"},{"type":"function","title":"Examples - Money.from_float!/3","doc":"iex> Money.from_float!(:USD, 1.234)\n    Money.new(:USD, \"1.234\")\n\n    Money.from_float!(:USD, 1.234567890987654)\n    #=> ** (Money.InvalidAmountError) The precision of the float 1.234567890987654 is greater than 15 which could lead to unexpected results. Reduce the precision or call Money.new/2 with a Decimal or String amount\n        (ex_money) lib/money.ex:293: Money.from_float!/2","ref":"Money.html#from_float!/3-examples"},{"type":"function","title":"Money.from_integer/3","doc":"Convert an integer representation of money into a `Money` struct.","ref":"Money.html#from_integer/3"},{"type":"function","title":"Arguments - Money.from_integer/3","doc":"* `integer` is an integer representation of a money amount including\n  any decimal digits.  ie. `20000` would be interpreted to mean `$200.00`\n  if the `currency` is `:USD` and no `:currency_digits` option\n  was provided.\n\n* `currency` is the currency code for the `integer`.  The assumed\n  decimal precision is derived from the currency code if no `fractional_digits`\n  option is specified.\n\n* `options` is a keyword list of options.","ref":"Money.html#from_integer/3-arguments"},{"type":"function","title":"Options - Money.from_integer/3","doc":"* `:currency_digits` which determines the currency precision implied\n  by the `integer`. The valid options are `:cash`, `:accounting`,\n  `:iso` or a non-negative integer. The default is `:iso` which uses the\n  [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217) definition of\n  currency digits.\n\nAll other options are passed to `Money.new/3`.","ref":"Money.html#from_integer/3-options"},{"type":"function","title":"Returns - Money.from_integer/3","doc":"* A `t:Money.t/0` struct or\n\n* `{:error, {exception, message}}`","ref":"Money.html#from_integer/3-returns"},{"type":"function","title":"Notes - Money.from_integer/3","doc":"* Some currencies, like the [Iraqi Dinar](https://en.wikipedia.org/wiki/Iraqi_dinar)\n  have a difference in the decimal digits defined by CLDR versus\n  those defined by [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217). CLDR\n  defines the decimal digits for `IQD` as `0` whereas ISO 4217 defines\n  `3` decimal digits.\n\n* Since converting an integer to a money amount is very\n  sensitive to the number of fractional digits specified it is\n  important to be very clear about the precision of the data used\n  with this function and care taken in specifying the `:fractional_digits`\n  parameter.","ref":"Money.html#from_integer/3-notes"},{"type":"function","title":"Examples - Money.from_integer/3","doc":"iex> Money.from_integer(20000, :USD)\n    Money.new(:USD, \"200.00\")\n\n    iex> Money.from_integer(200, :JPY)\n    Money.new(:JPY, \"200\")\n\n    iex> Money.from_integer(20012, :USD)\n    Money.new(:USD, \"200.12\")\n\n    iex> Money.from_integer(20012, :USD, currency_digits: 3)\n    Money.new(:USD, \"20.012\")\n\n    iex> Money.from_integer(20012, :IQD)\n    Money.new(:IQD, \"20.012\")","ref":"Money.html#from_integer/3-examples"},{"type":"function","title":"Money.get_env/3","doc":"","ref":"Money.html#get_env/3"},{"type":"function","title":"Money.integer?/1","doc":"Returns a boolean indicating if `t:Money.t/0`\nis an integer value (has no significant fractional\ndigits).","ref":"Money.html#integer?/1"},{"type":"function","title":"Arguments - Money.integer?/1","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`","ref":"Money.html#integer?/1-arguments"},{"type":"function","title":"Example - Money.integer?/1","doc":"iex> Money.integer?(Money.new(:USD, 0))\n    true\n\n    iex> Money.integer?(Money.new(:USD, \"1.10\"))\n    false","ref":"Money.html#integer?/1-example"},{"type":"function","title":"Money.known_currencies/0","doc":"","ref":"Money.html#known_currencies/0"},{"type":"function","title":"Money.known_current_currencies/0","doc":"","ref":"Money.html#known_current_currencies/0"},{"type":"function","title":"Money.known_historic_currencies/0","doc":"","ref":"Money.html#known_historic_currencies/0"},{"type":"function","title":"Money.known_tender_currencies/0","doc":"","ref":"Money.html#known_tender_currencies/0"},{"type":"function","title":"Money.localize/2","doc":"Localizes a `Money` by converting it to the currency\nof the specified locale.","ref":"Money.html#localize/2"},{"type":"function","title":"Arguments - Money.localize/2","doc":"* `money` is any `t:Money.t/0` struct returned by `Cldr.Currency.new/2`.\n\n* `options` is a keyword list of options.","ref":"Money.html#localize/2-arguments"},{"type":"function","title":"Options - Money.localize/2","doc":"* `:locale` is any valid locale returned by `Cldr.known_locale_names/1`\n  or a `Cldr.LanguageTag` struct returned by `Cldr.Locale.new!/2`\n  The default is ` .get_locale()`\n\n* `:backend` is any module() that includes `use Cldr` and therefore\n  is a `Cldr` backend module(). The default is `Money.default_backend!/0`","ref":"Money.html#localize/2-options"},{"type":"function","title":"Returns - Money.localize/2","doc":"* `{:ok, localized_money}` or\n\n* `{:error, {exception, reason}}`","ref":"Money.html#localize/2-returns"},{"type":"function","title":"Money.max/2","doc":"Return the maximum of two `t:Money.t/0` amounts.","ref":"Money.html#max/2"},{"type":"function","title":"Arguments - Money.max/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`. `money_1` and `money_2` should be of the same\n  currency.","ref":"Money.html#max/2-arguments"},{"type":"function","title":"Returns - Money.max/2","doc":"* `{:ok, maximum_money}` or\n\n* `{:error, reason}`.","ref":"Money.html#max/2-returns"},{"type":"function","title":"Example - Money.max/2","doc":"iex> Money.max(Money.new(:USD, 200), Money.new(:USD, 300))\n    {:ok, Money.new(:USD, 300)}\n\n    iex> Money.max(Money.new(:USD, 200), Money.new(:AUD, 200))\n    {:error,\n      {ArgumentError, \"Cannot compare monies with different currencies. Received :USD and :AUD.\"}}","ref":"Money.html#max/2-example"},{"type":"function","title":"Money.max!/2","doc":"Return the maximum of two `t:Money.t/0` amounts or\nraises an exception.","ref":"Money.html#max!/2"},{"type":"function","title":"Arguments - Money.max!/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`. `money_1` and `money_2` should be of the same\n  currency.","ref":"Money.html#max!/2-arguments"},{"type":"function","title":"Returns - Money.max!/2","doc":"* `maximum_money` or\n\n* raises an exception.","ref":"Money.html#max!/2-returns"},{"type":"function","title":"Example - Money.max!/2","doc":"iex> Money.max!(Money.new(:USD, 200), Money.new(:USD, 300))\n    Money.new(:USD, 300)\n\n    iex> Money.max!(Money.new(:USD, 200), Money.new(:AUD, 200))\n    ** (ArgumentError) Cannot compare monies with different currencies. Received :USD and :AUD.","ref":"Money.html#max!/2-example"},{"type":"function","title":"Money.min/2","doc":"Return the minimum of two `t:Money.t/0` amounts.","ref":"Money.html#min/2"},{"type":"function","title":"Arguments - Money.min/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`. `money_1` and `money_2` should be of the same\n  currency.","ref":"Money.html#min/2-arguments"},{"type":"function","title":"Returns - Money.min/2","doc":"* `{:ok, minimum_money}` or\n\n* `{:error, reason}`","ref":"Money.html#min/2-returns"},{"type":"function","title":"Example - Money.min/2","doc":"iex> Money.min(Money.new(:USD, 200), Money.new(:USD, 300))\n    {:ok, Money.new(:USD, 200)}\n\n    iex> Money.min(Money.new(:USD, 200), Money.new(:AUD, 200))\n    {:error,\n      {ArgumentError, \"Cannot compare monies with different currencies. Received :USD and :AUD.\"}}","ref":"Money.html#min/2-example"},{"type":"function","title":"Money.min!/2","doc":"Return the minimum of two `t:Money.t/0` amounts or\nraises an exception.","ref":"Money.html#min!/2"},{"type":"function","title":"Arguments - Money.min!/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`. `money_1` and `money_2` should be of the same\n  currency.","ref":"Money.html#min!/2-arguments"},{"type":"function","title":"Returns - Money.min!/2","doc":"* `minimum_money` or\n\n* raises an exception.","ref":"Money.html#min!/2-returns"},{"type":"function","title":"Example - Money.min!/2","doc":"iex> Money.min!(Money.new(:USD, 200), Money.new(:USD, 300))\n    Money.new(:USD, 200)\n\n    iex> Money.min!(Money.new(:USD, 200), Money.new(:AUD, 200))\n    ** (ArgumentError) Cannot compare monies with different currencies. Received :USD and :AUD.","ref":"Money.html#min!/2-example"},{"type":"function","title":"Money.mult/2","doc":"Multiply a `Money` value by a number.","ref":"Money.html#mult/2"},{"type":"function","title":"Arguments - Money.mult/2","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`.\n\n* `number` is an integer, float or `t:Decimal.t/0`.\n\n> Note that multipling one `t:Money.t/0` by another is not supported.","ref":"Money.html#mult/2-arguments"},{"type":"function","title":"Returns - Money.mult/2","doc":"* `{:ok, money}` or\n\n* `{:error, reason}`","ref":"Money.html#mult/2-returns"},{"type":"function","title":"Example - Money.mult/2","doc":"iex> Money.mult(Money.new(:USD, 200), 2)\n    {:ok, Money.new(:USD, 400)}\n\n    iex> Money.mult(Money.new(:USD, 200), \"xx\")\n    {:error, {ArgumentError, \"Cannot multiply money by \\\"xx\\\"\"}}","ref":"Money.html#mult/2-example"},{"type":"function","title":"Money.mult!/2","doc":"Multiply a `Money` value by a number and raise on error.","ref":"Money.html#mult!/2"},{"type":"function","title":"Arguments - Money.mult!/2","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`.\n\n* `number` is an integer, float or `t:Decimal.t/0`.","ref":"Money.html#mult!/2-arguments"},{"type":"function","title":"Returns - Money.mult!/2","doc":"* a `t:Money.t/0` or\n\n* raises an exception","ref":"Money.html#mult!/2-returns"},{"type":"function","title":"Examples - Money.mult!/2","doc":"iex> Money.mult!(Money.new(:USD, 200), 2)\n    Money.new(:USD, \"400\")\n\n    Money.mult!(Money.new(:USD, 200), :invalid)\n    ** (ArgumentError) Cannot multiply money by :invalid","ref":"Money.html#mult!/2-examples"},{"type":"function","title":"Money.negate/1","doc":"Negate a `t:Money.t/0` value.","ref":"Money.html#negate/1"},{"type":"function","title":"Argument - Money.negate/1","doc":"* `money_1` is any valid `t:Money.t/0` type.\n\n#### Returns\n\n* `{:ok, negated_money}` with the amount negated.","ref":"Money.html#negate/1-argument"},{"type":"function","title":"Example - Money.negate/1","doc":"iex> Money.negate(Money.new(:USD, 200))\n    {:ok, Money.new(:USD, -200)}\n\n    iex> Money.negate(Money.new(:USD, -200))\n    {:ok, Money.new(:USD, 200)}","ref":"Money.html#negate/1-example"},{"type":"function","title":"Money.negate!/1","doc":"Negate a `t:Money.t/0` value or raises an\nexception.","ref":"Money.html#negate!/1"},{"type":"function","title":"Argument - Money.negate!/1","doc":"* `money_1` is any valid `t:Money.t/0` type.\n\n#### Returns\n\n* `negated_money` with the amount negated or\n\n* raises an exception.","ref":"Money.html#negate!/1-argument"},{"type":"function","title":"Example - Money.negate!/1","doc":"iex> Money.negate!(Money.new(:USD, 200))\n    Money.new(:USD, -200)\n\n    iex> Money.negate!(Money.new(:USD, -200))\n    Money.new(:USD, 200)","ref":"Money.html#negate!/1-example"},{"type":"function","title":"Money.negative?/1","doc":"Returns a boolean indicating if `t:Money.t/0`\nhas a negative value.","ref":"Money.html#negative?/1"},{"type":"function","title":"Arguments - Money.negative?/1","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`","ref":"Money.html#negative?/1-arguments"},{"type":"function","title":"Example - Money.negative?/1","doc":"iex> Money.negative?(Money.new(:USD, -1))\n    true\n\n    iex> Money.negative?(Money.new(:USD, 0))\n    false\n\n    iex> Money.negative?(Money.new(:USD, 1))\n    false","ref":"Money.html#negative?/1-example"},{"type":"function","title":"Money.new/3","doc":"Returns a %Money{} struct from a currency code and a currency amount or\nan error tuple of the form `{:error, {exception, message}}`.","ref":"Money.html#new/3"},{"type":"function","title":"Arguments - Money.new/3","doc":"* `currency_code` is an [ISO4217](https://en.wikipedia.org/wiki/ISO_4217)\n  binary or atom currency code or an\n  [ISO 24165](https://www.iso.org/standard/80601.html) token identifier or shortname.\n\n* `amount` is an integer, string or Decimal money amount.\n\n* `options` is a keyword list of options.","ref":"Money.html#new/3-arguments"},{"type":"function","title":"Options - Money.new/3","doc":"* `:locale` is any known locale.  The locale is used to normalize any\n  binary (String) amounts to a form that can be consumed by `Decimal.new/1`.\n  This consists of removing any localised grouping characters and replacing\n  the localised decimal separator with a \".\".\n  The default is `Cldr.get_locale/0`.\n\n* `:backend` is any module that includes `use Cldr` and therefore\n  is a `Cldr` backend module. The default is `Money.default_backend!/0`.\n\n* `:separators` selects which of the available symbol\n  sets should be used when attempting to parse a string into a number.\n  The default is `:standard`. Some limited locales have an alternative `:us`\n  variant that can be used. See `Cldr.Number.Symbol.number_symbols_for/3`\n  for the symbols supported for a given locale and number system.\n\n* Any other options are considered as formatting options to\n  be applied by default when calling `Money.to_string/2`.\n\nNote that the `currency_code` and `amount` arguments can be supplied in\neither order,","ref":"Money.html#new/3-options"},{"type":"function","title":"Examples - Money.new/3","doc":"iex> Money.new(:USD, 100)\n    Money.new(:USD, \"100\")\n\n    iex> Money.new(100, :USD)\n    Money.new(:USD, \"100\")\n\n    iex> Money.new(\"USD\", 100)\n    Money.new(:USD, \"100\")\n\n    iex> Money.new(\"thb\", 500)\n    Money.new(:THB, \"500\")\n\n    iex> Money.new(\"EUR\", Decimal.new(100))\n    Money.new(:EUR, \"100\")\n\n    iex> Money.new(:EUR, \"100.30\")\n    Money.new(:EUR, \"100.30\")\n\n    iex> Money.new(:EUR, \"100.30\", fractional_digits: 4)\n    Money.new(:EUR, \"100.30\", fractional_digits: 4)\n\n    iex> Money.new(:XYZZ, 100)\n    {:error, {Money.UnknownCurrencyError, \"The currency :XYZZ is invalid\"}}\n\n    iex> Money.new(\"1.000,99\", :EUR, locale: \"de\")\n    Money.new(:EUR, \"1000.99\")\n\n    iex> Money.new 123.445, :USD\n    {:error,\n     {Money.InvalidAmountError,\n      \"Float amounts are not supported in new/2 due to potenial \" <>\n      \"rounding and precision issues.  If absolutely required, \" <>\n      \"use Money.from_float/2\"}}","ref":"Money.html#new/3-examples"},{"type":"function","title":"Money.new!/3","doc":"Returns a %Money{} struct from a currency code and a currency amount. Raises an\nexception if the current code is invalid.","ref":"Money.html#new!/3"},{"type":"function","title":"Arguments - Money.new!/3","doc":"* `currency_code` is an [ISO4217](https://en.wikipedia.org/wiki/ISO_4217)\n  binary or atom currency code or an\n  [ISO 24165](https://www.iso.org/standard/80601.html) token identifier or shortname.\n\n* `amount` is an integer, float or Decimal\n\n* `options` is a keyword list of options. See `Money.new/3`.","ref":"Money.html#new!/3-arguments"},{"type":"function","title":"Examples - Money.new!/3","doc":"Money.new!(:XYZZ, 100)\n    ** (Money.UnknownCurrencyError) Currency :XYZZ is not known\n      (ex_money) lib/money.ex:177: Money.new!/2","ref":"Money.html#new!/3-examples"},{"type":"function","title":"Money.normalize/1","doc":"Normalizes the underlying decimal amount in a\ngiven `t:Money.t/0`.\n\nThis will normalize the coefficient and exponent of the\ndecimal amount in a standard way that may aid in\nnative comparison of `t:Money.t/0` items.","ref":"Money.html#normalize/1"},{"type":"function","title":"Example - Money.normalize/1","doc":"iex> x = %Money{currency: :USD, amount: %Decimal{sign: 1, coef: 42, exp: 0}}\n    Money.new(:USD, \"42\")\n    iex> y = %Money{currency: :USD, amount: %Decimal{sign: 1, coef: 4200000000, exp: -8}}\n    Money.new(:USD, \"42.00000000\")\n    iex> x == y\n    false\n    iex> y = Money.normalize(x)\n    Money.new(:USD, \"42\")\n    iex> x == y\n    true","ref":"Money.html#normalize/1-example"},{"type":"function","title":"Money.parse/2","doc":"Parse a string and return a `t:Money.t/0` or an error.\n\nThe string to be parsed is required to have a currency\ncode and an amount.  The currency code may be placed\nbefore the amount or after, but not both.\n\nParsing is strict.  Additional text surrounding the\ncurrency code and amount will cause the parse to\nfail.","ref":"Money.html#parse/2"},{"type":"function","title":"Arguments - Money.parse/2","doc":"* `string` is a string to be parsed\n\n* `options` is a keyword list of options that is\n  passed to `Money.new/3` with the exception of\n  the options listed below","ref":"Money.html#parse/2-arguments"},{"type":"function","title":"Options - Money.parse/2","doc":"* `:backend` is any module() that includes `use Cldr` and therefore\n  is a `Cldr` backend module(). The default is `Money.default_backend!()`\n\n* `:locale` is any valid locale returned by `Cldr.known_locale_names/1`\n  or a `Cldr.LanguageTag` struct returned by `Cldr.Locale.new!/2`\n  The default is ` .get_locale()`\n\n* `:only` is an `atom` or list of `atoms` representing the\n  currencies or currency types to be considered for a match.\n  The equates to a list of acceptable currencies for parsing.\n  See the notes below for currency types.\n\n* `:except` is an `atom` or list of `atoms` representing the\n  currencies or currency types to be not considered for a match.\n  This equates to a list of unacceptable currencies for parsing.\n  See the notes below for currency types.\n\n* `:fuzzy` is a float greater than `0.0` and less than or\n  equal to `1.0` which is used as input to the\n  `String.jaro_distance/2` to determine is the provided\n  currency string is *close enough* to a known currency\n  string for it to identify definitively a currency code.\n  It is recommended to use numbers greater than `0.8` in\n  order to reduce false positives.\n\n* `:default_currency` is any valid currency code or `false`\n  that will used if no currency code, symbol or description is\n  indentified in the parsed string. The default is `nil`\n  which means that the default currency associated with\n  the `:locale` option will be used. If `false` then the\n  currency assocated with the `:locale` option will not be\n  used and an error will be returned if there is no currency\n  in the string being parsed.","ref":"Money.html#parse/2-options"},{"type":"function","title":"Returns - Money.parse/2","doc":"* a `t:Money.t/0` if parsing is successful or\n\n* `{:error, {exception, reason}}` if an error is\n  detected.","ref":"Money.html#parse/2-returns"},{"type":"function","title":"Notes - Money.parse/2","doc":"The `:only` and `:except` options accept a list of\ncurrency codes and/or currency types.  The following\ntypes are recognised.\n\nIf both `:only` and `:except` are specified,\nthe `:except` entries take priority - that means\nany entries in `:except` are removed from the `:only`\nentries.\n\n  * `:all`, the default, considers all currencies.\n\n  * `:current` considers those currencies that have a `:to`\n    date of nil and which also is a known ISO4217 currency.\n\n  * `:historic` is the opposite of `:current`.\n\n  * `:tender` considers currencies that are legal tender.\n\n  * `:unannotated` considers currencies that don't have\n    \"(some string)\" in their names.  These are usually\n    financial instruments.","ref":"Money.html#parse/2-notes"},{"type":"function","title":"Examples - Money.parse/2","doc":"iex> Money.parse(\"USD 100\")\n    Money.new(:USD, \"100\")\n\n    iex> Money.parse \"USD 100,00\", locale: \"de\"\n    Money.new(:USD, \"100.00\")\n\n    iex> Money.parse(\"100 USD\")\n    Money.new(:USD, \"100\")\n\n    iex> Money.parse(\"100 eurosports\", fuzzy: 0.8)\n    Money.new(:EUR, \"100\")\n\n    iex> Money.parse(\"100\", default_currency: :EUR)\n    Money.new(:EUR, \"100\")\n\n    iex> Money.parse(\"100 eurosports\", fuzzy: 0.9)\n    {:error, {Money.UnknownCurrencyError, \"The currency \\\"eurosports\\\" is unknown or not supported\"}}\n\n    iex> Money.parse(\"100 afghan afghanis\")\n    Money.new(:AFN, \"100\")\n\n    iex> Money.parse(\"100\", default_currency: false)\n    {:error, {Money.Invalid,\n      \"A currency code, symbol or description must be specified but was not found in \\\"100\\\"\"}}\n\n    iex> Money.parse(\"USD 100 with trailing text\")\n    {:error, {Money.ParseError, \"Could not parse \\\"USD 100 with trailing text\\\".\"}}","ref":"Money.html#parse/2-examples"},{"type":"function","title":"Money.positive?/1","doc":"Returns a boolean indicating if `t:Money.t/0`\nhas a positive value.","ref":"Money.html#positive?/1"},{"type":"function","title":"Arguments - Money.positive?/1","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`","ref":"Money.html#positive?/1-arguments"},{"type":"function","title":"Example - Money.positive?/1","doc":"iex> Money.positive?(Money.new(:USD, 1))\n    true\n\n    iex> Money.positive?(Money.new(:USD, 0))\n    false\n\n    iex> Money.positive?(Money.new(:USD, -1))\n    false","ref":"Money.html#positive?/1-example"},{"type":"function","title":"Money.put_format_options/2","doc":"Add format options to a `t:Money.t/0`.","ref":"Money.html#put_format_options/2"},{"type":"function","title":"Arguments - Money.put_format_options/2","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`.\n\n* `options` is a keyword list of options. These\n  options are used when calling `Money.to_string/2`.\n  The default is `[]`.","ref":"Money.html#put_format_options/2-arguments"},{"type":"function","title":"Money.put_fraction/2","doc":"Set the fractional part of a `Money`.","ref":"Money.html#put_fraction/2"},{"type":"function","title":"Arguments - Money.put_fraction/2","doc":"* `money` is any `t:Money.t/0`.\n\n* `fraction` is an integer amount that will be set\n  as the fraction of the `money`.","ref":"Money.html#put_fraction/2-arguments"},{"type":"function","title":"Notes - Money.put_fraction/2","doc":"The fraction can only be set if it matches the number of\ndecimal digits for the currency associated with the `money`.\nTherefore, for a currency with 2 decimal digits, the\nmaximum for `fraction` is `99`.","ref":"Money.html#put_fraction/2-notes"},{"type":"function","title":"Examples - Money.put_fraction/2","doc":"iex> Money.put_fraction Money.new(:USD, \"2.49\"), 99\n    Money.new(:USD, \"2.99\")\n\n    iex> Money.put_fraction Money.new(:USD, \"2.49\"), 0\n    Money.new(:USD, \"2.0\")\n\n    iex> Money.put_fraction Money.new(:USD, \"2.49\"), 999\n    {:error,\n     {Money.InvalidAmountError, \"Rounding up to 999 is invalid for currency :USD\"}}","ref":"Money.html#put_fraction/2-examples"},{"type":"function","title":"Money.reduce/1","doc":"","ref":"Money.html#reduce/1"},{"type":"function","title":"Money.round/2","doc":"Round a `Money` value into the acceptable range for the requested currency.","ref":"Money.html#round/2"},{"type":"function","title":"Arguments - Money.round/2","doc":"* `money` is any `t:Money.t/0`.\n\n* `options` is a keyword list of options.","ref":"Money.html#round/2-arguments"},{"type":"function","title":"Options - Money.round/2","doc":"* `:rounding_mode` that defines how the number will be rounded.  See\n    `Decimal.Context`.  The default is `:half_even` which is also known\n    as \"banker's rounding\".\n\n  * `:currency_digits` which determines the rounding increment.\n    The valid options are `:cash`, `:accounting` and `:iso` or\n    an integer value representing the rounding factor.  The\n    default is `:iso`.","ref":"Money.html#round/2-options"},{"type":"function","title":"Notes - Money.round/2","doc":"There are two kinds of rounding applied:\n\n1. Round to the appropriate number of currency digits.\n\n3. Apply an appropriate rounding increment.  Most currencies\n   round to the same precision as the number of decimal digits, but some\n   such as `:CHF` round to a minimum such as `0.05` when its a cash\n   amount. The rounding increment is applied when the option\n   `:fractional_digits` is set to `:cash`.\n\n3. Digital Tokens (crypto currencies) do not have formal definitions\n   of decimal digits or rounding strategies. Therefore the `money` is\n   returned unmodified.","ref":"Money.html#round/2-notes"},{"type":"function","title":"Examples - Money.round/2","doc":"iex> Money.round Money.new(\"123.73\", :CHF), currency_digits: :cash\n    Money.new(:CHF, \"123.75\")\n\n    iex> Money.round Money.new(\"123.73\", :CHF), currency_digits: 0\n    Money.new(:CHF, \"124\")\n\n    iex> Money.round Money.new(\"123.7456\", :CHF)\n    Money.new(:CHF, \"123.75\")\n\n    iex> Money.round Money.new(\"123.7456\", :JPY)\n    Money.new(:JPY, \"124\")","ref":"Money.html#round/2-examples"},{"type":"function","title":"Money.split/3","doc":"Split a `Money` value into a number of parts maintaining the currency's\nprecision and rounding and ensuring that the parts sum to the original\namount.","ref":"Money.html#split/3"},{"type":"function","title":"Arguments - Money.split/3","doc":"* `money` is any `t:Money.t/0`.\n\n* `parts` is an integer number of parts into which the\n  `money` is split.\n\n* `options` is a keyword list of options.","ref":"Money.html#split/3-arguments"},{"type":"function","title":"Options - Money.split/3","doc":"* See `Money.round/2`; any options are passed to\n  this function.","ref":"Money.html#split/3-options"},{"type":"function","title":"Returns - Money.split/3","doc":"* `{split_amount, remainder}` where `split_amount` is the amount\n  money allocated by the split and `remainder` is the amount\n  left over that could not be allocated evenly.","ref":"Money.html#split/3-returns"},{"type":"function","title":"Notes - Money.split/3","doc":"Returns a tuple `{dividend, remainder}` as the function result\nderived as follows:\n\n1. Divide the money by the integer divisor.\n\n2. Round the result of the division to the precision of the currency\n  using `Money.round/2`. If the rounding mode results in a\n  negative remainder, the rounding is done again using rounding mode\n  `:down`.\n\n3. Return two numbers: the result of the division and any remainder\n  that could not be applied given the precision of the currency.","ref":"Money.html#split/3-notes"},{"type":"function","title":"Examples - Money.split/3","doc":"iex> Money.split(Money.new(\"123.5\", :JPY), 3)\n    {Money.new(:JPY, \"41\"), Money.new(:JPY, \"0.5\")}\n\n    iex> Money.split(Money.new(\"123.4\", :JPY), 3)\n    {Money.new(:JPY, \"41\"), Money.new(:JPY, \"0.4\")}\n\n    iex> Money.split(Money.new(\"123.7\", :USD), 9)\n    {Money.new(:USD, \"13.74\"), Money.new(:USD, \"0.04\")}\n\n    iex> Money.split(Money.new(:USD, 200), 3)\n    {Money.new(:USD, \"66.66\"), Money.new(:USD, \"0.02\")}","ref":"Money.html#split/3-examples"},{"type":"function","title":"Money.sub/2","doc":"Subtract one `t:Money.t/0` value struct from another.","ref":"Money.html#sub/2"},{"type":"function","title":"Options - Money.sub/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`.","ref":"Money.html#sub/2-options"},{"type":"function","title":"Returns - Money.sub/2","doc":"* `{:ok, money}` or\n\n* `{:error, reason}`.","ref":"Money.html#sub/2-returns"},{"type":"function","title":"Example - Money.sub/2","doc":"iex> Money.sub Money.new(:USD, 200), Money.new(:USD, 100)\n    {:ok, Money.new(:USD, 100)}","ref":"Money.html#sub/2-example"},{"type":"function","title":"Money.sub!/2","doc":"Subtract one `Money` value struct from another and raise on error.\n\nReturns either `{:ok, money}` or `{:error, reason}`.","ref":"Money.html#sub!/2"},{"type":"function","title":"Arguments - Money.sub!/2","doc":"* `money_1` and `money_2` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`.","ref":"Money.html#sub!/2-arguments"},{"type":"function","title":"Returns - Money.sub!/2","doc":"* a `t:Money.t/0` struct or\n\n* raises an exception.","ref":"Money.html#sub!/2-returns"},{"type":"function","title":"Examples - Money.sub!/2","doc":"iex> Money.sub! Money.new(:USD, 200), Money.new(:USD, 100)\n    Money.new(:USD, \"100\")\n\n    Money.sub! Money.new(:USD, 200), Money.new(:CAD, 500)\n    ** (ArgumentError) Cannot subtract monies with different currencies. Received :USD and :CAD.","ref":"Money.html#sub!/2-examples"},{"type":"function","title":"Money.sum/2","doc":"Sum a list of monies that may be in different\ncurrencies.","ref":"Money.html#sum/2"},{"type":"function","title":"Arguments - Money.sum/2","doc":"* `money_list` is a list of any valid `t:Money.t/0` types returned\n  by `Money.new/2`.\n\n* `rates` is a map of exchange rates. The default is the rates\n  returned by `Money.ExchangeRates.latest_rates/0`. If the\n  exchange rates retriever is not running, then the default is\n  `%{}`.","ref":"Money.html#sum/2-arguments"},{"type":"function","title":"Returns - Money.sum/2","doc":"* `{:ok, money}` representing the sum of the maybe\n  converted money amounts. The currency of the sum is\n  the currency of the first `Money` in the `money_list`.\n\n* `{:error, {exception, reason}}` describing an error.","ref":"Money.html#sum/2-returns"},{"type":"function","title":"Examples - Money.sum/2","doc":"iex> Money.sum([Money.new(:USD, 100), Money.new(:USD, 200), Money.new(:USD, 50)])\n    {:ok, Money.new(:USD, 350)}\n\n    iex> Money.sum([Money.new(:USD, 100), Money.new(:USD, 200), Money.new(:AUD, 50)], %{})\n    {:error,\n     {Money.ExchangeRateError, \"No exchange rate is available for currency :AUD\"}}\n\n    iex> rates = %{AUD: Decimal.new(2), USD: Decimal.new(1)}\n    iex> Money.sum [Money.new(:USD, 100), Money.new(:USD, 200), Money.new(:AUD, 50)], rates\n    {:ok, Money.from_float(:USD, 325.0)}","ref":"Money.html#sum/2-examples"},{"type":"type","title":"Money.t/0","doc":"Money is composed of an atom representation of an ISO4217 currency code and\na `Decimal` representation of an amount.","ref":"Money.html#t:t/0"},{"type":"function","title":"Money.to_currency/3","doc":"Convert `money` from one currency to another.","ref":"Money.html#to_currency/3"},{"type":"function","title":"Arguments - Money.to_currency/3","doc":"* `money` is any `t:Money.t/0` struct returned by `Cldr.Currency.new/2`.\n\n* `to_currency` is a valid currency code into which the `money` is converted.\n\n* `rates` is a `Map` of currency rates where the map key is an upcased\n  atom or string and the value is a Decimal conversion factor.  The default is the\n  latest available exchange rates returned from `Money.ExchangeRates.latest_rates/0`.","ref":"Money.html#to_currency/3-arguments"},{"type":"function","title":"Converting to a currency defined in a locale - Money.to_currency/3","doc":"To convert a `Money` to a currency defined by a locale,\n`Cldr.Currency.currency_from_locale/1` can be called with\na `t:Cldr.LanguageTag.t/0` parameter. It will return\nthe currency configured for that locale.","ref":"Money.html#to_currency/3-converting-to-a-currency-defined-in-a-locale"},{"type":"function","title":"Examples - Money.to_currency/3","doc":"iex> Money.to_currency(Money.new(:USD, 100), :AUD,\n    ...>   %{USD: Decimal.new(1), AUD: Decimal.from_float(0.7345)})\n    {:ok, Money.new(:AUD, \"73.4500\")}\n\n    iex> Money.to_currency(Money.new(\"USD\", 100), \"AUD\",\n    ...>   %{\"USD\" => Decimal.new(1), \"AUD\" => Decimal.from_float(0.7345)})\n    {:ok, Money.new(:AUD, \"73.4500\")}\n\n    iex> Money.to_currency(Money.new(:USD, 100), :AUDD,\n    ...>   %{USD: Decimal.new(1), AUD: Decimal.from_float(0.7345)})\n    {:error, {Cldr.UnknownCurrencyError, \"The currency :AUDD is invalid\"}}\n\n    iex> Money.to_currency(Money.new(:USD, 100), :CHF,\n    ...>   %{USD: Decimal.new(1), AUD: Decimal.from_float(0.7345)})\n    {:error, {Money.ExchangeRateError,\n      \"No exchange rate is available for currency :CHF\"}}","ref":"Money.html#to_currency/3-examples"},{"type":"function","title":"Money.to_currency!/3","doc":"Convert `money` from one currency to another or raises on error.","ref":"Money.html#to_currency!/3"},{"type":"function","title":"Arguments - Money.to_currency!/3","doc":"* `money` is any `t:Money.t/0` struct returned by `Cldr.Currency.new/2`.\n\n* `to_currency` is a valid currency code into which the `money` is converted.\n\n* `rates` is a `Map` of currency rates where the map key is an upcased\n  atom or string and the value is a Decimal conversion factor.  The default is the\n  latest available exchange rates returned from `Money.ExchangeRates.latest_rates/0`.","ref":"Money.html#to_currency!/3-arguments"},{"type":"function","title":"Examples - Money.to_currency!/3","doc":"iex> Money.to_currency! Money.new(:USD, 100), :AUD,\n    ...>   %{USD: Decimal.new(1), AUD: Decimal.from_float(0.7345)}\n    Money.new(:AUD, \"73.4500\")\n\n    iex> Money.to_currency! Money.new(\"USD\", 100), \"AUD\",\n    ...>   %{\"USD\" => Decimal.new(1), \"AUD\" => Decimal.from_float(0.7345)}\n    Money.new(:AUD, \"73.4500\")\n\n    => Money.to_currency! Money.new(:USD, 100), :ZZZ,\n         %{USD: Decimal.new(1), AUD: Decimal.from_float(0.7345)}\n    ** (Cldr.UnknownCurrencyError) Currency :ZZZ is not known","ref":"Money.html#to_currency!/3-examples"},{"type":"function","title":"Money.to_currency_code/1","doc":"Returns the currecny code of a `Money` type\nas an `atom`.","ref":"Money.html#to_currency_code/1"},{"type":"function","title":"Arguments - Money.to_currency_code/1","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`.","ref":"Money.html#to_currency_code/1-arguments"},{"type":"function","title":"Returns - Money.to_currency_code/1","doc":"* the currency code as an `t:atom`.","ref":"Money.html#to_currency_code/1-returns"},{"type":"function","title":"Example - Money.to_currency_code/1","doc":"iex> m = Money.new(\"USD\", 100)\n    iex> Money.to_currency_code(m)\n    :USD","ref":"Money.html#to_currency_code/1-example"},{"type":"function","title":"Money.to_decimal/1","doc":"Returns the amount part of a `Money` type as a `Decimal`.","ref":"Money.html#to_decimal/1"},{"type":"function","title":"Arguments - Money.to_decimal/1","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`.","ref":"Money.html#to_decimal/1-arguments"},{"type":"function","title":"Returns - Money.to_decimal/1","doc":"* a `Decimal.t`.","ref":"Money.html#to_decimal/1-returns"},{"type":"function","title":"Example - Money.to_decimal/1","doc":"iex> m = Money.new(\"USD\", 100)\n    iex> Money.to_decimal(m)\n    Decimal.new(100)","ref":"Money.html#to_decimal/1-example"},{"type":"function","title":"Money.to_integer_exp/2","doc":"Returns a tuple comprising the currency code, integer amount,\nexponent and remainder.\n\nSome services require submission of money items as an integer\nwith an implied exponent that is appropriate to the currency.\n\nRather than return only the integer, `Money.to_integer_exp`\nreturns the currency code, integer, exponent and remainder.\nThe remainder is included because to return an integer\nmoney with an implied exponent the `Money` has to be rounded\npotentially leaving a remainder.","ref":"Money.html#to_integer_exp/2"},{"type":"function","title":"Options - Money.to_integer_exp/2","doc":"* `money` is any `t:Money.t/0` struct returned by `Cldr.Currency.new/2`.","ref":"Money.html#to_integer_exp/2-options"},{"type":"function","title":"Notes - Money.to_integer_exp/2","doc":"* Since the returned integer is expected to have the implied fractional\n  digits the `Money` needs to be rounded which is what this function does.","ref":"Money.html#to_integer_exp/2-notes"},{"type":"function","title":"Example - Money.to_integer_exp/2","doc":"iex> m = Money.new(:USD, \"200.012356\")\n    Money.new(:USD, \"200.012356\")\n    iex> Money.to_integer_exp(m)\n    {:USD, 20001, -2, Money.new(:USD, \"0.002356\")}\n\n    iex> m = Money.new(:USD, \"200.00\")\n    Money.new(:USD, \"200.00\")\n    iex> Money.to_integer_exp(m)\n    {:USD, 20000, -2, Money.new(:USD, \"0.00\")}","ref":"Money.html#to_integer_exp/2-example"},{"type":"function","title":"Money.to_string/2","doc":"Returns a formatted string representation of a `t:Money.t/0`.\n\nFormatting is performed according to the rules defined by CLDR. See\n`Cldr.Number.to_string/2` for formatting options.  The default is to format\nas a currency which applies the appropriate rounding and fractional digits\nfor the currency.","ref":"Money.html#to_string/2"},{"type":"function","title":"Arguments - Money.to_string/2","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`.\n\n* `options` is a keyword list of options or a `t:Cldr.Number.Format.Options.t/0`\n  struct.","ref":"Money.html#to_string/2-arguments"},{"type":"function","title":"Returns - Money.to_string/2","doc":"* `{:ok, string}` or\n\n* `{:error, reason}`.","ref":"Money.html#to_string/2-returns"},{"type":"function","title":"Options - Money.to_string/2","doc":"* `:backend` is any CLDR backend module.  The default is\n  `Money.default_backend!/0`.\n\n* `currency_symbol`: Allows overriding a currency symbol. The alternatives\n  are:\n  * `:iso` the ISO currency code will be used instead of the default\n    currency symbol.\n  * `:narrow` uses the narrow symbol defined for the locale. The same\n    narrow symbol can be defined for more than one currency and therefore this\n    should be used with care. If no narrow symbol is defined, the standard\n    symbol is used.\n  * `:symbol` uses the standard symbol defined in CLDR. A symbol is unique\n    for each currency and can be safely used.\n  * \"string\" uses `string` as the currency symbol\n  * `:standard` (the default and recommended) uses the CLDR-defined symbol\n    based upon the currency format for the locale.\n  * `:none` means format the amount without any currency symbol.\n\n* `:no_fraction_if_integer` is a boolean which, if `true`, will set `:fractional_digits`\n  to `0` if the money value is an integer value.\n\n* Any other options are passed to `Cldr.Number.to_string/3`.","ref":"Money.html#to_string/2-options"},{"type":"function","title":"Examples - Money.to_string/2","doc":"iex> Money.to_string Money.new(:USD, 1234)\n    {:ok, \"$1,234.00\"}\n\n    iex> Money.to_string Money.new(:USD, 1234), no_fraction_if_integer: true\n    {:ok, \"$1,234\"}\n\n    iex> Money.to_string Money.new(:JPY, 1234)\n    {:ok, \"¥1,234\"}\n\n    iex> Money.to_string Money.new(:THB, 1234)\n    {:ok, \"THB 1,234.00\"}\n\n    iex> Money.to_string Money.new(:THB, 1234, fractional_digits: 4)\n    {:ok, \"THB 1,234.0000\"}\n\n    iex> Money.to_string Money.new(:USD, 1234), format: :long\n    {:ok, \"1,234 US dollars\"}\n\n    iex> Money.to_string(Money.new(:EUR, \"100\"), locale: \"bn\")\n    {:ok, \"১০০.০০€\"}","ref":"Money.html#to_string/2-examples"},{"type":"function","title":"Money.to_string!/2","doc":"Returns a formatted string representation of a `t:Money.t/0` or raises if\nthere is an error.\n\nFormatting is performed according to the rules defined by CLDR. See\n`Cldr.Number.to_string!/2` for formatting options.  The default is to format\nas a currency which applies the appropriate rounding and fractional digits\nfor the currency.","ref":"Money.html#to_string!/2"},{"type":"function","title":"Arguments - Money.to_string!/2","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`.\n\n* `options` is a keyword list of options or a\n  `%Cldr.Number.Format.Options{}` struct.","ref":"Money.html#to_string!/2-arguments"},{"type":"function","title":"Options - Money.to_string!/2","doc":"* `:backend` is any CLDR backend module.  The default is\n  `Money.default_backend!/0`.\n\n* Any other options are passed to `Cldr.Number.to_string/3`.","ref":"Money.html#to_string!/2-options"},{"type":"function","title":"Examples - Money.to_string!/2","doc":"iex> Money.to_string! Money.new(:USD, 1234)\n    \"$1,234.00\"\n\n    iex> Money.to_string! Money.new(:JPY, 1234)\n    \"¥1,234\"\n\n    iex> Money.to_string! Money.new(:THB, 1234)\n    \"THB 1,234.00\"\n\n    iex> Money.to_string! Money.new(:USD, 1234), format: :long\n    \"1,234 US dollars\"","ref":"Money.html#to_string!/2-examples"},{"type":"function","title":"Money.within?/3","doc":"Returns a boolean indicating if the `t:Money.t/0` is in the\nrange `minimum..maximum`.\n\n#### Arguments\n\n* `money`, `minimum` and `maximum` are any valid `t:Money.t/0` types returned\n  by `Money.new/2`. They should be of the same currency.\n\n#### Returns\n\n* `true` or `false`.\n\n#### Examples\n\n    iex> Money.within?(Money.new(:USD, 100), Money.new(:USD, 50), Money.new(:USD, 200))\n    true\n\n    iex> Money.within?(Money.new(:USD, 10), Money.new(:USD, 50), Money.new(:USD, 200))\n    false\n\n    iex> Money.within?(Money.new(:USD, 10), Money.new(:USD, 50), Money.new(:USD, 50))\n    false\n\n    iex> Money.within?(Money.new(:USD, 50), Money.new(:USD, 50), Money.new(:USD, 50))\n    true\n\n    iex> Money.within?(Money.new(:USD, 100), Money.new(:USD, 300), Money.new(:USD, 200))\n    ** (ArgumentError) Minimum must be less than maximum. Found Money.new(:USD, \"300\") and Money.new(:USD, \"200\")\n\n    iex> Money.within?(Money.new(:USD, 10), Money.new(:AUD, 300), Money.new(:EUR, 200))\n    ** (ArgumentError) Cannot compare monies with different currencies. Received :USD, :AUD and :EUR","ref":"Money.html#within?/3"},{"type":"function","title":"Money.zero/2","doc":"Return a zero amount `t:Money.t/0` in the given currency.","ref":"Money.html#zero/2"},{"type":"function","title":"Arguments - Money.zero/2","doc":"* `money_or_currency` is either a `t:Money.t/0` or\n  a currency code.\n\n* `options` is a keyword list of options passed\n  to `Money.new/3`. The default is `[]`.","ref":"Money.html#zero/2-arguments"},{"type":"function","title":"Example - Money.zero/2","doc":"iex> Money.zero(:USD)\n    Money.new(:USD, \"0\")\n\n    iex> money = Money.new(:USD, 200)\n    iex> Money.zero(money)\n    Money.new(:USD, \"0\")\n\n    iex> Money.zero :ZZZ\n    {:error, {Cldr.UnknownCurrencyError, \"The currency :ZZZ is invalid\"}}","ref":"Money.html#zero/2-example"},{"type":"function","title":"Money.zero?/1","doc":"Returns a boolean indicating if `t:Money.t/0`\nhas a zero value.","ref":"Money.html#zero?/1"},{"type":"function","title":"Arguments - Money.zero?/1","doc":"* `money` is any valid `t:Money.t/0` type returned\n  by `Money.new/2`","ref":"Money.html#zero?/1-arguments"},{"type":"function","title":"Example - Money.zero?/1","doc":"iex> Money.zero?(Money.new(:USD, 0))\n    true\n\n    iex> Money.zero?(Money.new(:USD, 1))\n    false\n\n    iex> Money.zero?(Money.new(:USD, -1))\n    false","ref":"Money.html#zero?/1-example"},{"type":"module","title":"Money.Application","doc":"","ref":"Money.Application.html"},{"type":"function","title":"Money.Application.start/2","doc":"","ref":"Money.Application.html#start/2"},{"type":"module","title":"Money.Currency","doc":"Functions to return lists of known, historic and\nlegal tender currencies.","ref":"Money.Currency.html"},{"type":"function","title":"Money.Currency.currency_for_code/1","doc":"","ref":"Money.Currency.html#currency_for_code/1"},{"type":"function","title":"Money.Currency.known_current_currencies/0","doc":"Returns the list of currently active ISO 4217 currency codes.","ref":"Money.Currency.html#known_current_currencies/0"},{"type":"function","title":"Example: - Money.Currency.known_current_currencies/0","doc":"iex> Money.Currency.known_current_currencies()\n    [:AED, :AFN, :ALL, :AMD, :ANG, :AOA, :ARS, :AUD, :AWG, :AZN, :BAM, :BBD, :BDT,\n     :BGN, :BHD, :BIF, :BMD, :BND, :BOB, :BOV, :BRL, :BSD, :BTN, :BWP, :BYN, :BZD,\n     :CAD, :CDF, :CHE, :CHF, :CHW, :CLF, :CLP, :CNY, :COP, :COU, :CRC, :CUC, :CUP,\n     :CVE, :CZK, :DJF, :DKK, :DOP, :DZD, :EGP, :ERN, :ETB, :EUR, :FJD, :FKP, :GBP,\n     :GEL, :GHS, :GIP, :GMD, :GNF, :GTQ, :GYD, :HKD, :HNL, :HRK, :HTG, :HUF, :IDR,\n     :ILS, :INR, :IQD, :IRR, :ISK, :JMD, :JOD, :JPY, :KES, :KGS, :KHR, :KMF, :KPW,\n     :KRW, :KWD, :KYD, :KZT, :LAK, :LBP, :LKR, :LRD, :LSL, :LYD, :MAD, :MDL, :MGA,\n     :MKD, :MMK, :MNT, :MOP, :MRU, :MUR, :MVR, :MWK, :MXN, :MXV, :MYR, :MZN, :NAD,\n     :NGN, :NIO, :NOK, :NPR, :NZD, :OMR, :PAB, :PEN, :PGK, :PHP, :PKR, :PLN, :PYG,\n     :QAR, :RON, :RSD, :RUB, :RWF, :SAR, :SBD, :SCR, :SDG, :SEK, :SGD, :SHP, :SLL,\n     :SOS, :SRD, :SSP, :STN, :SVC, :SYP, :SZL, :THB, :TJS, :TMT, :TND, :TOP, :TRY,\n     :TTD, :TWD, :TZS, :UAH, :UGX, :USD, :USN, :UYI, :UYU, :UYW, :UZS, :VES, :VND,\n     :VUV, :WST, :XAF, :XAG, :XAU, :XBA, :XBB, :XBC, :XBD, :XCD, :XDR, :XOF, :XPD,\n     :XPF, :XPT, :XSU, :XTS, :XUA, :XXX, :YER, :ZAR, :ZMW, :ZWL]","ref":"Money.Currency.html#known_current_currencies/0-example"},{"type":"function","title":"Money.Currency.known_historic_currencies/0","doc":"Returns the list of historic ISO 4217 currency codes.","ref":"Money.Currency.html#known_historic_currencies/0"},{"type":"function","title":"Example: - Money.Currency.known_historic_currencies/0","doc":"iex> Money.Currency.known_historic_currencies()\n    [:ADP, :AFA, :ALK, :AOK, :AON, :AOR, :ARA, :ARL, :ARM, :ARP, :ATS, :AZM, :BAD,\n     :BAN, :BEC, :BEF, :BEL, :BGL, :BGM, :BGO, :BOL, :BOP, :BRB, :BRC, :BRE, :BRN,\n     :BRR, :BRZ, :BUK, :BYB, :BYR, :CLE, :CNH, :CNX, :CSD, :CSK, :CYP, :DDM, :DEM,\n     :ECS, :ECV, :EEK, :ESA, :ESB, :ESP, :FIM, :FRF, :GEK, :GHC, :GNS, :GQE, :GRD,\n     :GWE, :GWP, :HRD, :IEP, :ILP, :ILR, :ISJ, :ITL, :KRH, :KRO, :LTL, :LTT, :LUC,\n     :LUF, :LUL, :LVL, :LVR, :MAF, :MCF, :MDC, :MGF, :MKN, :MLF, :MRO, :MTL, :MTP,\n     :MVP, :MXP, :MZE, :MZM, :NIC, :NLG, :PEI, :PES, :PLZ, :PTE, :RHD, :ROL, :RUR,\n     :SDD, :SDP, :SIT, :SKK, :SLE, :SRG, :STD, :SUR, :TJR, :TMM, :TPE, :TRL, :UAK,\n     :UGS, :USS, :UYP, :VEB, :VED, :VEF, :VNN, :XCG, :XEU, :XFO, :XFU, :XRE, :YDD,\n     :YUD, :YUM, :YUN, :YUR, :ZAL, :ZMK, :ZRN, :ZRZ, :ZWD, :ZWG, :ZWR]","ref":"Money.Currency.html#known_historic_currencies/0-example"},{"type":"function","title":"Money.Currency.known_tender_currencies/0","doc":"Returns the list of legal tender ISO 4217 currency codes.","ref":"Money.Currency.html#known_tender_currencies/0"},{"type":"function","title":"Example: - Money.Currency.known_tender_currencies/0","doc":"iex> Money.Currency.known_tender_currencies()\n    [:ADP, :AED, :AFA, :AFN, :ALK, :ALL, :AMD, :ANG, :AOA, :AOK, :AON, :AOR, :ARA,\n     :ARL, :ARM, :ARP, :ARS, :ATS, :AUD, :AWG, :AZM, :AZN, :BAD, :BAM, :BAN, :BBD,\n     :BDT, :BEC, :BEF, :BEL, :BGL, :BGM, :BGN, :BGO, :BHD, :BIF, :BMD, :BND, :BOB,\n     :BOL, :BOP, :BOV, :BRB, :BRC, :BRE, :BRL, :BRN, :BRR, :BRZ, :BSD, :BTN, :BUK,\n     :BWP, :BYB, :BYN, :BYR, :BZD, :CAD, :CDF, :CHE, :CHF, :CHW, :CLE, :CLF, :CLP,\n     :CNH, :CNX, :CNY, :COP, :COU, :CRC, :CSD, :CSK, :CUC, :CUP, :CVE, :CYP, :CZK,\n     :DDM, :DEM, :DJF, :DKK, :DOP, :DZD, :ECS, :ECV, :EEK, :EGP, :ERN, :ESA, :ESB,\n     :ESP, :ETB, :EUR, :FIM, :FJD, :FKP, :FRF, :GBP, :GEK, :GEL, :GHC, :GHS, :GIP,\n     :GMD, :GNF, :GNS, :GQE, :GRD, :GTQ, :GWE, :GWP, :GYD, :HKD, :HNL, :HRD, :HRK,\n     :HTG, :HUF, :IDR, :IEP, :ILP, :ILR, :ILS, :INR, :IQD, :IRR, :ISJ, :ISK, :ITL,\n     :JMD, :JOD, :JPY, :KES, :KGS, :KHR, :KMF, :KPW, :KRH, :KRO, :KRW, :KWD, :KYD,\n     :KZT, :LAK, :LBP, :LKR, :LRD, :LSL, :LTL, :LTT, :LUC, :LUF, :LUL, :LVL, :LVR,\n     :LYD, :MAD, :MAF, :MCF, :MDC, :MDL, :MGA, :MGF, :MKD, :MKN, :MLF, :MMK, :MNT,\n     :MOP, :MRO, :MRU, :MTL, :MTP, :MUR, :MVP, :MVR, :MWK, :MXN, :MXP, :MXV, :MYR,\n     :MZE, :MZM, :MZN, :NAD, :NGN, :NIC, :NIO, :NLG, :NOK, :NPR, :NZD, :OMR, :PAB,\n     :PEI, :PEN, :PES, :PGK, :PHP, :PKR, :PLN, :PLZ, :PTE, :PYG, :QAR, :RHD, :ROL,\n     :RON, :RSD, :RUB, :RUR, :RWF, :SAR, :SBD, :SCR, :SDD, :SDG, :SDP, :SEK, :SGD,\n     :SHP, :SIT, :SKK, :SLE, :SLL, :SOS, :SRD, :SRG, :SSP, :STD, :STN, :SUR, :SVC,\n     :SYP, :SZL, :THB, :TJR, :TJS, :TMM, :TMT, :TND, :TOP, :TPE, :TRL, :TRY, :TTD,\n     :TWD, :TZS, :UAH, :UAK, :UGS, :UGX, :USD, :USN, :USS, :UYI, :UYP, :UYU, :UYW,\n     :UZS, :VEB, :VED, :VEF, :VES, :VND, :VNN, :VUV, :WST, :XAF, :XAG, :XAU, :XBA,\n     :XBB, :XBC, :XBD, :XCD, :XCG, :XDR, :XEU, :XFO, :XFU, :XOF, :XPD, :XPF, :XPT,\n     :XRE, :XSU, :XTS, :XUA, :XXX, :YDD, :YER, :YUD, :YUM, :YUN, :YUR, :ZAL, :ZAR,\n     :ZMK, :ZMW, :ZRN, :ZRZ, :ZWD, :ZWG, :ZWL, :ZWR]","ref":"Money.Currency.html#known_tender_currencies/0-example"},{"type":"module","title":"Money.Financial","doc":"A set of financial functions, primarily related to discounted cash flows.\n\nSome of the algorithms are from [finance formulas](http://www.financeformulas.net)","ref":"Money.Financial.html"},{"type":"function","title":"Money.Financial.future_value/2","doc":"Calculates the future value for a list of cash flows and an interest rate.\n\n* `flows` is a list of tuples representing a cash flow.  Each flow is\nrepresented as a tuple of the form `{period, %Money{}}`\n\n* `interest_rate` is a float representation of an interest rate.  For\nexample, 12% would be represented as `0.12`","ref":"Money.Financial.html#future_value/2"},{"type":"function","title":"Example - Money.Financial.future_value/2","doc":"iex> Money.Financial.future_value([{4, Money.new(:USD, 10000)}, {5, Money.new(:USD, 10000)}, {6, Money.new(:USD, 10000)}], 0.13)\n    Money.new(:USD, \"34068.99999999999999999999999\")\n\n    iex> Money.Financial.future_value [{0, Money.new(:USD, 5000)},{1, Money.new(:USD, 2000)}], 0.12\n    Money.new(:USD, \"7600.000000000000000000000000\")","ref":"Money.Financial.html#future_value/2-example"},{"type":"function","title":"Money.Financial.future_value/3","doc":"Calculates the future value for a present value, an interest rate\nand a number of periods.\n\n* `present_value` is a %Money{} representation of the present value\n\n* `interest_rate` is a float representation of an interest rate.  For\nexample, 12% would be represented as `0.12`\n\n* `periods` in an integer number of periods","ref":"Money.Financial.html#future_value/3"},{"type":"function","title":"Examples - Money.Financial.future_value/3","doc":"iex> Money.Financial.future_value Money.new(:USD, 10000), 0.08, 1\n    Money.new(:USD, \"10800.00\")\n\n    iex> Money.Financial.future_value Money.new(:USD, 10000), 0.04, 2\n    Money.new(:USD, \"10816.0000\")\n\n    iex> Money.Financial.future_value Money.new(:USD, 10000), 0.02, 4\n    Money.new(:USD, \"10824.32160000\")","ref":"Money.Financial.html#future_value/3-examples"},{"type":"function","title":"Money.Financial.interest_rate/3","doc":"Calculates the effective interest rate for a given present value,\na future value and a number of periods.\n\n* `present_value` is a %Money{} representation of the present value\n\n* `future_value` is a %Money{} representation of the future value\n\n* `periods` is an integer number of a period","ref":"Money.Financial.html#interest_rate/3"},{"type":"function","title":"Examples - Money.Financial.interest_rate/3","doc":"iex> Money.Financial.interest_rate Money.new(:USD, 10000), Money.new(:USD, 10816), 2\n    Decimal.new(\"0.04\")\n\n    iex> Money.Financial.interest_rate Money.new(:USD, 10000), Money.new(:USD, \"10824.3216\"), 4\n    Decimal.new(\"0.02\")","ref":"Money.Financial.html#interest_rate/3-examples"},{"type":"function","title":"Money.Financial.internal_rate_of_return/1","doc":"Calculates the interal rate of return for a given list of cash flows.\n\n* `flows` is a list of tuples representing a cash flow.  Each flow is\n  represented as a tuple of the form `{period, %Money{}}`","ref":"Money.Financial.html#internal_rate_of_return/1"},{"type":"function","title":"Money.Financial.net_present_value/2","doc":"Calculates the net present value of an initial investment, a list of\ncash flows and an interest rate.\n\n* `flows` is a list of tuples representing a cash flow.  Each flow is\nrepresented as a tuple of the form `{period, %Money{}}`\n\n* `interest_rate` is a float representation of an interest rate.  For\nexample, 12% would be represented as `0.12`\n\n* `investment` is a %Money{} struct representing the initial investment","ref":"Money.Financial.html#net_present_value/2"},{"type":"function","title":"Example - Money.Financial.net_present_value/2","doc":"iex> flows = [{0, Money.new(:USD, 5000)},{1, Money.new(:USD, 2000)},{2, Money.new(:USD, 500)},{3, Money.new(:USD,10_000)}]\n    iex> Money.Financial.net_present_value flows, 0.08, Money.new(:USD, 100)\n    Money.new(:USD, \"15118.84367220444038002337042\")\n    iex> Money.Financial.net_present_value flows, 0.08\n    Money.new(:USD, \"15218.84367220444038002337042\")","ref":"Money.Financial.html#net_present_value/2-example"},{"type":"function","title":"Money.Financial.net_present_value/3","doc":"Calculates the net present value of an initial investment, a recurring\npayment, an interest rate and a number of periods\n\n* `investment` is a %Money{} struct representing the initial investment\n\n* `future_value` is a %Money{} representation of the future value\n\n* `interest_rate` is a float representation of an interest rate.  For\n  example, 12% would be represented as `0.12`\n\n* `periods` in an integer number of a period","ref":"Money.Financial.html#net_present_value/3"},{"type":"function","title":"Example - Money.Financial.net_present_value/3","doc":"iex> Money.Financial.net_present_value Money.new(:USD, 10000), 0.13, 2\n    Money.new(:USD, \"7831.466833737959119743127888\")\n\n    iex> Money.Financial.net_present_value Money.new(:USD, 10000), 0.13, 2, Money.new(:USD, 100)\n    Money.new(:USD, \"7731.466833737959119743127888\")","ref":"Money.Financial.html#net_present_value/3-example"},{"type":"function","title":"Money.Financial.net_present_value/4","doc":"","ref":"Money.Financial.html#net_present_value/4"},{"type":"function","title":"Money.Financial.payment/3","doc":"Calculates the payment for a given loan or annuity given a\npresent value, an interest rate and a number of periods.\n\n* `present_value` is a %Money{} representation of the present value\n\n* `interest_rate` is a float representation of an interest rate.  For\nexample, 12% would be represented as `0.12`\n\n* `periods` is an integer number of periods","ref":"Money.Financial.html#payment/3"},{"type":"function","title":"Example - Money.Financial.payment/3","doc":"iex> Money.Financial.payment Money.new(:USD, 100), 0.12, 20\n    Money.new(:USD, \"13.38787800396606622792492299\")","ref":"Money.Financial.html#payment/3-example"},{"type":"function","title":"Money.Financial.periods/3","doc":"Calculates the number of periods between a present value and\na future value with a given interest rate.\n\n* `present_value` is a %Money{} representation of the present value\n\n* `future_value` is a %Money{} representation of the future value\n\n* `interest_rate` is a float representation of an interest rate.  For\nexample, 12% would be represented as `0.12`","ref":"Money.Financial.html#periods/3"},{"type":"function","title":"Example - Money.Financial.periods/3","doc":"iex> Money.Financial.periods Money.new(:USD, 1500), Money.new(:USD, 2000), 0.005\n    Decimal.new(\"57.68013595323872502502238648\")","ref":"Money.Financial.html#periods/3-example"},{"type":"function","title":"Money.Financial.present_value/2","doc":"Calculates the present value for a list of cash flows and an interest rate.\n\n* `flows` is a list of tuples representing a cash flow.  Each flow is\nrepresented as a tuple of the form `{period, %Money{}}`\n\n* `interest_rate` is a float representation of an interest rate.  For\nexample, 12% would be represented as `0.12`","ref":"Money.Financial.html#present_value/2"},{"type":"function","title":"Example - Money.Financial.present_value/2","doc":"iex> Money.Financial.present_value([{4, Money.new(:USD, 10000)}, {5, Money.new(:USD, 10000)}, {6, Money.new(:USD, 10000)}], 0.13)\n    Money.new(:USD, \"16363.97191111964880256655144\")\n\n    iex> Money.Financial.present_value [{0, Money.new(:USD, -1000)},{1, Money.new(:USD, -4000)}], 0.1\n    Money.new(:USD, \"-4636.363636363636363636363636\")","ref":"Money.Financial.html#present_value/2-example"},{"type":"function","title":"Money.Financial.present_value/3","doc":"Calculates the present value for future value, an interest rate\nand a number of periods\n\n* `future_value` is a %Money{} representation of the future value\n\n* `interest_rate` is a float representation of an interest rate.  For\nexample, 12% would be represented as `0.12`\n\n* `periods` in an integer number of periods","ref":"Money.Financial.html#present_value/3"},{"type":"function","title":"Examples - Money.Financial.present_value/3","doc":"iex> Money.Financial.present_value Money.new(:USD, 100), 0.08, 2\n    Money.new(:USD, \"85.73388203017832647462277092\")\n\n    iex> Money.Financial.present_value Money.new(:USD, 1000), 0.10, 20\n    Money.new(:USD, \"148.6436280241436864020760472\")","ref":"Money.Financial.html#present_value/3-examples"},{"type":"module","title":"Money.Sigil","doc":"","ref":"Money.Sigil.html"},{"type":"function","title":"Money.Sigil.sigil_M/2","doc":"Implements the sigil `~M` for Money\n\nThe lower case `~m` variant does not exist as interpolation and excape\ncharacters are not useful for Money sigils.","ref":"Money.Sigil.html#sigil_M/2"},{"type":"function","title":"Example - Money.Sigil.sigil_M/2","doc":"iex> import Money.Sigil\n    iex> ~M[1000]usd\n    Money.new(:USD, \"1000\")\n    iex> ~M[1000.34]usd\n    Money.new(:USD, \"1000.34\")","ref":"Money.Sigil.html#sigil_M/2-example"},{"type":"function","title":"Money.Sigil.validate_currency!/1","doc":"","ref":"Money.Sigil.html#validate_currency!/1"},{"type":"behaviour","title":"Money.ExchangeRates","doc":"Implements a behaviour and functions to retrieve exchange rates\nfrom an exchange rate service.\n\nConfiguration for the exchange rate service is defined\nin a `Money.ExchangeRates.Config` struct.  A default\nconfiguration is returned by `Money.ExchangeRates.default_config/0`.\n\nThe default configuration is:\n\n    config :ex_money,\n      auto_start_exchange_rate_service: false,\n      exchange_rates_retrieve_every: 300_000,\n      api_module: Money.ExchangeRates.OpenExchangeRates,\n      callback_module: Money.ExchangeRates.Callback,\n      preload_historic_rates: nil\n      log_failure: :warn,\n      log_info: :info,\n      log_success: nil\n\nThese keys are are defined as follows:\n\n* `:auto_start_exchange_rate_service` is a boolean that determines whether to\n  automatically start the exchange rate retrieval service.\n  The default it false.\n\n* `:exchange_rates_retrieve_every` defines how often the exchange\n  rates are retrieved in milliseconds. The default is 5 minutes\n  (300,000 milliseconds).\n\n* `:api_module` identifies the module that does the retrieval of\n  exchange rates. This is any module that implements the\n  `Money.ExchangeRates` behaviour. The default is\n  `Money.ExchangeRates.OpenExchangeRates`.\n\n* `:callback_module` defines a module that follows the\n  Money.ExchangeRates.Callback behaviour whereby the function\n  `rates_retrieved/2` is invoked after every successful retrieval\n  of exchange rates. The default is `Money.ExchangeRates.Callback`.\n\n* `:preload_historic_rates` defines a date or a date range,\n  that will be requested when the exchange rate service starts up.\n  The date or date range should be specified as either a `Date.t`\n  or a `Date.Range.t` or a tuple of `{Date.t, Date.t}` representing\n  the `from` and `to` dates for the rates to be retrieved. The\n  default is `nil` meaning no historic rates are preloaded.\n\n* `:log_failure` defines the log level at which api retrieval\n  errors are logged. The default is `:warning`.\n\n* `:log_success` defines the log level at which successful api\n  retrieval notifications are logged. The default is `nil` which\n  means no logging.\n\n* `:log_info` defines the log level at which service startup messages\n  are logged. The default is `:info`.\n\n* `:retriever_options` is available for exchange rate retriever\n  module developers as a place to add retriever-specific configuration\n  information.  This information should be added in the `init/1`\n  callback in the retriever module.  See `Money.ExchangeRates.OpenExchangeRates.init/1`\n  for an example.\n\nKeys can also be configured to retrieve values from environment\nvariables. This lookup is done at runtime to facilitate deployment\nstrategies. If the value of a configuration key is\n`{:system, \"some_string\"}` then \"some_string\" is interpreted as\nan environment variable name which is passed to System.get_env/2.\n\nAn example configuration might be:\n\n    config :ex_money,\n      auto_start_exchange_rate_service: {:system, \"RATE_SERVICE\"},\n      exchange_rates_retrieve_every: {:system, \"RETRIEVE_EVERY\"},","ref":"Money.ExchangeRates.html"},{"type":"behaviour","title":"Open Exchange Rates - Money.ExchangeRates","doc":"If you plan to use the provided Open Exchange Rates module\nto retrieve exchange rates then you should also provide the additional\nconfiguration key for `app_id`:\n\n    config :ex_money,\n      open_exchange_rates_app_id: \"your_app_id\"\n\nor configure it via environment variable:\n\n    config :ex_money,\n      open_exchange_rates_app_id: {:system, \"OPEN_EXCHANGE_RATES_APP_ID\"}\n\nThe default exchange rate retrieval module is provided in\n`Money.ExchangeRates.OpenExchangeRates` which can be used\nas a example to implement your own retrieval module for\nother services.","ref":"Money.ExchangeRates.html#module-open-exchange-rates"},{"type":"behaviour","title":"Managing the configuration at runtime - Money.ExchangeRates","doc":"During exchange rate service startup, the function `init/1` is called\non the configuration exchange rate retrieval module.  This module is\nexpected to return an updated configuration allowing a developer to\ncustomise how the configuration is to be managed.  See the implementation\nat `Money.ExchangeRates.OpenExchangeRates.init/1` for an example.","ref":"Money.ExchangeRates.html#module-managing-the-configuration-at-runtime"},{"type":"function","title":"Money.ExchangeRates.config/0","doc":"Returns the configuration for `ex_money` including the\nconfiguration merged from the configured exchange rates\nretriever module.","ref":"Money.ExchangeRates.html#config/0"},{"type":"callback","title":"Money.ExchangeRates.decode_rates/1","doc":"Decode the body returned from the API request and\nreturn a map of rates.  THe map of rates must have\nan upcased atom key representing an ISO 4217 currency\ncode and the value must be a Decimal number.","ref":"Money.ExchangeRates.html#c:decode_rates/1"},{"type":"function","title":"Money.ExchangeRates.default_config/0","doc":"Returns the default configuration for the exchange rates retriever.","ref":"Money.ExchangeRates.html#default_config/0"},{"type":"callback","title":"Money.ExchangeRates.get_historic_rates/2","doc":"Invoked to return the historic exchange rates from the configured\nexchange rate retrieval service.\n\n* `config` is an `%Money.ExchangeRataes.Config{}` struct\n\nReturns `{:ok, map_of_historic_rates}` or `{:error, reason}`","ref":"Money.ExchangeRates.html#c:get_historic_rates/2"},{"type":"callback","title":"Money.ExchangeRates.get_latest_rates/1","doc":"Invoked to return the latest exchange rates from the configured\nexchange rate retrieval service.\n\n* `config` is an `%Money.ExchangeRataes.Config{}` struct\n\nReturns `{:ok, map_of_rates}` or `{:error, reason}`","ref":"Money.ExchangeRates.html#c:get_latest_rates/1"},{"type":"function","title":"Money.ExchangeRates.historic_rates/1","doc":"Return historic exchange rates.\n\n* `date` is a date returned by `Date.new/3` or any struct with the\n  elements `:year`, `:month` and `:day`.\n\nReturns:\n\n* `{:ok, rates}` if exchange rates are successfully retrieved.  `rates` is a map of\n  exchange rates.\n\n* `{:error, reason}` if no exchange rates can be returned.\n\n**Note;** all dates are expected to be in the Calendar.ISO calendar\n\nThis function looks up the historic exchange rates in a an ETS table\ncalled `:exchange_rates`.  The actual retrieval of rates is requested\nthrough `Money.ExchangeRates.Retriever.historic_rates/1`.","ref":"Money.ExchangeRates.html#historic_rates/1"},{"type":"callback","title":"Money.ExchangeRates.init/1","doc":"Given the default configuration, returns an updated configuration at runtime\nduring exchange rates service startup.\n\nThis callback is optional.  If the callback is not defined, the default\nconfiguration returned by `Money.ExchangeRates.default_config/0` is used.\n\n* `config` is the configuration returned by `Money.ExchangeRates.default_config/0`\n\nThe callback is expected to return a `%Money.ExchangeRates.Config.t()` struct\nwhich may have been updated.  The configuration key `:retriever_options` is\navailable for any service-specific configuration.","ref":"Money.ExchangeRates.html#c:init/1"},{"type":"function","title":"Money.ExchangeRates.last_updated/0","doc":"Return the timestamp of the last successful retrieval of exchange rates or\n`{:error, reason}` if no timestamp is known.","ref":"Money.ExchangeRates.html#last_updated/0"},{"type":"function","title":"Example - Money.ExchangeRates.last_updated/0","doc":"Money.ExchangeRates.last_updated\n    #> {:ok,\n     %DateTime{calendar: Calendar.ISO, day: 20, hour: 12, microsecond: {731942, 6},\n      minute: 36, month: 11, second: 6, std_offset: 0, time_zone: \"Etc/UTC\",\n      utc_offset: 0, year: 2016, zone_abbr: \"UTC\"}}","ref":"Money.ExchangeRates.html#last_updated/0-example"},{"type":"function","title":"Money.ExchangeRates.latest_rates/0","doc":"Return the latest exchange rates.\n\nReturns:\n\n* `{:ok, rates}` if exchange rates are successfully retrieved.  `rates` is a map of\n  exchange rates.\n\n* `{:error, reason}` if no exchange rates can be returned.\n\nThis function looks up the latest exchange rates in a an ETS table\ncalled `:exchange_rates`.  The actual retrieval of rates is requested\nthrough `Money.ExchangeRates.Retriever.latest_rates/0`.","ref":"Money.ExchangeRates.html#latest_rates/0"},{"type":"function","title":"Money.ExchangeRates.latest_rates_available?/0","doc":"Returns `true` if the latest exchange rates are available\nand false otherwise.","ref":"Money.ExchangeRates.html#latest_rates_available?/0"},{"type":"type","title":"Money.ExchangeRates.t/0","doc":"","ref":"Money.ExchangeRates.html#t:t/0"},{"type":"behaviour","title":"Money.ExchangeRates.Cache","doc":"Defines a cache behaviour and default inplementation\nof a cache for exchange rates","ref":"Money.ExchangeRates.Cache.html"},{"type":"function","title":"Money.ExchangeRates.Cache.cache/0","doc":"","ref":"Money.ExchangeRates.Cache.html#cache/0"},{"type":"function","title":"Money.ExchangeRates.Cache.historic_rates/1","doc":"","ref":"Money.ExchangeRates.Cache.html#historic_rates/1"},{"type":"callback","title":"Money.ExchangeRates.Cache.historic_rates/1","doc":"Returns the exchange rates for a given\ndate.","ref":"Money.ExchangeRates.Cache.html#c:historic_rates/1"},{"type":"callback","title":"Money.ExchangeRates.Cache.init/0","doc":"Initialize the cache when the exchange rates\nretriever is started","ref":"Money.ExchangeRates.Cache.html#c:init/0"},{"type":"function","title":"Money.ExchangeRates.Cache.latest_rates/0","doc":"","ref":"Money.ExchangeRates.Cache.html#latest_rates/0"},{"type":"callback","title":"Money.ExchangeRates.Cache.latest_rates/0","doc":"Retrieve the latest exchange rates from the\ncache.","ref":"Money.ExchangeRates.Cache.html#c:latest_rates/0"},{"type":"callback","title":"Money.ExchangeRates.Cache.store_historic_rates/2","doc":"Store the historic exchange rates for a given\ndate in the cache.","ref":"Money.ExchangeRates.Cache.html#c:store_historic_rates/2"},{"type":"callback","title":"Money.ExchangeRates.Cache.store_latest_rates/2","doc":"Store the latest exchange rates in the cache.","ref":"Money.ExchangeRates.Cache.html#c:store_latest_rates/2"},{"type":"callback","title":"Money.ExchangeRates.Cache.terminate/0","doc":"Terminate the cache when the retriver process\nstops normally","ref":"Money.ExchangeRates.Cache.html#c:terminate/0"},{"type":"module","title":"Money.ExchangeRates.Cache.Dets","doc":"Money.ExchangeRates.Cache implementation for\n:dets","ref":"Money.ExchangeRates.Cache.Dets.html"},{"type":"function","title":"Money.ExchangeRates.Cache.Dets.get/1","doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#get/1"},{"type":"function","title":"Money.ExchangeRates.Cache.Dets.historic_rates/1","doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#historic_rates/1"},{"type":"function","title":"Money.ExchangeRates.Cache.Dets.init/0","doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#init/0"},{"type":"function","title":"Money.ExchangeRates.Cache.Dets.last_updated/0","doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#last_updated/0"},{"type":"function","title":"Money.ExchangeRates.Cache.Dets.latest_rates/0","doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#latest_rates/0"},{"type":"function","title":"Money.ExchangeRates.Cache.Dets.put/2","doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#put/2"},{"type":"function","title":"Money.ExchangeRates.Cache.Dets.store_historic_rates/2","doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#store_historic_rates/2"},{"type":"function","title":"Money.ExchangeRates.Cache.Dets.store_latest_rates/2","doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#store_latest_rates/2"},{"type":"function","title":"Money.ExchangeRates.Cache.Dets.terminate/0","doc":"","ref":"Money.ExchangeRates.Cache.Dets.html#terminate/0"},{"type":"module","title":"Money.ExchangeRates.Cache.Ets","doc":"Money.ExchangeRates.Cache implementation for\n:ets and :dets","ref":"Money.ExchangeRates.Cache.Ets.html"},{"type":"function","title":"Money.ExchangeRates.Cache.Ets.get/1","doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#get/1"},{"type":"function","title":"Money.ExchangeRates.Cache.Ets.historic_rates/1","doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#historic_rates/1"},{"type":"function","title":"Money.ExchangeRates.Cache.Ets.init/0","doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#init/0"},{"type":"function","title":"Money.ExchangeRates.Cache.Ets.last_updated/0","doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#last_updated/0"},{"type":"function","title":"Money.ExchangeRates.Cache.Ets.latest_rates/0","doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#latest_rates/0"},{"type":"function","title":"Money.ExchangeRates.Cache.Ets.put/2","doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#put/2"},{"type":"function","title":"Money.ExchangeRates.Cache.Ets.store_historic_rates/2","doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#store_historic_rates/2"},{"type":"function","title":"Money.ExchangeRates.Cache.Ets.store_latest_rates/2","doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#store_latest_rates/2"},{"type":"function","title":"Money.ExchangeRates.Cache.Ets.terminate/0","doc":"","ref":"Money.ExchangeRates.Cache.Ets.html#terminate/0"},{"type":"module","title":"Money.ExchangeRates.Cache.EtsDets","doc":"","ref":"Money.ExchangeRates.Cache.EtsDets.html"},{"type":"macro","title":"Money.ExchangeRates.Cache.EtsDets.define_common_functions/0","doc":"","ref":"Money.ExchangeRates.Cache.EtsDets.html#define_common_functions/0"},{"type":"behaviour","title":"Money.ExchangeRates.Callback","doc":"Default exchange rates retrieval callback module.\n\nWhen exchange rates are successfully retrieved, the function\n`latest_rates_retrieved/2` or `historic_rates_retrieved/2` is\ncalled to perform any desired serialization or proocessing.","ref":"Money.ExchangeRates.Callback.html"},{"type":"function","title":"Money.ExchangeRates.Callback.historic_rates_retrieved/2","doc":"Callback function invoked when historic exchange rates are retrieved.","ref":"Money.ExchangeRates.Callback.html#historic_rates_retrieved/2"},{"type":"callback","title":"Money.ExchangeRates.Callback.historic_rates_retrieved/2","doc":"Defines the behaviour to retrieve historic exchange rates from an external\ndata source.","ref":"Money.ExchangeRates.Callback.html#c:historic_rates_retrieved/2"},{"type":"function","title":"Money.ExchangeRates.Callback.latest_rates_retrieved/2","doc":"Callback function invoked when the latest exchange rates are retrieved.","ref":"Money.ExchangeRates.Callback.html#latest_rates_retrieved/2"},{"type":"callback","title":"Money.ExchangeRates.Callback.latest_rates_retrieved/2","doc":"Defines the behaviour to retrieve the latest exchange rates from an external\ndata source.","ref":"Money.ExchangeRates.Callback.html#c:latest_rates_retrieved/2"},{"type":"module","title":"Money.ExchangeRates.Config","doc":"","ref":"Money.ExchangeRates.Config.html"},{"type":"type","title":"Money.ExchangeRates.Config.t/0","doc":"","ref":"Money.ExchangeRates.Config.html#t:t/0"},{"type":"module","title":"Money.ExchangeRates.OpenExchangeRates","doc":"Implements the `Money.ExchangeRates` for the Open Exchange\nRates service.","ref":"Money.ExchangeRates.OpenExchangeRates.html"},{"type":"module","title":"Required configuration: - Money.ExchangeRates.OpenExchangeRates","doc":"The configuration key `:open_exchange_rates_app_id` should be\nset to your `app_id`.  for example:\n\n    config :ex_money,\n      open_exchange_rates_app_id: \"your_app_id\"\n\nor configure it via environment variable:\n\n    config :ex_money,\n      open_exchange_rates_app_id: {:system, \"OPEN_EXCHANGE_RATES_APP_ID\"}\n\nIt is also possible to configure an alternative base url for this\nservice in case it changes in the future. For example:\n\n    config :ex_money,\n      open_exchange_rates_app_id: \"your_app_id\"\n      open_exchange_rates_url: \"https://openexchangerates.org/alternative_api\"","ref":"Money.ExchangeRates.OpenExchangeRates.html#module-required-configuration"},{"type":"function","title":"Money.ExchangeRates.OpenExchangeRates.decode_rates/1","doc":"","ref":"Money.ExchangeRates.OpenExchangeRates.html#decode_rates/1"},{"type":"function","title":"Money.ExchangeRates.OpenExchangeRates.get_historic_rates/2","doc":"Retrieves the historic exchange rates from Open Exchange Rates site.\n\n* `date` is a date returned by `Date.new/3` or any struct with the\n  elements `:year`, `:month` and `:day`.\n\n* `config` is the retrieval configuration. When invoked from the\n  exchange rates services this will be the config returned from\n  `Money.ExchangeRates.config/0`\n\nReturns:\n\n* `{:ok, rates}` if the rates can be retrieved\n\n* `{:error, reason}` if rates cannot be retrieved\n\nTypically this function is called by the exchange rates retrieval\nservice although it can be called outside that context as\nrequired.","ref":"Money.ExchangeRates.OpenExchangeRates.html#get_historic_rates/2"},{"type":"function","title":"Money.ExchangeRates.OpenExchangeRates.get_latest_rates/1","doc":"Retrieves the latest exchange rates from Open Exchange Rates site.\n\n* `config` is the retrieval configuration. When invoked from the\nexchange rates services this will be the config returned from\n`Money.ExchangeRates.config/0`\n\nReturns:\n\n* `{:ok, rates}` if the rates can be retrieved\n\n* `{:error, reason}` if rates cannot be retrieved\n\nTypically this function is called by the exchange rates retrieval\nservice although it can be called outside that context as\nrequired.","ref":"Money.ExchangeRates.OpenExchangeRates.html#get_latest_rates/1"},{"type":"function","title":"Money.ExchangeRates.OpenExchangeRates.init/1","doc":"Update the retriever configuration to include the requirements\nfor Open Exchange Rates.  This function is invoked when the\nexchange rate service starts up, just after the ets table\n:exchange_rates is created.\n\n* `default_config` is the configuration returned by `Money.ExchangeRates.default_config/0`\n\nReturns the configuration either unchanged or updated with\nadditional configuration specific to this exchange\nrates retrieval module.","ref":"Money.ExchangeRates.OpenExchangeRates.html#init/1"},{"type":"module","title":"Money.ExchangeRates.Retriever","doc":"Implements a `GenServer` to retrieve exchange rates from\na configured retrieveal module on a periodic or on demand basis.\n\nBy default exchange rates are retrieved from [Open Exchange Rates](http://openexchangerates.org).\n\nThe default period of execution is 5 minutes (300_000 milliseconds). The\nperiod of retrieval is configured in `config.exs` or the appropriate\nenvironment configuration.  For example:\n\n    config :ex_money,\n      retrieve_every: 300_000","ref":"Money.ExchangeRates.Retriever.html"},{"type":"function","title":"Money.ExchangeRates.Retriever.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Money.ExchangeRates.Retriever.html#child_spec/1"},{"type":"function","title":"Money.ExchangeRates.Retriever.config/0","doc":"Return the current configuration of the Exchange Rates\nRetrieval service","ref":"Money.ExchangeRates.Retriever.html#config/0"},{"type":"function","title":"Money.ExchangeRates.Retriever.delete/0","doc":"Delete the exchange rates retrieval service\n\nThe service can be started again with `start/1`","ref":"Money.ExchangeRates.Retriever.html#delete/0"},{"type":"function","title":"Money.ExchangeRates.Retriever.historic_rates/1","doc":"Forces retrieval of historic exchange rates for a single date\n\n* `date` is a date returned by `Date.new/3` or any struct with the\n  elements `:year`, `:month` and `:day` or\n\n* a `Date.Range.t` created by `Date.range/2` that specifies a\n  range of dates to retrieve\n\nReturns:\n\n* `{:ok, rates}` if exchange rates request is successfully sent.\n\n* `{:error, reason}` if the request cannot be send.\n\nSends a message ot the exchange rate retrieval worker to request\nhistoric rates for a specified date or range be retrieved and\nstored.\n\nThis function does not return exchange rates, for that see\n`Money.ExchangeRates.latest_rates/0` or\n`Money.ExchangeRates.historic_rates/1`.","ref":"Money.ExchangeRates.Retriever.html#historic_rates/1"},{"type":"function","title":"Money.ExchangeRates.Retriever.historic_rates/2","doc":"Forces retrieval of historic exchange rates for a range of dates\n\n* `from` is a date returned by `Date.new/3` or any struct with the\n  elements `:year`, `:month` and `:day`.\n\n* `to` is a date returned by `Date.new/3` or any struct with the\n  elements `:year`, `:month` and `:day`.\n\nReturns:\n\n* `{:ok, rates}` if exchange rates request is successfully sent.\n\n* `{:error, reason}` if the request cannot be send.\n\nSends a message to the exchange rate retrieval process for each\ndate in the range `from`..`to` to request historic rates be\nretrieved.","ref":"Money.ExchangeRates.Retriever.html#historic_rates/2"},{"type":"function","title":"Money.ExchangeRates.Retriever.latest_rates/0","doc":"Forces retrieval of the latest exchange rates\n\nSends a message ot the exchange rate retrieval worker to request\ncurrent rates be retrieved and stored.\n\nReturns:\n\n* `{:ok, rates}` if exchange rates request is successfully sent.\n\n* `{:error, reason}` if the request cannot be send.\n\nThis function does not return exchange rates, for that see\n`Money.ExchangeRates.latest_rates/0` or\n`Money.ExchangeRates.historic_rates/1`.","ref":"Money.ExchangeRates.Retriever.html#latest_rates/0"},{"type":"function","title":"Money.ExchangeRates.Retriever.reconfigure/1","doc":"Updated the configuration for the Exchange Rate\nService","ref":"Money.ExchangeRates.Retriever.html#reconfigure/1"},{"type":"function","title":"Money.ExchangeRates.Retriever.restart/0","doc":"Restart the exchange rates retrieval service","ref":"Money.ExchangeRates.Retriever.html#restart/0"},{"type":"function","title":"Money.ExchangeRates.Retriever.retrieve_rates/2","doc":"Retrieve exchange rates from an external HTTP\nservice.\n\nThis function is primarily intended for use by\nan exchange rates api module.","ref":"Money.ExchangeRates.Retriever.html#retrieve_rates/2"},{"type":"function","title":"Money.ExchangeRates.Retriever.start/1","doc":"Starts the exchange rates retrieval service","ref":"Money.ExchangeRates.Retriever.html#start/1"},{"type":"function","title":"Money.ExchangeRates.Retriever.stop/0","doc":"Stop the exchange rates retrieval service.\n\nThe service can be restarted with `restart/0`.","ref":"Money.ExchangeRates.Retriever.html#stop/0"},{"type":"module","title":"Money.ExchangeRates.Supervisor","doc":"Functions to manage the starting, stopping,\ndeleting and restarting of the Exchange\nRates Retriever.","ref":"Money.ExchangeRates.Supervisor.html"},{"type":"function","title":"Money.ExchangeRates.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Money.ExchangeRates.Supervisor.html#child_spec/1"},{"type":"function","title":"Money.ExchangeRates.Supervisor.default_supervisor/0","doc":"Returns the name of the default supervisor\nwhich is `Money.Supervisor`","ref":"Money.ExchangeRates.Supervisor.html#default_supervisor/0"},{"type":"function","title":"Money.ExchangeRates.Supervisor.delete_retriever/0","doc":"Deleted the retriever child specification from\nthe exchange rates supervisor.\n\nThis is primarily of use if you want to change\nthe configuration of the retriever after it is\nstopped and before it is restarted.\n\nIn this situation the sequence of calls would be:\n\n```\niex> Money.ExchangeRates.Retriever.stop\niex> Money.ExchangeRates.Retriever.delete\niex> Money.ExchangeRates.Retriever.start(config)\n```","ref":"Money.ExchangeRates.Supervisor.html#delete_retriever/0"},{"type":"function","title":"Money.ExchangeRates.Supervisor.restart_retriever/0","doc":"Restarts a stopped retriever.\n\nSee also `Money.ExchangeRates.Retriever.stop/0`","ref":"Money.ExchangeRates.Supervisor.html#restart_retriever/0"},{"type":"function","title":"Money.ExchangeRates.Supervisor.retriever_running?/0","doc":"Returns a boolean indicating of the\nretriever process is configured and\nrunning","ref":"Money.ExchangeRates.Supervisor.html#retriever_running?/0"},{"type":"function","title":"Money.ExchangeRates.Supervisor.retriever_status/0","doc":"Returns the status of the exchange rates\nretriever.  The returned value is one of:\n\n* `:running` if the service is running. In this\n  state the valid action is `Money.ExchangeRates.Service.stop/0`.\n\n* `:stopped` if it is stopped. In this state\n  the valid actions are `Money.ExchangeRates.Supervisor.restart_retriever/0`\n  or `Money.ExchangeRates.Supervisor.delete_retriever/0`.\n\n* `:not_started` if it is not configured\n  in the supervisor and is not running.  In\n  this state the only valid action is\n  `Money.ExchangeRates.Supervisor.start_retriever/1`.","ref":"Money.ExchangeRates.Supervisor.html#retriever_status/0"},{"type":"function","title":"Money.ExchangeRates.Supervisor.start_link/0","doc":"Starts the Exchange Rates supervisor and\noptionally starts the exchange rates\nretrieval service as well.","ref":"Money.ExchangeRates.Supervisor.html#start_link/0"},{"type":"function","title":"Options - Money.ExchangeRates.Supervisor.start_link/0","doc":"* `:restart` is a boolean value indicating\n  if the supervisor is to be restarted.  This is\n  typically used to move the supervisor from its\n  default position under the `ex_money` supervision\n  tree to a different supervision tree.  The default\n  is `false`\n\n* `:start_retriever` is a boolean indicating\n  if the exchange rates retriever is to be started\n  when the supervisor is started.  The default is\n  defined by the configuration key\n  `:auto_start_exchange_rate_service`","ref":"Money.ExchangeRates.Supervisor.html#start_link/0-options"},{"type":"function","title":"Money.ExchangeRates.Supervisor.start_link/1","doc":"","ref":"Money.ExchangeRates.Supervisor.html#start_link/1"},{"type":"function","title":"Money.ExchangeRates.Supervisor.start_retriever/1","doc":"Starts the exchange rates retriever.","ref":"Money.ExchangeRates.Supervisor.html#start_retriever/1"},{"type":"function","title":"Arguments - Money.ExchangeRates.Supervisor.start_retriever/1","doc":"* `config` is a `t:Money.ExchangeRates.Config.t/0`\n  struct returned by `Money.ExchangeRates.config/0`\n  and adjusted as required.  The default is\n  `Money.ExchangeRates.config/0`.","ref":"Money.ExchangeRates.Supervisor.html#start_retriever/1-arguments"},{"type":"function","title":"Money.ExchangeRates.Supervisor.stop/1","doc":"Stop the Money.ExchangeRates.Supervisor.\n\nUnless `ex_money` is configured in `mix.exs` as\n`rumtime: false`, the Money.ExchangeRates.Supervisor\nis always started when `ex_money` starts even if the\nconfig key `:auto_start_exchange_rates_service` is\nset to `false`.\n\n\nIn some instances an application may require the\n`Money.ExchangeRates.Supervisor` to be started under\na different supervision tree.  In this case it is\nrequired to call this function first before a new\nconfiguration is started.\n\nOne use case is when the Exchange Rates service is\nconfigured with either an API module, a Callback module\nor a Cache module which uses Ecto and therefore its\na requirement that Ecto is started first.\n\nSee the README section on \"Using Ecto or other applications\nfrom within the callback module\" for an eanple of how\nto configure the supervisor in this case.","ref":"Money.ExchangeRates.Supervisor.html#stop/1"},{"type":"function","title":"Money.ExchangeRates.Supervisor.stop_retriever/0","doc":"Stop the exchange rates retriever.","ref":"Money.ExchangeRates.Supervisor.html#stop_retriever/0"},{"type":"module","title":"Money.Subscription","doc":"Provides functions to create, upgrade and downgrade subscriptions\nfrom one plan to another.\n\nSince moving from one plan to another may require\nprorating the payment stream at the point of transition,\nthis module is introduced to provide a single point of\ncalculation of the proration in order to give clear focus\nto the issues of calculating the carry-over amount or\nthe carry-over period at the point of plan change.","ref":"Money.Subscription.html"},{"type":"module","title":"Defining a subscription - Money.Subscription","doc":"A subscription records this current state and history of\nall plans assigned over time to a subscriber.  The definition\nis deliberately minimal to simplify integration into applications\nthat have a specific implementation of a subscription.\n\nA new subscription is created with `Money.Subscription.new/3`\nwhich has the following attributes:\n\n* `plan` which defines the initial plan for the subscription.\nThis option is required.\n\n* `effective_date` which determines the effective date of\nthe inital plan.  This option is required.\n\n* `options` which include `:created_at` and `:id` with which\n  a subscription may be annotated","ref":"Money.Subscription.html#module-defining-a-subscription"},{"type":"module","title":"Changing a subscription plan - Money.Subscription","doc":"Changing a subscription plan requires the following\ninformation be provided:\n\n* A Subscription or the definition of the current plan\n\n* The definition of the new plan\n\n* The strategy for changing the plan which is either:\n\n  * to have the effective date of the new plan be after\n    the current interval of the current plan\n\n  * To change the plan immediately in which case there will\n    be a credit on the current plan which needs to be applied\n    to the new plan.\n\nSee `Money.Subscription.change_plan/3`","ref":"Money.Subscription.html#module-changing-a-subscription-plan"},{"type":"module","title":"When the new plan is effective at the end of the current billing period - Money.Subscription","doc":"The first strategy simply finishes the current billing period before\nthe new plan is introduced and therefore no proration is required.\nThis is the default strategy.","ref":"Money.Subscription.html#module-when-the-new-plan-is-effective-at-the-end-of-the-current-billing-period"},{"type":"module","title":"When the new plan is effective immediately - Money.Subscription","doc":"If the new plan is to be effective immediately then any credit\nbalance remaining on the old plan needs to be applied to the\nnew plan.  There are two options of applying the credit:\n\n1. Reduce the billing amount of the first period of the new plan\n   be the amount of the credit left on the old plan. This means\n   that the billing amount for the first period of the new plan\n   will be different (less) than the billing amount for subsequent\n   periods on the new plan.\n\n2. Extend the first period of the new plan by the interval amount\n   that can be funded by the credit amount left on the old plan. In\n   the situation where the credit amount does not fully fund an integral\n   interval the additional interval can be truncated or rounded up to the next\n   integral period.","ref":"Money.Subscription.html#module-when-the-new-plan-is-effective-immediately"},{"type":"module","title":"Plan definition - Money.Subscription","doc":"This module, and `Money` in general, does not provide a full\nbilling or subscription solution - its focus is to support a reliable\nmeans of calcuating the accounting outcome of a plan change only.\nTherefore the plan definition required by `Money.Subscription` can be\nany `Map.t` that includes the following fields:\n\n* `interval` which defines the time interval for a plan. The value\n  can be one of `day`, `week`, `month` or `year`.\n\n* `interval_count` which defines the number of `interval`s for the\n  current plan interval.  This must be a positive integer.\n\n* `price` which is a `Money.t` representing the price of the plan\n  to be paid each interval count.","ref":"Money.Subscription.html#module-plan-definition"},{"type":"module","title":"Billing in advance - Money.Subscription","doc":"This module calculates all subscription changes on the basis\nthat billing is done in advance.  This primarily affects the\ncalculation of plan credit when a plan changes.  The assumption\nis that the period from the start of the current interval to\nthe point of change has been consumed and therefore the credit\nis based upon that period of the plan that has not yet been\nconsumed.\n\nIf the calculation was based upon \"payment in arrears\" then\nthe credit would actually be a debit since the part of the\ncurrent period consumed has not yet been paid.","ref":"Money.Subscription.html#module-billing-in-advance"},{"type":"function","title":"Money.Subscription.__struct__/0","doc":"A `struct` defining a subscription\n\n* `:id` any term that uniquely identifies this subscription\n\n* `:plans` is a list of `{change, plan}` tuples that record the history\n  of plans assigned to this subscription\n\n* `:created_at` records the `DateTime.t` when the subscription was created","ref":"Money.Subscription.html#__struct__/0"},{"type":"function","title":"Money.Subscription.cancel_pending_plan/2","doc":"Cancel a subscription's pending plan.\n\nA pending plan arise when a a `Subscription.change_plan/3` has\nbeen executed but the effective date is in the future.  Only\none plan may be pending at any one time so that if\n`Subscription.change_plan/3` is attemtped a second time an\nerror tuple will be returned.\n\n`Subscription.cancel_pending_plan/2`\ncan be used to roll back the pending plan change.","ref":"Money.Subscription.html#cancel_pending_plan/2"},{"type":"function","title":"Arguments - Money.Subscription.cancel_pending_plan/2","doc":"* `:subscription` is any `Money.Subscription.t`\n\n* `:options` is a `Keyword.t`","ref":"Money.Subscription.html#cancel_pending_plan/2-arguments"},{"type":"function","title":"Options - Money.Subscription.cancel_pending_plan/2","doc":"* `:today` is a `Date.t` that represents today.\n  The default is `Date.utc_today`","ref":"Money.Subscription.html#cancel_pending_plan/2-options"},{"type":"function","title":"Returns - Money.Subscription.cancel_pending_plan/2","doc":"* An updated `Money.Subscription.t` which may or may not\nhave had a pending plan.  If it did have a pending plan\nthat plan is deleted.  If there was no pending plan then\nthe subscription is returned unchanged.","ref":"Money.Subscription.html#cancel_pending_plan/2-returns"},{"type":"function","title":"Money.Subscription.change_plan/3","doc":"Change plan from the current plan to a new plan.","ref":"Money.Subscription.html#change_plan/3"},{"type":"function","title":"Arguments - Money.Subscription.change_plan/3","doc":"* `subscription_or_plan` is either a `Money.Subscription.t` or `Money.Subscription.Plan.t`\n  or a map with the same fields\n\n* `new_plan` is a `Money.Subscription.Plan.t` or a map with at least the fields\n  `interval`, `interval_count` and `price`\n\n* `current_interval_started` is a `Date.t` or other map with the fields `year`, `month`,\n  `day` and `calendar`\n\n* `options` is a keyword list of options the define how the change is to be made","ref":"Money.Subscription.html#change_plan/3-arguments"},{"type":"function","title":"Options - Money.Subscription.change_plan/3","doc":"* `:effective` defines when the new plan comes into effect.  The values are `:immediately`,\n  a `Date.t` or `:next_period`.  The default is `:next_period`.  Note that the date\n  applied in the case of `:immediately` is the date returned by `Date.utc_today`.\n\n* `:prorate` which determines how to prorate the current plan into the new plan.  The\n  options are `:price` which will reduce the price of the first period of the new plan\n  by the credit amount left on the old plan (this is the default). Or `:period` in which\n  case the first period of the new plan is extended by the `interval` amount of the new\n  plan that the credit on the old plan will fund.\n\n* `:round` determines whether when prorating the `:period` it is truncated or rounded up\n  to the next nearest full `interval_count`. Valid values are `:down`, `:half_up`,\n  `:half_even`, `:ceiling`, `:floor`, `:half_down`, `:up`.  The default is `:up`.\n\n* `:first_interval_started` determines the anchor day for monthly billing.  For\n  example if a monthly plan starts on January 31st then the next period will start\n  on February 28th (or 29th).  The period following that should, however, be March 31st.\n  If `subscription_or_plan` is a `Money.Subscription.t` then the `:first_interval_started`\n  is automatically populated from the subscription. If `:first_interval_started` is\n  `nil` then the date defined by `:effective` is used.","ref":"Money.Subscription.html#change_plan/3-options"},{"type":"function","title":"Returns - Money.Subscription.change_plan/3","doc":"A `Money.Subscription.Change.t` with the following elements:\n\n* `:first_interval_starts` which is the start date of the first interval for the new\n  plan\n\n* `:first_billing_amount` is the amount to be billed, net of any credit, at\n  the `:first_interval_starts`\n\n* `:next_interval_starts` is the start date of the next interval after the `\n  first interval `including any `credit_days_applied`\n\n* `:credit_amount` is the amount of unconsumed credit of the current plan\n\n* `:credit_amount_applied` is the amount of credit applied to the new plan. If\n  the `:prorate` option is `:price` (the default) then `:first_billing_amount`\n  is the plan `:price` reduced by the `:credit_amount_applied`. If the `:prorate`\n  option is `:period` then the `:first_billing_amount` is the plan `price` and\n  the `:next_interval_date` is extended by the `:credit_days_applied`\n  instead.\n\n* `:credit_days_applied` is the number of days credit applied to the first\n  interval by adding days to the `:first_interval_starts` date.\n\n* `:credit_period_ends` is the date on which any applied credit is consumed or `nil`\n\n* `:carry_forward` is any amount of credit carried forward to a subsequent period.\n  If non-zero, this amount is a negative `Money.t`. It is non-zero when the credit\n  amount for the current plan is greater than the `:price` of the new plan.  In\n  this case the `:first_billing_amount` is zero.","ref":"Money.Subscription.html#change_plan/3-returns"},{"type":"function","title":"Returns - Money.Subscription.change_plan/3","doc":"* `{:ok, updated_subscription}` or\n\n* `{:error, {exception, message}}`","ref":"Money.Subscription.html#change_plan/3-returns"},{"type":"function","title":"Examples - Money.Subscription.change_plan/3","doc":"# Change at end of the current period so no proration\n    iex> current = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 1)\n    iex> new = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 3)\n    iex> Money.Subscription.change_plan current, new, current_interval_started: ~D[2018-01-01]\n    {:ok, %Money.Subscription.Change{\n      carry_forward: Money.zero(:USD),\n      credit_amount: Money.zero(:USD),\n      credit_amount_applied: Money.zero(:USD),\n      credit_days_applied: 0,\n      credit_period_ends: nil,\n      next_interval_starts: ~D[2018-05-01],\n      first_billing_amount: Money.new(:USD, 10),\n      first_interval_starts: ~D[2018-02-01]\n    }}\n\n    # Change during the current plan generates a credit amount\n    iex> current = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 1)\n    iex> new = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 3)\n    iex> Money.Subscription.change_plan current, new, current_interval_started: ~D[2018-01-01], effective: ~D[2018-01-15]\n    {:ok, %Money.Subscription.Change{\n      carry_forward: Money.zero(:USD),\n      credit_amount: Money.new(:USD, \"5.49\"),\n      credit_amount_applied: Money.new(:USD, \"5.49\"),\n      credit_days_applied: 0,\n      credit_period_ends: nil,\n      next_interval_starts: ~D[2018-04-15],\n      first_billing_amount: Money.new(:USD, \"4.51\"),\n      first_interval_starts: ~D[2018-01-15]\n    }}\n\n    # Change during the current plan generates a credit period\n    iex> current = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 1)\n    iex> new = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 3)\n    iex> Money.Subscription.change_plan current, new, current_interval_started: ~D[2018-01-01], effective: ~D[2018-01-15], prorate: :period\n    {:ok, %Money.Subscription.Change{\n      carry_forward: Money.zero(:USD),\n      credit_amount: Money.new(:USD, \"5.49\"),\n      credit_amount_applied: Money.zero(:USD),\n      credit_days_applied: 50,\n      credit_period_ends: ~D[2018-03-05],\n      next_interval_starts: ~D[2018-06-04],\n      first_billing_amount: Money.new(:USD, 10),\n      first_interval_starts: ~D[2018-01-15]\n    }}","ref":"Money.Subscription.html#change_plan/3-examples"},{"type":"function","title":"Money.Subscription.change_plan!/3","doc":"Change plan from the current plan to a new plan.\n\nRetuns the plan or raises an exception on error.\n\nSee `Money.Subscription.change_plan/3` for the description\nof arguments, options and return.","ref":"Money.Subscription.html#change_plan!/3"},{"type":"function","title":"Money.Subscription.current_interval_start_date/2","doc":"Returns the first date of the current interval of a plan.","ref":"Money.Subscription.html#current_interval_start_date/2"},{"type":"function","title":"Arguments - Money.Subscription.current_interval_start_date/2","doc":"* `:subscription_or_changeset` is any`Money.Subscription.t` or\n  a `{Change.t, Plan.t}` tuple\n\n* `:options` is a keyword list of options","ref":"Money.Subscription.html#current_interval_start_date/2-arguments"},{"type":"function","title":"Options - Money.Subscription.current_interval_start_date/2","doc":"* `:today` is a `Date.t` that represents today.\n  The default is `Date.utc_today`","ref":"Money.Subscription.html#current_interval_start_date/2-options"},{"type":"function","title":"Returns - Money.Subscription.current_interval_start_date/2","doc":"* The `Date.t` that is the first date of the current interval","ref":"Money.Subscription.html#current_interval_start_date/2-returns"},{"type":"function","title":"Money.Subscription.current_plan/2","doc":"Retrieve the plan that is currently in affect.\n\nThe plan in affect is not necessarily the first\nplan in the list.  We may have upgraded plans to\nbe in affect at some later time.","ref":"Money.Subscription.html#current_plan/2"},{"type":"function","title":"Arguments - Money.Subscription.current_plan/2","doc":"* `subscription` is a `Money.Subscription.t` or any\n  map that provides the field `:plans`","ref":"Money.Subscription.html#current_plan/2-arguments"},{"type":"function","title":"Returns - Money.Subscription.current_plan/2","doc":"* The `Money.Subscription.Plan.t` that is the plan currently in affect or\n  `nil`","ref":"Money.Subscription.html#current_plan/2-returns"},{"type":"function","title":"Money.Subscription.current_plan_start_date/1","doc":"Returns the start date of the current plan.","ref":"Money.Subscription.html#current_plan_start_date/1"},{"type":"function","title":"Arguments - Money.Subscription.current_plan_start_date/1","doc":"* `subscription` is a `Money.Subscription.t` or any\n  map that provides the field `:plans`","ref":"Money.Subscription.html#current_plan_start_date/1-arguments"},{"type":"function","title":"Returns - Money.Subscription.current_plan_start_date/1","doc":"* The start `Date.t` of the current plan","ref":"Money.Subscription.html#current_plan_start_date/1-returns"},{"type":"function","title":"Money.Subscription.days_remaining/4","doc":"Returns number of days remaining in a plan interval.","ref":"Money.Subscription.html#days_remaining/4"},{"type":"function","title":"Arguments - Money.Subscription.days_remaining/4","doc":"* `plan` is any `Money.Subscription.Plan.t`\n\n* `current_interval_started` is a `Date.t`\n\n* `effective_date` is a `Date.t` after the\n  `current_interval_started` and before the end of\n  the `plan_days`","ref":"Money.Subscription.html#days_remaining/4-arguments"},{"type":"function","title":"Returns - Money.Subscription.days_remaining/4","doc":"The number of days remaining in a plan interval","ref":"Money.Subscription.html#days_remaining/4-returns"},{"type":"function","title":"Examples - Money.Subscription.days_remaining/4","doc":"iex> plan = Money.Subscription.Plan.new! Money.new!(:USD, 100), :month, 1\n    iex> Money.Subscription.days_remaining plan, ~D[2018-01-01], ~D[2018-01-02]\n    30\n    iex> Money.Subscription.days_remaining plan, ~D[2018-02-01], ~D[2018-02-02]\n    27","ref":"Money.Subscription.html#days_remaining/4-examples"},{"type":"type","title":"Money.Subscription.id/0","doc":"An id that uniquely identifies a subscription","ref":"Money.Subscription.html#t:id/0"},{"type":"function","title":"Money.Subscription.latest_plan/1","doc":"Returns the latest plan for a subscription.\n\nThe latest plan may not be in affect since\nits start date may be in the future.","ref":"Money.Subscription.html#latest_plan/1"},{"type":"function","title":"Arguments - Money.Subscription.latest_plan/1","doc":"* `subscription` is a `Money.Subscription.t` or any\n  map that provides the field `:plans`","ref":"Money.Subscription.html#latest_plan/1-arguments"},{"type":"function","title":"Returns - Money.Subscription.latest_plan/1","doc":"* The `Money.Subscription.Plan.t` that is the most recent\n  plan - whether or not it is the currently active plan.","ref":"Money.Subscription.html#latest_plan/1-returns"},{"type":"function","title":"Money.Subscription.new/3","doc":"Creates a new subscription.","ref":"Money.Subscription.html#new/3"},{"type":"function","title":"Arguments - Money.Subscription.new/3","doc":"* `plan` is any `Money.Subscription.Plan.t` the defines the initial plan\n\n* `effective_date` is a `Date.t` that represents the effective\n  date of the initial plan. This defines the start of the first interval\n\n* `options` is a keyword list of options","ref":"Money.Subscription.html#new/3-arguments"},{"type":"function","title":"Options - Money.Subscription.new/3","doc":"* `:id` is any term that an application can use to uniquely identify\n  this subscription.  It is not used in any function in this module.\n\n* `:created_at` is a `DateTime.t` that records the timestamp when\n  the subscription was created.  The default is `DateTime.utc_now/0`","ref":"Money.Subscription.html#new/3-options"},{"type":"function","title":"Returns - Money.Subscription.new/3","doc":"* `{:ok, Money.Subscription.t}` or\n\n* `{:error, {exception, message}}`","ref":"Money.Subscription.html#new/3-returns"},{"type":"function","title":"Money.Subscription.new!/3","doc":"Creates a new subscription or raises an exception.","ref":"Money.Subscription.html#new!/3"},{"type":"function","title":"Arguments - Money.Subscription.new!/3","doc":"* `plan` is any `Money.Subscription.Plan.t` the defines the initial plan\n\n* `effective_date` is a `Date.t` that represents the effective\n  date of the initial plan. This defines the start of the first interval\n\n* `:options` is a keyword list of options","ref":"Money.Subscription.html#new!/3-arguments"},{"type":"function","title":"Options - Money.Subscription.new!/3","doc":"* `:id` is any term that an application can use to uniquely identify\n  this subscription.  It is not used in any function in this module.\n\n* `:created_at` is a `DateTime.t` that records the timestamp when\n  the subscription was created.  The default is `DateTime.utc_now/0`","ref":"Money.Subscription.html#new!/3-options"},{"type":"function","title":"Returns - Money.Subscription.new!/3","doc":"* A `Money.Subscription.t` or\n\n* raises an exception","ref":"Money.Subscription.html#new!/3-returns"},{"type":"function","title":"Money.Subscription.next_interval_starts/3","doc":"Returns the next interval start date for a plan.","ref":"Money.Subscription.html#next_interval_starts/3"},{"type":"function","title":"Arguments - Money.Subscription.next_interval_starts/3","doc":"* `plan` is any `Money.Subscription.Plan.t`\n\n* `:current_interval_started` is the `Date.t` that\n  represents the start of the current interval","ref":"Money.Subscription.html#next_interval_starts/3-arguments"},{"type":"function","title":"Returns - Money.Subscription.next_interval_starts/3","doc":"The next interval start date as a `Date.t`.","ref":"Money.Subscription.html#next_interval_starts/3-returns"},{"type":"function","title":"Example - Money.Subscription.next_interval_starts/3","doc":"iex> plan = Money.Subscription.Plan.new!(Money.new!(:USD, 100), :month)\n    iex> Money.Subscription.next_interval_starts(plan, ~D[2018-03-01])\n    ~D[2018-04-01]\n\n    iex> plan = Money.Subscription.Plan.new!(Money.new!(:USD, 100), :day, 30)\n    iex> Money.Subscription.next_interval_starts(plan, ~D[2018-02-01])\n    ~D[2018-03-03]","ref":"Money.Subscription.html#next_interval_starts/3-example"},{"type":"function","title":"Money.Subscription.plan_days/3","doc":"Returns number of days in a plan interval.","ref":"Money.Subscription.html#plan_days/3"},{"type":"function","title":"Arguments - Money.Subscription.plan_days/3","doc":"* `plan` is any `Money.Subscription.Plan.t`\n\n* `current_interval_started` is any `Date.t`","ref":"Money.Subscription.html#plan_days/3-arguments"},{"type":"function","title":"Returns - Money.Subscription.plan_days/3","doc":"The number of days in a plan interval.","ref":"Money.Subscription.html#plan_days/3-returns"},{"type":"function","title":"Examples - Money.Subscription.plan_days/3","doc":"iex> plan = Money.Subscription.Plan.new! Money.new!(:USD, 100), :month, 1\n    iex> Money.Subscription.plan_days plan, ~D[2018-01-01]\n    31\n    iex> Money.Subscription.plan_days plan, ~D[2018-02-01]\n    28\n    iex> Money.Subscription.plan_days plan, ~D[2018-04-01]\n    30","ref":"Money.Subscription.html#plan_days/3-examples"},{"type":"function","title":"Money.Subscription.plan_pending?/2","doc":"Returns a `boolean` indicating if there is a pending plan.\n\nA pending plan is one where the subscription has changed\nplans but the plan is not yet in effect.  There can only\nbe one pending plan.","ref":"Money.Subscription.html#plan_pending?/2"},{"type":"function","title":"Arguments - Money.Subscription.plan_pending?/2","doc":"* `:subscription` is any `Money.Subscription.t`\n\n* `:options` is a keyword list of options","ref":"Money.Subscription.html#plan_pending?/2-arguments"},{"type":"function","title":"Options - Money.Subscription.plan_pending?/2","doc":"* `:today` is a `Date.t` that represents the effective\n  date used to determine is there is a pending plan.\n  The default is `Date.utc_today/1`.","ref":"Money.Subscription.html#plan_pending?/2-options"},{"type":"function","title":"Returns - Money.Subscription.plan_pending?/2","doc":"* Either `true` or `false`","ref":"Money.Subscription.html#plan_pending?/2-returns"},{"type":"type","title":"Money.Subscription.t/0","doc":"A Money.Subscription type","ref":"Money.Subscription.html#t:t/0"},{"type":"module","title":"Money.Subscription.Change","doc":"Defines the structure of a plan changeset.\n\n* `:first_interval_starts` which is the start date of the first interval for the new\n  plan\n\n* `:first_billing_amount` is the amount to be billed, net of any credit, at\n  the `:first_interval_starts`\n\n* `:next_interval_starts` is the start date of the next interval after the `\n  first interval including any `credit_days_applied`\n\n* `:credit_amount` is the amount of unconsumed credit of the current plan\n\n* `:credit_amount_applied` is the amount of credit applied to the new plan. If\n  the `:prorate` option is `:price` (the default) the `:first_billing_amount`\n  is the plan `:price` reduced by the `:credit_amount_applied`. If the `:prorate`\n  option is `:period` then the `:first_billing_amount` is the plan `price and\n  the `:next_interval_date` is extended by the `:credit_days_applied`\n  instead.\n\n* `:credit_days_applied` is the number of days credit applied to the first\n  interval by adding days to the `:first_interval_starts` date.\n\n* `:credit_period_ends` is the date on which any applied credit is consumed or `nil`\n\n* `:carry_forward` is any amount of credit carried forward to a subsequent period.\n  If non-zero this amount is a negative `Money.t`. It is non-zero when the credit\n  amount for the current plan is greater than the price of the new plan.  In\n  this case the `:first_billing_amount` is zero.","ref":"Money.Subscription.Change.html"},{"type":"function","title":"Money.Subscription.Change.__struct__/0","doc":"A struct defining the changes between two plans.","ref":"Money.Subscription.Change.html#__struct__/0"},{"type":"type","title":"Money.Subscription.Change.t/0","doc":"A plan change record struct.","ref":"Money.Subscription.Change.html#t:t/0"},{"type":"exception","title":"Money.Subscription.DateError","doc":"","ref":"Money.Subscription.DateError.html"},{"type":"exception","title":"Money.Subscription.NoCurrentPlan","doc":"","ref":"Money.Subscription.NoCurrentPlan.html"},{"type":"module","title":"Money.Subscription.Plan","doc":"Defines a standard subscription plan data structure.","ref":"Money.Subscription.Plan.html"},{"type":"function","title":"Money.Subscription.Plan.__struct__/0","doc":"Defines the structure of a subscription plan.","ref":"Money.Subscription.Plan.html#__struct__/0"},{"type":"type","title":"Money.Subscription.Plan.interval/0","doc":"A plan interval type.","ref":"Money.Subscription.Plan.html#t:interval/0"},{"type":"type","title":"Money.Subscription.Plan.interval_count/0","doc":"A integer interval count for a plan.","ref":"Money.Subscription.Plan.html#t:interval_count/0"},{"type":"function","title":"Money.Subscription.Plan.new/3","doc":"Returns `{:ok, Money.Subscription.Plan.t}` or an `{:error, reason}`\ntuple.","ref":"Money.Subscription.Plan.html#new/3"},{"type":"function","title":"Arguments - Money.Subscription.Plan.new/3","doc":"* `:price` is any `Money.t`\n\n* `:interval` is the period of the plan.  The valid intervals are\n`  `:day`, `:week`, `:month` or ':year`.\n\n* `:interval_count` is an integer count of the number of `:interval`s\n  of the plan.  The default is `1`","ref":"Money.Subscription.Plan.html#new/3-arguments"},{"type":"function","title":"Returns - Money.Subscription.Plan.new/3","doc":"A `Money.Subscription.Plan.t`","ref":"Money.Subscription.Plan.html#new/3-returns"},{"type":"function","title":"Examples - Money.Subscription.Plan.new/3","doc":"iex> Money.Subscription.Plan.new Money.new(:USD, 100), :month, 1\n    {:ok,\n     %Money.Subscription.Plan{\n       interval: :month,\n       interval_count: 1,\n       price: Money.new(:USD, 100)\n     }}\n\n    iex> Money.Subscription.Plan.new Money.new(:USD, 100), :month\n    {:ok,\n     %Money.Subscription.Plan{\n       interval: :month,\n       interval_count: 1,\n       price: Money.new(:USD, 100)\n     }}\n\n    iex> Money.Subscription.Plan.new Money.new(:USD, 100), :day, 30\n    {:ok,\n     %Money.Subscription.Plan{\n       interval: :day,\n       interval_count: 30,\n       price: Money.new(:USD, 100)\n     }}\n\n    iex> Money.Subscription.Plan.new 23, :day, 30\n    {:error, {Money.Invalid, \"Invalid subscription plan definition\"}}","ref":"Money.Subscription.Plan.html#new/3-examples"},{"type":"function","title":"Money.Subscription.Plan.new!/3","doc":"Returns `{:ok, Money.Subscription.Plan.t}` or raises an\nexception.\n\nTakes the same arguments as `Money.Subscription.Plan.new/3`.","ref":"Money.Subscription.Plan.html#new!/3"},{"type":"function","title":"Example - Money.Subscription.Plan.new!/3","doc":"iex> Money.Subscription.Plan.new! Money.new(:USD, 100), :day, 30\n    %Money.Subscription.Plan{\n      interval: :day,\n      interval_count: 30,\n      price: Money.new(:USD, 100)\n    }","ref":"Money.Subscription.Plan.html#new!/3-example"},{"type":"type","title":"Money.Subscription.Plan.t/0","doc":"A Subscription Plan","ref":"Money.Subscription.Plan.html#t:t/0"},{"type":"exception","title":"Money.Subscription.PlanError","doc":"","ref":"Money.Subscription.PlanError.html"},{"type":"exception","title":"Money.Subscription.PlanPending","doc":"","ref":"Money.Subscription.PlanPending.html"},{"type":"exception","title":"Money.ExchangeRateError","doc":"","ref":"Money.ExchangeRateError.html"},{"type":"exception","title":"Money.Invalid","doc":"","ref":"Money.Invalid.html"},{"type":"exception","title":"Money.InvalidAmountError","doc":"","ref":"Money.InvalidAmountError.html"},{"type":"exception","title":"Money.InvalidDigitsError","doc":"","ref":"Money.InvalidDigitsError.html"},{"type":"exception","title":"Money.ParseError","doc":"","ref":"Money.ParseError.html"},{"type":"exception","title":"Money.UnknownCurrencyError","doc":"","ref":"Money.UnknownCurrencyError.html"},{"type":"extras","title":"Introduction to Money","doc":"# Introduction to Money\n\n![Build status](https://github.com/kipcole9/money/actions/workflows/ci.yml/badge.svg)\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_money.svg)](https://hex.pm/packages/ex_money)\n[![Hex.pm](https://img.shields.io/hexpm/dw/ex_money.svg?)](https://hex.pm/packages/ex_money)\n[![Hex.pm](https://img.shields.io/hexpm/dt/ex_money.svg?)](https://hex.pm/packages/ex_money)\n[![Hex.pm](https://img.shields.io/hexpm/l/ex_money.svg)](https://hex.pm/packages/ex_money)\n\nMoney implements a set of functions to store, retrieve, convert and perform arithmetic\non a `t:Money.t/0` type that is composed of an [ISO 4217](https://www.iso.org/iso-4217-currency-codes.html) currency code or an [ISO 24165](https://www.iso.org/standard/80601.html) Digital Token Identifier (crypto currency) with a currency amount.\n\nMoney is opinionated in the interests of serving as a dependable library that can underpin accounting and financial applications.\n\nHow is this opinion expressed?\n\n1. Money must always have both a amount and a currency code or digital token identifier.\n\n2. The currency code must always be a valid [ISO 4217](https://www.iso.org/iso-4217-currency-codes.html) code or a valid [ISO 24165](https://www.iso.org/standard/80601.html) digital token idenfier. [Current](https://www.currency-iso.org/en/home/tables/table-a1.html) and [historical](https://www.currency-iso.org/en/home/tables/table-a3.html) currency codes can be used.  See the [ISO Currency](https://www.currency-iso.org/en/home/tables.html) for more information. You can also identify the relevant codes by:\n\n   * `Money.known_currencies/0` returns all the ISO 4217 currency codes known to `Money`\n   * `Money.known_current_currencies/0` returns the ISO 4217 currency codes currently in use\n   * `Money.known_historic_currencies/0` returns the list of historic ISO 4217 currency codes\n   * `Money.known_tender_currencies/0` returns the list of ISO 4217 currencies known to be legal tender\n   * `DigitalToken.tokens/0` returns a map of the known ISO 24165 digital tokens.\n\n3. Money arithmetic can only be performed when both operands are of the same currency.\n\n4. Money amounts are represented as a `Decimal`.\n\n5. Money can be serialised to the database as a composite Postgres type that includes both the amount and the currency. For MySQL, money is serialized into a json column with the amount converted to a string to preserve precision since json does not have a decimal type. Serialization is entirely optional.\n\n6. All arithmetic functions work on a `Decimal`. No rounding occurs automatically (unless expressly called out for a function, as is the case for `Money.split/2`).\n\n7. Explicit rounding obeys the rounding rules for a given currency. The rounding rules are defined by the Unicode consortium in its CLDR repository as implemented by the hex package [ex_cldr](https://hex.pm/packages/ex_cldr). These rules define the number of fractional digits for a currency and the rounding increment where appropriate.\n\n8. Money output string formatting output using the hex package [ex_cldr](https://hex.pm/packages/ex_cldr) that correctly rounds to the appropriate number of fractional digits and to the correct rounding increment for currencies that have minimum cash increments (like the Swiss Franc and Australian Dollar)","ref":"readme.html"},{"type":"extras","title":"Prerequisities - Introduction to Money","doc":"* `Money` is supported on Elixir 1.11 and later only.","ref":"readme.html#prerequisities"},{"type":"extras","title":"Supervisor configuration and operation - Introduction to Money","doc":"`Money` starts a supervisor `Money.Supervisor` by default unless the dependency is configured as `runtime: false` in `mix.exs`. If configured as `runtime: false` then the application can be started later via `Money.Application.start(:normal, supervisor_options)` where `supervisor_options` is a keyword list of options that is given the `Supervisor.start_link/2`. The default options are `[strategy: :one_for_one, name: Money.Supervisor]`.\n\nThe application supervisor is used by default to start the exchange rates service when required. The exchange rate service can be configured to run in a user defined supervision tree as explained in the next section.","ref":"readme.html#supervisor-configuration-and-operation"},{"type":"extras","title":"Private Use Currencies - Introduction to Money","doc":"As of [ex_cldr_currencies version 2.6.0](https://hex.pm/packages/ex_cldr_currencies/2.6.0) it is possible to define private use currencies. These are currencies that are [ISO 4217](https://www.iso.org/iso-4217-currency-codes.html) compliant but guaranteed never to be allocated by the ISO committee and therefore safe to be used by developers.","ref":"readme.html#private-use-currencies"},{"type":"extras","title":"Defining private use currencies - Introduction to Money","doc":"See [Cldr.Currency.new/2](https://hexdocs.pm/ex_cldr_currencies/Cldr.Currency.html#new/2)","ref":"readme.html#defining-private-use-currencies"},{"type":"extras","title":"Starting the private use currency store - Introduction to Money","doc":"In order to define private use currencies, a special `:ets` table is required to hold their definitions. The is implemented by a supervisor and two workers that together aim to preserve the availability of the `:ets` table as resiliently as possible. The implementation is an embedded and updated version of [eternal](https://hex.pm/packages/eternal).\n\nThe basic requirement is to add a the private use currency supervisor to your applications supervision tree. For example:\n```elixir\ndefmodule MyApp do\n  use Application\n\n  def start(_type, _args) do\n\n    # Start the service which maintains the\n    # :ets table that holds the private use currencies\n    children = [\n      Cldr.Currency\n      ...\n    ]\n\n    opts = [strategy: :one_for_one, name: MoneyTest.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\nIt is also possible to define a callback that is called when the `Cldr.Currency` supervisor is started so that private use currencies can be defined. For further information see [Defining Private Use Currencies](https://hexdocs.pm/ex_cldr_currencies/readme.html#defining-private-use-currencies).","ref":"readme.html#starting-the-private-use-currency-store"},{"type":"extras","title":"Exchange rates and currency conversion - Introduction to Money","doc":"Money includes a process to retrieve exchange rates on a periodic basis.  These exchange rates can then be used to support currency conversion.  This service is not started by default.  If started it will attempt to retrieve exchange rates every 5 minutes by default.\n\nBy default, exchange rates are retrieved from [Open Exchange Rates](http://openexchangerates.org) however any module that conforms to the `Money.ExchangeRates` behaviour can be configured.\n\nAn optional callback module can also be defined.  This module defines a `rates_retrieved/2` function that is invoked upon every successful retrieval of exchange rates.  This might be used to serialize exchange rate to a data store or to stream rates to other applications or systems.","ref":"readme.html#exchange-rates-and-currency-conversion"},{"type":"extras","title":"Configuration - Introduction to Money","doc":"`Money` provides a set of configuration keys to customize behaviour. The default configuration is:\n\n    config :ex_money,\n      exchange_rates_retrieve_every: 300_000,\n      api_module: Money.ExchangeRates.OpenExchangeRates,\n      callback_module: Money.ExchangeRates.Callback,\n      exchange_rates_cache_module: Money.ExchangeRates.Cache.Ets,\n      preload_historic_rates: nil,\n      retriever_options: nil,\n      log_failure: :warning,\n      log_info: :info,\n      log_success: nil,\n      json_library: Jason on Elixir up to 1.17, JSON on Elixir 1.18+,\n      default_cldr_backend: MyApp.Cldr,\n      exclude_protocol_implementations: []","ref":"readme.html#configuration"},{"type":"extras","title":"Configuration key definitions - Introduction to Money","doc":"* `:default_cldr_backend` defines the `Cldr` backend module that is default for `Money`.  See the [ex_cldr documentation](https://hexdocs.pm/ex_cldr/2.0.0/readme.html) for further information on how to define this module.  **This is a required option**.\n\n* `:exchange_rates_retrieve_every` defines how often the exchange rates are retrieved in milliseconds.  The default is `:never`. An `atom` value is interpreted to mean that there should be no periodic retrieval.\n\n* `:api_module` identifies the module that does the retrieval of exchange rates. This is any module that implements the `Money.ExchangeRates` behaviour.  The  default is `Money.ExchangeRates.OpenExchangeRates`.\n\n* `:exchange_rates_cache_module` defines the module that provides an exchange rates cache.  Any module that implements the `Money.ExchangeRates.Cache` behaviour.  Two alternative strategies are provided:\n\n  * `Money.ExchangeRates.Cache.Ets` which is also the default.\n  * `Money.ExchangeRates.Cache.Dets`\n\n* `:preload_historic_rates` defines a date or a date range that will be requested when the exchange rate service starts up. The date or date range should be specified as either a `Date.t` or a `Date.Range.t` or a tuple of `{Date.t, Date.t}` representing the `from` and `to` dates for the rates to be retrieved. The default is `nil` meaning no historic rates are preloaded. See [Preloading historic rates](#preloading-historic-rates) for more information.\n\n* `callback_module` defines a module that follows the `Money.ExchangeRates.Callback` behaviour whereby the function `rates_retrieved/2` is invoked after every successful retrieval of exchange rates.  The default is `Money.ExchangeRates.Callback`.\n\n* `log_failure` defines the log level at which api retrieval errors are logged.  The default is `:warning`.\n\n* `log_success` defines the log level at which successful api retrieval notifications are logged.  The default is `nil` which means no logging.\n\n* `log_info` defines the log level at which service startup messages are logged.  The default is `info`.\n\n* `:retriever_options` is available for exchange rate retriever module developers as a place to add retriever-specific configuration information.  This information should be added in the `init/1` callback in the retriever module.  See `Money.ExchangeRates.OpenExchangeRates.init/1` for an example.\n\n* `:json_library` determines which json library to be used for decoding.  Two common options are `Poison` and `Jason`. The default is `Cldr.Config.json_library/0` which will attempt to use `Json` (built in as of Elixir 1.18) or `:json` (built into OTP as of OTP 27).\n\n* `:exclude_protocol_implementations` is a protocol module, or list of protocol modules, that will not be defined by `ex_money`. The default is `[]`. The protocol implementations influenced by this option at `Json.Encoder`, `Phoenix.HTML.Safe` and `Gringotts.Money`.","ref":"readme.html#configuration-key-definitions"},{"type":"extras","title":"JSON library configuration - Introduction to Money","doc":"Note that `ex_money` does not define a json library dependency and therefore it is the users responsibility to configure the required json library as a dependency in the application's `mix.exs`.\n\nThe recommended library is [jason](https://hex.pm/packages/jason) which would be configured as:\n```\n  defp deps do\n    [\n      {:jason, \"~> 1.0\"},\n      ...\n    ]\n  end\n```\n`ex_money` depends on `ex_cldr` which provides currency and localisation data. The default configuration of `ex_money` uses the default `json_library` from `ex_cldr`.  This can be configured as follows in `config.exs`:\n```\nconfig :ex_cldr,\n  json_library: Jason\n```\nIn most cases this is not required since the presence of `Jason` (or `Poison`) is automatic.","ref":"readme.html#json-library-configuration"},{"type":"extras","title":"Configuring locales to support localised formatting - Introduction to Money","doc":"`Money` uses [ex_cldr](https://hex.pm/packages/ex_cldr) and [ex_cldr_numbers](https://hex.pm/packages/ex_cldr_numbers) to support configuring locales and providing locale formatting.  These packages are also the source of currency definitions, names, formats and so on.\n\nTo use `Cldr` and therefore `Money`, a backend module must be defined.  This module will host the `Cldr` data and public API used by `Money`.  A simple example would be:\n```\ndefmodule MyApp.Cldr do\n  use Cldr,\n    locales: [\"en\", \"fr\", \"zh\"],\n    default_locale: \"en\",\n    providers: [Cldr.Number, Money]\nend\n```","ref":"readme.html#configuring-locales-to-support-localised-formatting"},{"type":"extras","title":"Preloading historic rates - Introduction to Money","doc":"The current implementation will call the api_module to retrieve the historic rates once for each date in the `:preload_historic_rates` range.  Some exchange rate services, like Open Exchange Rates, provides a bulk retrieval api that can retrieve multiple dates in a single call.  However this endpoint is only available for premium subscribers and it is still charged on a \"per date retrieved\" basis. So while there is a network/performance/efficiency benefit there is no economic benefit.  Please file an issue on [github](https://github.com/kipcole9/money) if implementing a bulk api is important to you.\n\nSome examples of configuring the `:preload_historic_rates` key follow:\n\n  * `preload_historic_rates: ~D[2017-01-01]`\n  * `preload_historic_rates: Date.range(~D[2017-01-01], ~D[2017-10-01])`\n  * `preload_historic_rates: {~D[2017-01-01], ~D[2017-10-01]}`","ref":"readme.html#preloading-historic-rates"},{"type":"extras","title":"Open Exchange Rates configuration - Introduction to Money","doc":"If you plan to use the provided Open Exchange Rates module to retrieve exchange rates then you should also provide the addition configuration key for `app_id`:\n\n      config :ex_money,\n        open_exchange_rates_app_id: \"your_app_id\"\n\n  or configure it via environment variable, for example:\n\n      config :ex_money,\n        open_exchange_rates_app_id: {:system, \"OPEN_EXCHANGE_RATES_APP_ID\"}\n\nThe default exchange rate retrieval module is provided in `Money.ExchangeRates.OpenExchangeRates` which can be used as a example to implement your own retrieval module for  other services.","ref":"readme.html#open-exchange-rates-configuration"},{"type":"extras","title":"Managing the configuration at runtime - Introduction to Money","doc":"During exchange rate service startup, the function `init/1` is called on the configured exchange rate retrieval module.  This module is expected to return an updated configuration allowing a developer to customise how the configuration is to be managed.  See the implementation at `Money.ExchangeRates.OpenExchangeRates.init/1` for an example.\n\nTo support runtime (re-)configuration the following functions are provided:\n\n  * `Money.ExchangeRates.Retriever.config/0` returns the current configuration of the exchange rates retrieval service.\n\n  * `Money.ExchangeRates.Retriever.stop/0` and `Money.ExchangeRates.Retriever.start/0` stop and start the exchange rates retrieval service respectively.\n\n  * `Money.ExchangeRates.Retriever.reconfigure/1` reconfigures the exchange rates retrieval service.  It does not restart the service, the service remains active during the recongiguration.","ref":"readme.html#managing-the-configuration-at-runtime"},{"type":"extras","title":"Using Environment Variables in the configuration - Introduction to Money","doc":"Keys can also be configured to retrieve values from environment variables.  This lookup is done at runtime to facilitate deployment strategies.  If the value of a configuration key is `{:system, \"some_string\"}` then `\"some_string\"` is interpreted as an environment variable name which is passed to `System.get_env/2`.  An example configuration might be:\n\n    config :ex_money,\n      auto_start_exchange_rate_service: {:system, \"RATE_SERVICE\"},\n      exchange_rates_retrieve_every: {:system, \"RETRIEVE_EVERY\"},\n      open_exchange_rates_app_id: {:system, \"OPEN_EXCHANGE_RATES_APP_ID\"}\n\nNote that the `{:system, \"ENV KEY\"}` approach is **not** currently supported for the `:preload_historic_rates` configuration key.","ref":"readme.html#using-environment-variables-in-the-configuration"},{"type":"extras","title":"The Exchange rates service process supervision and startup - Introduction to Money","doc":"If the exchange rate service is configured to automatically start up (because the config key `auto_start_exchange_rate_service` is set to `true`) then a supervisor process named `Money.ExchangeRates.Supervisor` is started which in turns starts a child `GenServer` called `Money.ExchangeRates.Retriever`.  It is `Money.ExchangeRates.Retriever` which will call the configured `api_module` to retrieve the rates.  It is also responsible for calling the configured `callback_module` after a successfull retrieval.\n\n                                         +-----------------+\n                                         |                 |\n    +-------------+    +-----------+     |   api_module    |-> External Service\n    |             |    |           |---> |                 |\n    | Supervisor  |--->| Retriever |     +-----------------+\n    |             |    |           |---> +-----------------+\n    +-------------+    +-----------+     |                 |\n                                         | callback_module |\n                                         |                 |\n                                         +-----------------+\n\nOn application start (or manual start if `:auto_start_exchange_rate_service` is set to `false`), `Money.ExchangeRates.Retriever` will schedule the first retrieval to be executed after immediately and then each `:exchange_rates_retrieve_every` milliseconds thereafter.","ref":"readme.html#the-exchange-rates-service-process-supervision-and-startup"},{"type":"extras","title":"Using Ecto or other applications from within the callback module - Introduction to Money","doc":"If you provide your own callback module and that module depends on some other applications, like `Ecto`, already being started then automatically starting `Money.ExchangeRates.Supervisor` may not work since your `Ecto.Repo` is unlikely to have already been started.\n\nIn this situation the appropriate way to configure the exchange rates retrieval service is the following:\n\n1. Set the configuration key `auto_start_exchange_rate_service` to `false` to prevent automatic startup of the service.\n\n2. Configure your `api_module`, `callback_module` and any other required configuration as appropriate\n\n3. In your client application code, add the `Money.ExchangeRates.Supervisor` to the `children` configuration of your application.  For example, in an application that uses `Ecto` and where your `callback_module` is designed to save exchange rates to a database, your application may would look something like:\n\n```elixir\ndefmodule Application do\n  use Application\n\n  def start(_type, _args) do\n    import Supervisor.Spec\n\n    children = [\n\n      # Start your repo first so that it is running before your\n      # exchange rates callback module is called\n      supervisor(MoneyTest.Repo, []),\n\n      # Include the Money.ExchangeRates.Supervisor in your application's\n      # supervision tree.  This supervisor will start the child process\n      # Money.ExchangeRates.Retriever.\n\n      # Note the use of double `[]` around\n      # the parameters which are required to ensure that the supervisor\n      # is stopped before including in your supervisor tree.\n      # The `start_retriever: true` is optional.  The default value is `false`.\n      supervisor(Money.ExchangeRates.Supervisor, [[restart: true, start_retriever: true]])\n    ]\n\n    opts = [strategy: :one_for_one, name: Application.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"readme.html#using-ecto-or-other-applications-from-within-the-callback-module"},{"type":"extras","title":"API Usage Examples - Introduction to Money","doc":"","ref":"readme.html#api-usage-examples"},{"type":"extras","title":"Creating a %Money{} struct - Introduction to Money","doc":"iex> Money.new(:USD, 100)\n    #Money<:USD, 100>\n\n    iex> Money.new(100, :USD)\n    #Money<:USD, 100>\n\n    iex> Money.new(\"CHF\", \"130.02\")\n    #Money<:CHF, 130.02>\n\n    iex> Money.new(\"thb\", 11)\n    #Money<:THB, 11>\n\n    iex> Money.new(\"1.000,99\", :EUR, locale: \"de\")\n    #Money<:EUR, 1000.99>\n\n    iex> Money.parse(\"USD 100\")\n    #Money<:USD, 100>\n\n    iex> Money.parse(\"USD 100,00\", locale: \"de\")\n    #Money<:USD, 100.00>\n\nThe canonical representation of a currency code is an `atom` that is a valid\n[ISO4217](http://www.currency-iso.org/en/home/tables/table-a1.html) currency code or a `t:String.t/0` representation of an [ISO 24165](https://www.iso.org/standard/80601.html) digital token identifier. The amount of a `%Money{}` is represented by a `Decimal`.\n\nNote that the amount and currency code arguments to `Money.new/3` can be supplied in either order.","ref":"readme.html#creating-a-money-struct"},{"type":"extras","title":"Parsing money strings - Introduction to Money","doc":"`Money` provides an ability to parse strings that contain a currency and an amount.  The currency can be represented in different ways depending on the locale. See `Money.parse/2` for further information.  Some examples are:\n\n```\n  # These are the strings available for a given currency\n  # and locale that are recognised during parsing\n  iex> Cldr.Currency.strings_for_currency :AUD, \"de\"\n  [\"aud\", \"au$\", \"australischer dollar\", \"australische dollar\"]\n\n  iex> Money.parse \"$au 12 346\", locale: \"fr\"\n  #Money<:AUD, 12346>\n\n  iex> Money.parse \"12 346 dollar australien\", locale: \"fr\"\n  #Money<:AUD, 12346>\n\n  iex> Money.parse \"A$ 12346\", locale: \"en\"\n  #Money<:AUD, 12346>\n\n  iex> Money.parse \"australian dollar 12346.45\", locale: \"en\"\n  #Money<:AUD, 12346.45>\n\n  # Parse using a default currency\n  iex> Money.parse(\"100\", default_currency: :EUR)\n  #Money<:EUR, 100>\n\n  # Parse using the default currency of the locale\n\t# If no `:locale` option is provided then\n\t# the locale associated with `Money.default_backend/0`\n\t# is used.\n  iex> Money.parse(\"100\", locale: \"en\")\n  #Money<:EUR, 100>\n\n  # Parse with a default currency for the current locale\n  iex> Money.parse(\"100\", default_currency: Money.default_currency_for_locale())\n  #Money<:USD, 100>\n\n  # Note that the decimal separator in the \"de\" locale\n  # is a `.`\n  iex> Money.parse \"AU$ 12346,45\", locale: \"de\"\n  #Money<:AUD, 12346.45>\n\n  # Round trip formatting is supported\n  iex> {:ok, string} = Cldr.Number.to_string 1234, Money.Cldr, currency: :AUD\n  {:ok, \"A$1,234.00\"}\n  iex> Money.parse string\n  #Money<:AUD, 1234.00>\n\n  # Fuzzy matching is possible\n  iex> Money.parse(\"100 eurosports\", fuzzy: 0.8)\n  #Money<:EUR, 100>\n\n  iex> Money.parse(\"100 eurosports\", fuzzy: 0.9)\n  {:error,\n   {Money.Invalid, \"Unable to create money from \\\"eurosports\\\" and \\\"100\\\"\"}}\n\n  # Eligible currencies can be filtered\n  iex> Money.parse(\"100 eurosports\", fuzzy: 0.8, only: [:current])\n  #Money<:EUR, 100>\n\n  iex> Money.parse(\"100 euro\", only: [:EUR, :USD, :COP])\n  #Money<:EUR, 100>\n\n  iex> Money.parse(\"100 euro\", except: [:EUR, :USD, :COP])\n  {:error,\n    {Money.UnknownCurrencyError,\n      \"The currency \\\"euro\\\" is unknown or not supported\"}}\n\n  iex> Money.parse \"100 afghan afghanis\"\n  #Money<:AFN, 100>\n\n  iex> Money.parse \"100 afa\", only: [:current]\n  {:error,\n   {Money.UnknownCurrencyError,\n    \"The currency \\\"afa\\\" is unknown or not supported\"}}\n```","ref":"readme.html#parsing-money-strings"},{"type":"extras","title":"Casting a money type (basic support for HTML forms) - Introduction to Money","doc":"`Money` supports form field inputs that are a single string combining both a currency code and an amount.  When a form field (or other data) is [cast](https://hexdocs.pm/ecto/Ecto.Changeset.html#cast/4) then `Money` will attempt to parse a string field into a `Money.t` using `Money.parse/2`. Therefore simple money form input can be supported with a single input field of `type=text`.\n\nNote that when parsing the input text, the amount is interpreted in the context of the current locale set on the default backend configured for `ex_money`.  This affects how separator characters are interpreted in exactly the same way as is done for `Money.new/3`.","ref":"readme.html#casting-a-money-type-basic-support-for-html-forms"},{"type":"extras","title":"Float amounts cannot be provided to `Money.new/2` - Introduction to Money","doc":"Float have well-known issues in computing due to issues of rounding and potential precision loss.  Internally `Money` uses `Decimal` to store the amount which allows arbitrary precision arithmetic.  `Money` also uses the `numeric` type in Postgres to preserve precision and even goes to far as to store the amount as a `string` in `MySQL` for the same reason.\n\nTherefore an error is returned if an attempt is made to use `Money.new/2` with a float amount:\n\n    {:error,\n     {Money.InvalidAmountError,\n      \"Float amounts are not supported in new/2 due to potenial rounding \" <>\n        \"and precision issues.  If absolutely required, use Money.from_float/2\"}}\n\nIf the use of `float`s is require then the function `Money.from_float/2` is provided with the same arguments as those for `Money.new/2`.  `Money.from_float/2` provides an addition check and will return an error if the precision (number of digits) of the provided float is more than 15 (the number of digits guaranteed to round-trip between a 64-bit float and a string).","ref":"readme.html#float-amounts-cannot-be-provided-to-money-new-2"},{"type":"extras","title":"Comparison functions - Introduction to Money","doc":"`Money` values can be compared as long as they have the same currency. The recommended function is `Money.compare/2` which, given two compatible money amounts, will return `:lt`, `:eq` or `:gt` depending on the relationship. For example:\n```\niex> Money.compare Money.new(:USD, 100), Money.new(:USD, 200)\n:lt\n\niex> Money.compare Money.new(:USD, 100), Money.new(:AUD, 200)\n{:error,\n {ArgumentError,\n  \"Cannot compare monies with different currencies. Received :USD and :AUD.\"}}\n```\n\nFrom Elixir verison `1.10.0` onwards, several functions in the `Enum` module can use the `Money.compare/2` function to simplify sorting. For example:\n```\niex> list = [Money.new(:USD, 100), Money.new(:USD, 200)]\n[#Money<:USD, 100>, #Money<:USD, 200>]\niex> Enum.sort list, Money\n[#Money<:USD, 100>, #Money<:USD, 200>]\niex> Enum.sort list, {:asc, Money}\n[#Money<:USD, 100>, #Money<:USD, 200>]\niex> Enum.sort list, {:desc, Money}\n[#Money<:USD, 200>, #Money<:USD, 100>]\n```\n\n**Note that `Enum.sort/2` will sort money amounts even when the currencies are incompatible. In this case the order of the result is not predictable. It is the developers responsibility to filter the list to compatible currencies prior to sorting. This is a limitation of the `Enum.sort/2` implementation.**","ref":"readme.html#comparison-functions"},{"type":"extras","title":"Optional ~M sigil - Introduction to Money","doc":"An optional sigil module is available to aid in creating %Money{} structs.  It needs to be imported before use:\n\n    import Money.Sigil\n\n    ~M[100]USD\n    #> #Money<:USD, 100>","ref":"readme.html#optional-m-sigil"},{"type":"extras","title":"Localised Money formatting - Introduction to Money","doc":"`Money` provides locale-specific formatted output that is controlled be either the locale that has been set for this process or by the `:locale` parameter supplied to `Money.to_string/2`.  Configuring your localised environment requires configuring `ex_cldr` which is a dependency to `Money`.  See the [Configuration](https://github.com/kipcole9/cldr#configuration) section of the `ex_cldr` readme for more information.\n\nThe main API for formatting `Money` is `Money.to_string/2`. Additionally formatting options are passed to `Cldr.Number.to_string/2`.  Those options are described in the [readme for ex_cldr_numbers](https://github.com/kipcole9/cldr_numbers#primary-public-api) which is also a dependency to `Money`.\n\n    iex> Money.to_string Money.new(\"thb\", 11)\n    {:ok, \"THB11.00\"}\n\n    # The default locale is \"en-001\" which is\n    # \"global english\"\n    iex> Money.to_string Money.new(\"USD\", \"234.467\")\n    {:ok, \"$US234.47\"}\n\n    # The locale \"en\" is \"American English\".  For\n    # UK English use the locale \"en-GB\".  Australian\n    # English is \"en-AU\" and so on.\n    iex> Money.to_string Money.new(\"USD\", \"234.467\"), locale: \"en\"\n    {:ok, \"$234.47\"}\n\n    iex> Money.to_string Money.new(\"USD\", \"234.467\"), format: :long\n    {:ok, \"234.47 US dollars\"}\n\n    iex> Money.to_string Money.new(\"USD\", \"234.467\"), locale: \"fr\"\n    {:ok, \"234,47 $US\"}\n\n    iex> Money.to_string Money.new(\"USD\", \"234.467\"), locale: \"de\"\n    {:ok, \"234,47 $\"}\n\n    iex> Money.to_string Money.new(\"EUR\", \"234.467\"), locale: \"de\"\n    {:ok, \"234,47 €\"}\n\n    iex> Money.to_string Money.new(\"EUR\", \"234.467\"), locale: \"fr\"\n    {:ok, \"234,47 €\"}\n\n**Note that the output is influenced by the locale in effect.**  By default the locale used is that returned by `Cldr.get_locale/0`.  Its default value is `:en-001`.  Additional locales can be configured, see `Cldr`.  The formatting options are defined in `Cldr.Number.to_string/2`.","ref":"readme.html#localised-money-formatting"},{"type":"extras","title":"Arithmetic Functions - Introduction to Money","doc":"See also the module `Money.Arithmetic`:\n\n    iex> m1 = Money.new(:USD, 100)\n    #Money<:USD, 100>}\n\n    iex> m2 = Money.new(:USD, 200)\n    #Money<:USD, 200>}\n\n    iex> Money.add(m1, m2)\n    {:ok, #Money<:USD, 300>}\n\n    iex> Money.add!(m1, m2)\n    #Money<:USD, 300>\n\n    iex> m3 = Money.new(:AUD, 300)\n    #Money<:AUD, 300>\n\n    iex> Money.add Money.new(:USD, 200), Money.new(:AUD, 100)\n    {:error, {ArgumentError, \"Cannot add monies with different currencies. Received :USD and :AUD.\"}}\n\n    # Split a %Money{} returning the a dividend and a remainder. All\n    # operations respect the number of fractional digits defined for a currency\n    iex> m1 = Money.new(:USD, 100)\n    #Money<:USD, 100>\n\n    iex> Money.split(m1, 3)\n    {#Money<:USD, 33.33>, #Money<:USD, 0.01>}\n\n    # Rounding applies the currency definitions of CLDR as implemented in\n    # the hex package [ex_cldr](https://hex.pm/packages/ex_cldr)\n    iex> Money.round Money.new(:USD, \"100.678\")\n    #Money<:USD, 100.68>\n\n    iex> Money.round Money.new(:JPY, \"100.678\")\n    #Money<:JPY, 101>","ref":"readme.html#arithmetic-functions"},{"type":"extras","title":"Currency Conversion - Introduction to Money","doc":"A `%Money{}` struct can be converted to another currency using `Money.to_currency/3` or `Money.to_currency!/3`.  For example:\n\n    iex> Money.to_currency Money.new(:USD, 100), :AUD\n    {:ok, #Money<:AUD, 136.43>}\n\n    iex> Money.to_currency Money.new(:USD, 100), :AUD, ExchangeRates.historic_rates(~D[2017-01-01])\n    {:ok, #Money<:AUD, 128.76>}\n\n    iex> Money.to_currency Money.new(:USD, 100) , :AUDD, %{USD: Decimal.new(1), AUD: Decimal.new(0.7345)}\n    {:error, {Cldr.UnknownCurrencyError, \"Currency :AUDD is not known\"}}\n\n    iex> Money.to_currency! Money.new(:USD, 100), :XXX\n    ** (Money.ExchangeRateError) No exchange rate is available for currency :XXX\n\nA user-defined map of exchange rates can also be supplied:\n\n    iex> Money.to_currency Money.new(:USD, 100), :AUD, %{USD: Decimal.new(1.0), AUD: Decimal.new(1.3)}\n    #Money<:AUD, 130>","ref":"readme.html#currency-conversion"},{"type":"extras","title":"Historic Conversion Rates - Introduction to Money","doc":"As noted in the [configuration](#configuration) section, `ex_money` can preload historic exchange rates when the exchange rates service starts up.  It can be anticipated that additional historic rates may be required subsequently.\n\n* `Money.ExchangeRates.Retriever.historic_rates/1` can be called to request retrieval of historic rates at any time.  This call will send a message to the retrieval service to request retrieval.  It does not return the rates.\n\n* `Money.ExchangeRates.historic_rates/1` is the partner function to `Money.ExchangeRates.latest_rates/0`.  It returns the exchange rates for a given date, and will return an error if no rates are available.","ref":"readme.html#historic-conversion-rates"},{"type":"extras","title":"Financial Functions - Introduction to Money","doc":"A set of basic financial functions are available in the module `Money.Financial`.   These functions are:\n\n* Present value: `Money.Financial.present_value/3`\n* Future value: `Money.Financial.future_value/3`\n* Interest rate: `Money.Financial.interest_rate/3`\n* Number of periods: `Money.Financial.periods/3`\n* Payment amount: `Money.Financial.payment/3`\n* Net Present Value of a set of cash flows: `Money.Financial.net_present_value/2`\n* Internal rate of return: `Money.Financial.internal_rate_of_return/1`\n\nFor more detail see `Money.Financial`.","ref":"readme.html#financial-functions"},{"type":"extras","title":"Subscriptions - Introduction to Money","doc":"Subscriptions, especially in the context of a SaaS, can involve changing plans - either from a smaller plan to a larger or a larger plan to smaller.  In either situation a credit amount needs to be calculated based upon the current plan which is then applied to the new plan.  `Money.Subscription` is a module that provides functions to support this subscription pricing, credit calculations and payment dates.\n\nThe primary functions supporting subscriptions are:\n\n* Create a new subscription: `Money.Subscription.new/3`\n* Create a subscription plan: `Money.Subscription.Plan.new/3`\n* Change a from one plan to another: `Money.Subscription.change_plan/3`\n* Calculate the start date for the next interval of a plan: `Money.Subscription.next_interval_starts/3`\n* Calculate the number of days in a plan interval: `Money.Subscription.plan_days/3`\n* Calculate the number of days left in a plan interval: `Money.Subscription.days_remaining/4`","ref":"readme.html#subscriptions"},{"type":"extras","title":"Examples - Introduction to Money","doc":"# Create the current plan\n    iex> current_plan = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 1)\n    %Money.Subscription.Plan{\n      interval: :month,\n      interval_count: 1,\n      price: #Money<:USD, 10>\n    }\n\n    # How many days in a billing period?\n    iex> Money.Subscription.plan_days current_plan, ~D[2018-03-01]\n    31\n\n    iex> Money.Subscription.plan_days current_plan, ~D[2018-02-01]\n    28\n\n    # How many days remaining in the current billing period\n    iex> Money.Subscription.days_remaining current_plan, ~D[2018-03-01], ~D[2018-03-10]\n    22\n\n    # When is the next billing date\n    iex> Money.Subscription.next_interval_starts current_plan, ~D[2018-03-01]\n    ~D[2018-04-01]\n\n    # Create a new plan\n    iex> new_plan = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 3)\n    %Money.Subscription.Plan{\n      interval: :month,\n      interval_count: 3,\n      price: #Money<:USD, 10>\n    }\n\n    # Change plans at the end of the current billing period\n    iex> Money.Subscription.change_plan current_plan, new_plan, current_interval_started: ~D[2018-03-01]\n    %Money.Subscription.Change{\n      carry_forward: #Money<:USD, 0>,\n      credit_amount: #Money<:USD, 0>,\n      credit_amount_applied: #Money<:USD, 0>,\n      credit_days_applied: 0,\n      credit_period_ends: nil,\n      first_billing_amount: #Money<:USD, 10>,\n      first_interval_starts: ~D[2018-04-01],\n      next_interval_starts: ~D[2018-07-01]\n    }\n\n    # Change plans in the middle of the current plan period\n    # and credit the balance of the current plan to the new plan\n    iex> Money.Subscription.change_plan current_plan, new_plan, current_interval_started: ~D[2018-03-01], effective: ~D[2018-03-15]\n    %Money.Subscription.Change{\n      carry_forward: #Money<:USD, 0>,\n      credit_amount: #Money<:USD, 5.49>,\n      credit_amount_applied: #Money<:USD, 5.49>,\n      credit_days_applied: 0,\n      credit_period_ends: nil,\n      first_billing_amount: #Money<:USD, 4.51>,\n      first_interval_starts: ~D[2018-03-15],\n      next_interval_starts: ~D[2018-06-15]\n    }\n\n    # Change plans in the middle of the current plan period\n    # but instead of a monetary credit, apply the credit as\n    # extra days on the new plan in the first billing period\n    iex> Money.Subscription.change_plan current_plan, new_plan, current_interval_started: ~D[2018-03-01], effective: ~D[2018-03-15], prorate: :period\n    %Money.Subscription.Change{\n      carry_forward: #Money<:USD, 0>,\n      credit_amount: #Money<:USD, 5.49>,\n      credit_amount_applied: #Money<:USD, 0>,\n      credit_days_applied: 51,\n      credit_period_ends: ~D[2018-05-04],\n      first_billing_amount: #Money<:USD, 10>,\n      first_interval_starts: ~D[2018-03-15],\n      next_interval_starts: ~D[2018-08-05]\n    }\n\n    # Create a subscription\n    iex> plan = Money.Subscription.Plan.new!(Money.new(:USD, 200), :month, 3)\n    iex> subscription = Money.Subscription.new! plan, ~D[2018-01-01]\n    %Money.Subscription{\n      created_at: #DateTime<2018-03-23 07:45:44.418916Z>,\n      id: nil,\n      plans: [\n        {%Money.Subscription.Change{\n           carry_forward: #Money<:USD, 0>,\n           credit_amount: #Money<:USD, 0>,\n           credit_amount_applied: #Money<:USD, 0>,\n           credit_days_applied: 0,\n           credit_period_ends: nil,\n           first_billing_amount: #Money<:USD, 200>,\n           first_interval_starts: ~D[2018-01-01],\n           next_interval_starts: ~D[2018-04-01]\n         },\n         %Money.Subscription.Plan{\n           interval: :month,\n           interval_count: 3,\n           price: #Money<:USD, 200>\n         }}\n      ]\n    }\n\n    # Change a subscription's plan\n    iex> new_plan = Money.Subscription.Plan.new!(Money.new(:USD, 150), :day, 30)\n    iex> Money.Subscription.change_plan! subscription, new_plan\n    %Money.Subscription{\n      created_at: #DateTime<2018-03-23 07:47:48.593973Z>,\n      id: nil,\n      plans: [\n        {%Money.Subscription.Change{\n           carry_forward: #Money<:USD, 0>,\n           credit_amount: #Money<:USD, 0>,\n           credit_amount_applied: #Money<:USD, 0>,\n           credit_days_applied: 0,\n           credit_period_ends: nil,\n           first_billing_amount: #Money<:USD, 150>,\n           first_interval_starts: ~D[2018-04-01],\n           next_interval_starts: ~D[2018-05-01]\n         },\n         %Money.Subscription.Plan{\n           interval: :day,\n           interval_count: 30,\n           price: #Money<:USD, 150>\n         }},\n        {%Money.Subscription.Change{\n           carry_forward: #Money<:USD, 0>,\n           credit_amount: #Money<:USD, 0>,\n           credit_amount_applied: #Money<:USD, 0>,\n           credit_days_applied: 0,\n           credit_period_ends: nil,\n           first_billing_amount: #Money<:USD, 200>,\n           first_interval_starts: ~D[2018-01-01],\n           next_interval_starts: ~D[2018-04-01]\n         },\n         %Money.Subscription.Plan{\n           interval: :month,\n           interval_count: 3,\n           price: #Money<:USD, 200>\n         }}\n      ]\n    }","ref":"readme.html#examples"},{"type":"extras","title":"Serializing to a database with Ecto - Introduction to Money","doc":"The companion package [ex_money_sql](https://hex.pm/packages/ex_money_sql) provides functions for the serialization of `Money` data.  See the [README](https://hexdocs.pm/ex_money_sql/readme.html) for further information.","ref":"readme.html#serializing-to-a-database-with-ecto"},{"type":"extras","title":"Installation - Introduction to Money","doc":"`Money` can be installed by adding `ex_money` to your list of dependencies in `mix.exs` and then executing `mix deps.get`\n\n```elixir\ndef deps do\n  [\n    {:ex_money, \"~> 5.0\"},\n    ...\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Why yet another Money package? - Introduction to Money","doc":"* Fully localized formatting and rounding using [ex_cldr](https://hex.pm/packages/ex_cldr)\n\n* Provides serialization to Postgres using a composite type and MySQL using a JSON type that keeps both the currency code and the amount together removing a source of potential error\n\n* Uses the `Decimal` type in Elixir and the Postgres `numeric` type to preserve precision.  For MySQL the amount is serialised as a string to preserve precision that might otherwise be lost if stored as a JSON numeric type (which is either an integer or a float)\n\n* Includes a set of financial calculations (arithmetic and cash flow calculations) that follow solid rounding rules","ref":"readme.html#why-yet-another-money-package"},{"type":"extras","title":"Falsehoods programmers believe about prices - Introduction to Money","doc":"The [github gist](https://gist.github.com/rgs/6509585) gives a good summary of the challenges of managing money in an application. The following described how `Money` handles each of these assertions.\n\n**1. You can store a price in a floating point variable.**\n\n`Money` operates and serialises in a arbitrary precision Decimal value.\n\n**2. All currencies are subdivided in 1/100th units (like US dollar/cents, euro/eurocents etc.).**\n\n`Money` leverages CLDR which defines the appropriate number of decimal places of a currency. As of CLDR version 32 there are:\n\n  * 52 currencies with zero decimal digits\n  * 241 currencies with two decimal digits\n  * 6 currencies with three decimal digits\n  * and 1 currency with four decimal digits\n\n**3. All currencies are subdivided in decimal units (like dinar/fils)**\n\n**4. All currencies currently in circulation are subdivided in decimal units. (to exclude shillings, pennies) (counter-example: MGA)**\n\n**5. All currencies are subdivided. (counter-examples: KRW, COP, JPY... Or subdivisions can be deprecated.)**\n\n`Money` correctly manages the appropriate number of decimal places for a currency.  It also round correctly when formatting a currency for output (different currencies have different rounding levels for cash or transactions).\n\n**6. Prices can't have more precision than the smaller sub-unit of the currency. (e.g. gas prices)**\n\nAll `Money` calculations are done with decimal arithmetic to the maxium precision of 28 decimal digits.\n\n**7. For any currency you can have a price of 1. (ZWL)**\n\n`Money` makes no assumption about the value assigned as long as its a real number.\n\n**8. Every country has its own currency. (EUR is the best example, but also Franc CFA, etc.)**\n\n`Money` makes no assumptions about the linkage of currencies to territories.\n\n**9. No country uses another's country official currency as its official currency. (many countries use USD: Ecuador, Micronesia...)**\n\n**10. Countries have only one currency.**\n\n`Money` doesn't link territories (countries) to a currency - it focuses only on the `Money` domain.  The addon package [cldr_territories](https://github.com/Schultzer/cldr_territories) does have knowledge of what curriencies are in effect throughout history for a given territory.  See `Cldr.Territory.info/1`.\n\n**11. Countries have only one currency currently in circulation. (Panama officially uses both PAB and USD)**\n\n`Money` makes no assumptions about currencies in circulation.\n\n**12. I'll only deal with currencies currently in circulation anyway.**\n\n`Money` makes no assumptions about currencies in circulation.\n\n**13. All currencies have an ISO 4217 3-letter code. (The Transnistrian ruble has none, for example)**\n\n`Money` does validate currency codes against the ISO 4217 list.  Custom currencies can be created in accordance with ISO 4217 using `Cldr.Currency.new/2`.\n\n**14. All currencies have a different name. (French franc, \"nouveau franc\")**\n\n`Money` has localised names for all ISO 4217 currencies in each of the over 500 locales defined by CLDR.\n\n**15. You always put the currency symbol after the price.**\n\n`Money` formats currency strings according to a format mask that is either defined by CLDR or user supplied.\n\n**16. You always put the currency symbol before the price.**\n\n`Money` formats currency strings according to a format mask that is either defined by CLDR or user supplied.\n\n**17. You always put the currency symbol either after, or before the price, never in the middle.**\n\n`Money` formats currency strings according to a format mask that is either defined by CLDR or user supplied.\n\n**18. There's only one currency symbol for any currency. (元, 角, 分 are increasing units of the Chinese renminbi.)**\n\n`Money` uses format masks defined by CLDR which, for the Chinese renminbi uses the \"￥\" symbol.\n\n**19. For a given currency, you always, but always, put the symbol in the same place.**\n\n`Money` makes no assumpions about symbol placement.  The symbol can be places anywhere in a formatted string but is typically, for CLDR format masks, placed either before or after the formatted number.\n\n**20. OK. But if you only use the ISO 4217 currency codes, you always put it before the price. (Hint: it depends on the language.)**\n\nSame as for the answer to 19 above.\n\n**21. Before the price means on the left. (ILS)**\n\n`Money` formats according to a locale and correctly places symbols for languages written right-to-left.\n\n**22. You can always use a dot (or a comma, etc.) as a decimal separator.**\n\nThe decimal separator is defined per locale according to the CLDR definitions.\n\n**23. You can always use a space (or a dot, or a comma, etc.) as a thousands separator.**\n\nThe thousands (acutally grouping since not all locales format in thousands) separator is defined per locale according to the CLDR definitions.\n\n**24. You separate big prices by grouping numbers in triplets (thousands). (One writes ¥1 0000)**\n\nGrouping is done according the CLDR definitions.  For many languages the grouping is in thousands.  Some format other ways.  For example in India numbers are formatted with the first group as a triplet and subsequent groups as doublets.\n\n**25. Prices at a single company will never range from five digits before the decimal to five digits after.**\n\n`Money`'s default precision is 28 decimal digits.  All arithmetic is done using arbitrary precision decimal arithemetic.  No round is performed unless either explicitly requested or a money value is formatted for output.  When formatting rounding is applied according the locale-specific rules.\n\n**26. Prices contains only digits and punctuation. (Germans can write 12,- €)**\n\n`Money` format masks can contain very flexible formatting masks.  A set of formats is defined for each locale and a user-defined masks can also be defined.\n\n**27. A price can be at most 10^N for some value of N.**\n\nSee the answer to 25.\n\n**28. Given two currencies, there is only one exchange rate between them at any given point in time.**\n\n`Money` supports an exchange rate mechansim, currency conversions and retrieval from external exchange rate services.  It does not impose any constraint on underlying conversion tables.\n\n**29. Given two currencies, there is at least one exchange rate between them at any given point in time. (restriction on export of MAD, ARS, CNY, for example)**\n\nSee the answer to 28.\n\n**30. And the final one: a standalone $ character is always pronounced dollar. (It's also the peso sign.)**\n\nThis is outside the domain of `Money`.","ref":"readme.html#falsehoods-programmers-believe-about-prices"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n**Note** `ex_money` 5.17.0 and later is supported on Elixir 1.12 and later versions only.","ref":"changelog.html"},{"type":"extras","title":"Money v5.20.0 - Changelog","doc":"This is the changelog for Money v5.20.0 released on March 18th, 2025.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-20-0"},{"type":"extras","title":"Soft Deprecation - Changelog","doc":"* The option `:fractional_digits` for `Money.from_integer/3` has been deprecated in favour of `:currency_digits` for consistency with `Money.round/2` and functions in [ex_cldr_numbers](https://hex.pm/packages/ex_cldr_numbers). `:fractional_digits` will continue to be supported for an indefinite period.","ref":"changelog.html#soft-deprecation"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Ensure `Money.split/3` always returns a remainder that is greater than or equal to 0. Thanks to @jdewar for the report and @coladarci, @Wigny for their collaboration. Closes #173.\n\n* Allow a [non-breaking-space](https://en.wikipedia.org/wiki/Non-breaking_space) to be part of a number string. Some locales (like `en-ZA`) will format numbers with a nbsp when using standard separators for that locale.\n\n* Fix documentation for `Money.ExchangeRates` replacing `:exchange_rate_service` with `:auto_start_exchange_rate_service`. Thanks to @cw789 for the PR. Closes #174.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update to [CLDR 47](https://cldr.unicode.org/downloads/cldr-47) data including update to ISO 4217 currency information.\n\n* Support passing rounding options to `Money.split/3`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.19.2 - Changelog","doc":"This is the changelog for Money v5.19.2 released on February 13th, 2025.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-19-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix using Elixir 1.18's JSON module in `ex_money` (specifically decoding exchange rates). Thanks to @allenwyma and @maikelthedev for the report. Closes #176.\n\n* Document `currency_symbol: :none` option for `Money.to_string/2`. This option formats the money amount without a currency symbol. This may be useful for UI forms that separate the currency and the amount for input. Requires [ex_cldr_numbers version 2.33.6](https://hex.pm/packages/ex_cldr_numbers/2.33.6) or later.\n\n* Fix parsing money strings that contain trailing [RTL markers](https://en.wikipedia.org/wiki/Implicit_directional_marks). Requires [ex_cldr_currencies version 2.16.4](https://hex.pm/packages/ex_cldr_currencies/2.16.4) or later.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.19.1 - Changelog","doc":"This is the changelog for Money v5.19.1 released on January 22nd, 2025.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-19-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixes `Money.within/3` when `min` and `max` are the same. Thanks to @joewunderlich for the report. Closes #177.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.19.0 - Changelog","doc":"This is the changelog for Money v5.19.0 released on January 1st, 2025.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-19-0"},{"type":"extras","title":"Deprecations - Changelog","doc":"* `Money.default_backend/0` is soft deprecated in favor of `Money.default_backend!/0` whose naming better expresses the side effect of an exception being raised if no default backend is configured.","ref":"changelog.html#deprecations"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Surface an error exception if, when starting the exchange rates service, no `Money.default_backend!/0` is configured.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.18.0 - Changelog","doc":"This is the changelog for Money v5.18.0 released on September 18th, 2024.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-18-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds `min/2`, `max/2`, `min!/2`, `max!/2`, `clamp/3`, `clamp!/3`, `negate/1`, `negate!/1` and `within?/3`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.17.2 - Changelog","doc":"This is the changelog for Money v5.17.2 released on September 18th, 2024.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-17-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix implementation of `Money.exclude_protocol_implementation/1` for `Jason.Encoder`. Thanks to @wkirschbaum for the PR. Closes #171.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add  `Money.exclude_protocol_implementation/1` for `String.Chars`. Thanks to @wkirschbaum for the PR. Closes #172.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.17.1 - Changelog","doc":"This is the changelog for Money v5.17.1 released on September 6th, 2024.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-17-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Update `poison` optional dependency to allow `~> 6.0`.\n\n* Update `stream_data` test dependency to `~> 1.0`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Improve specs and broaden dialyzer configuration.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.17.0 - Changelog","doc":"This is the changelog for Money v5.17.0 released on May 28th, 2024.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-17-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixes warnings on Elixir 1.17.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.16.1 - Changelog","doc":"This is the changelog for Money v5.16.1 released on April 23rd, 2024.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-16-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* `Money.sum/2` default exchange rates is always `%{}` even if the exchange rate server is not running. Thanks to @haste for the report. Closes #168.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.16.0 - Changelog","doc":"This is the changelog for Money v5.16.0 released on April 21st, 2024.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-16-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* When parsing numbers, use the localized number system separators where they exist. Thanks to @pshoukry for the report. Closes #167.\n\n* Surface errors when starting the exchange rates retriever. Thanks to @danschultzer for the PR. Closes #165.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update to [CLDR 45.0](https://cldr.unicode.org/index/downloads/cldr-45) data.\n\n* Return structured errors for `Money.ExchangeRates.latest_rates/0`, `Money.ExchangeRates.historic_rates/1`, `Money.ExchangeRates.last_updated/0` and `Money.ExchangeRates.latest_rates_available?/0` when the exchange rates retrieval process is not running.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.15.4 - Changelog","doc":"This is the changelog for Money v5.15.4 released on March 1st, 2024.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-15-4"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix exchange rate conversions for digital tokens. Thanks much to @ddanschultzer for the PR. Closes 164.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Format the exchange rate retrieval interval used in the init message using the default `cldr` backend configured for `:ex_money`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.15.3 - Changelog","doc":"This is the changelog for Money v5.15.3 released on January 4th, 2024.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-15-3"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Add `or ~> 4.0` for `:phoenix_html` dependency. Thanks to @wkirschbaum for the PR. Closes #161.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.15.2 - Changelog","doc":"This is the changelog for Money v5.15.2 released on November 3rd, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-15-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix compilation warnings on doctests on Elixir 1.16.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.15.1 - Changelog","doc":"This is the changelog for Money v5.15.1 released on October 10th, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-15-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixes the exchange rate retriever, removing the double retrieval loop.  Thanks to @dbernheisel for the report. Closes #152.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.15.0 - Changelog","doc":"This is the changelog for Money v5.15.0 released on July 24th, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-15-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds an option `:no_fraction_if_integer` to `Money.to_string/2`. If `truthy` this option will set `fractional_digits: 0` if `money` is an integer value. This may be helpful in cases where integer money amounts such as `Money.new(:USD, 1234)` should be formatted as `$1,234` rather than `$1,234.00`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.14.1 - Changelog","doc":"This is the changelog for Money v5.14.1 released on July 23rd, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-14-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix `Logger.warn/1` warnings by moving to `Logger.warning/1`.\n\n* Fix failing test case.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.14.0 - Changelog","doc":"This is the changelog for Money v5.14.0 released on April 29th, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-14-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds `Money.integer?/1` to return a boolean indicatng if a money amount is an integer value (ie has no significant fractional digits).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.13.0 - Changelog","doc":"This is the changelog for Money v5.13.0 released on April 28th, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-13-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Updates to [ex_cldr version 2.37.0](https://hex.pm/packages/ex_cldr/2.37.0) which includes data from [CLDR release 43](https://cldr.unicode.org/index/downloads/cldr-43)\n\n* Tests now assume Decimal ~> 2.0 since the `Inspect` protocol implementation now emits executable code examples.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.12.4 - Changelog","doc":"This is the changelog for Money v5.12.4 released on March 30th, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.12.4` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.25` or later.","ref":"changelog.html#money-v5-12-4"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Delegates http requests (used in exchange rates retrieval) to `Cldr.Http.get_with_headers/2`. This centralizes all HTTP get requests for all `ex_cldr_*` libraries to this one function which can then be reviewed and managed for security concerns.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.12.3 - Changelog","doc":"This is the changelog for Money v5.12.3 released on October 13th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.12.3` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.25` or later.","ref":"changelog.html#money-v5-12-3"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix `NaN` and `Inf` amount detection to be compatible with Decimal 1.x and 2.x. Thanks to @Lostkobrakai for the PR. Closes #144.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.12.2 - Changelog","doc":"This is the changelog for Money v5.12.2 released on October 13th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.12.2` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.25` or later.","ref":"changelog.html#money-v5-12-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Don't create \"NaN\" or \"Inf\" valued Money structs. Thanks for @coladarci for the report. Closes #143.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.12.1 - Changelog","doc":"This is the changelog for Money v5.12.1 released on August 27th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.12.1` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.25` or later.","ref":"changelog.html#money-v5-12-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Removes compile-time warnings for Elixir 1.14 (use `Application.compile_env/2`, not `Application.get_env/2`)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.12.0 - Changelog","doc":"This is the changelog for Money v5.12.0 released on June 8th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.12.0` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.25` or later.","ref":"changelog.html#money-v5-12-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `Money.localize/2` to convert a money amount into the currency in affect for the given locale.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.11.0 - Changelog","doc":"This is the changelog for Money v5.11.0 released on May 14th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.11.0` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.25` or later.","ref":"changelog.html#money-v5-11-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds support for [ISO 24165 Digital Tokens (crypto currency)](https://www.iso.org/standard/80601.html). Digital Token-based money behaves the same as currency-based money with the following exceptions due to limited data availability:\n\n  * Digital token names are not localized (there is no localised data available in CLDR)\n  * Digital token names are not pluralized (also because there is no localised data available)\n  * Digital token amounts are never rounded (there is no data available to standardise on rounding rules or the number of fractional digits to round to)","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.10.0 - Changelog","doc":"This is the changelog for Money v5.10.0 released on April 6th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.10.0` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.25` or later.","ref":"changelog.html#money-v5-10-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `Money.zero?/1`, `Money.positive?/1` and `Money.negative?/1`. Thanks to @emaiax for the PR.\n\n* Update [CLDR](https://cldr.unicode.org) to [release 41](https://cldr.unicode.org/index/downloads/cldr-41) in [ex_cldr version 2.28.0](https://hex.pm/packages/ex_cldr/2.28.0) and [ex_cldr_numbers 2.26.0](https://hex.pm/packages/ex_cldr_numbers/2.26.0).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.9.0 - Changelog","doc":"This is the changelog for Money v5.9.0 released on February 21st, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.9.0` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.25` or later.","ref":"changelog.html#money-v5-9-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Updates to [ex_cldr version 2.26.0](https://hex.pm/packages/ex_cldr/2.26.0) and [ex_cldr_numbers version 2.25.0](https://hex.pm/packages/ex_cldr_numbers/2.25.0) which use atoms for locale names and rbnf locale names. This is consistent with other elements of `t:Cldr.LanguageTag` where atoms are used when the cardinality of the data is fixed and relatively small and strings where the data is free format.\n\n* Adjusts the output of `Money.inspect/2` to be executable code. Instead of `#Money<:USD, 100>` the output will be `Money.new(:USD, \"100\")`. This improved developer experience by allowing for copy/paste of `inspect/2` results into `iex`. It is also in line with similar changes being made in `elixir`, `Decimal` and others.\n\n* Add documentation for `:currency_symbol` option for `Money.to_string/2`. Although its an option that is passed through to `Cldr.Number.to_string/3`, its very relevant to `t:Money` formatting.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.8.0 - Changelog","doc":"This is the changelog for Money v5.8.0 released on January 31st, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.8.0` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.23` or later.","ref":"changelog.html#money-v5-8-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds configuration option `:exclude_protocol_implementations` to omit generating implementations for one or more protocols from the list `Json.Encoder`, `Phoneix.HTML.Safe` and `Gringotts.Money`. Thanks to @jgough-playoxygen for the [suggestion](https://elixirforum.com/t/cldr-number-custom-formatter/45520).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.7.4 - Changelog","doc":"This is the changelog for Money v5.7.4 released on December 23rd, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.7.4` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.23` or later.","ref":"changelog.html#money-v5-7-4"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix `Money.to_integer_exp/1` when `t:Money` has a negative amount.  Thanks to @hamptokr for the report and the PR.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.7.3 - Changelog","doc":"This is the changelog for Money v5.7.3 released on December 19th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.7.3` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.23` or later.","ref":"changelog.html#money-v5-7-3"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixes retrieving exchange rates on OTP releases before OTP 22.  Thanks to @fbettag for the report, collaboration and patience.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.7.2 - Changelog","doc":"This is the changelog for Money v5.7.2 released on December 17th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.7.2` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.23` or later.","ref":"changelog.html#money-v5-7-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix spec for `Money.from_integer/3`. Thanks to @jdewar for the report.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support a `:fractional_digits` option for `Money.from_integer/3` and improve the documentation.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.7.1 - Changelog","doc":"This is the changelog for Money v5.7.1 released on December 8th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\n**Note** `ex_money 5.7.1` is supported on Elixir 1.10 and later versions only. It also requires `ex_cldr_numbers 2.23` or later.","ref":"changelog.html#money-v5-7-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix dialyzer warnings on Elixir 1.12 and 1.13\n\n* Replace `use Mix.Config` with `import Config` in configuration files since the former is deprecated.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.7.0 - Changelog","doc":"This is the changelog for Money v5.7.0 released on October 28th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-7-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Updates to support [CLDR release 40](https://cldr.unicode.org/index/downloads/cldr-40) via [ex_cldr version 2.24](https://hex.pm/packages/ex_cldr/2.24.0)","ref":"changelog.html#enhancements"},{"type":"extras","title":"Deprecations - Changelog","doc":"* Don't call deprecated `Cldr.Config.get_locale/2`, use `Cldr.Locale.Loader.get_locale/2` instead.","ref":"changelog.html#deprecations"},{"type":"extras","title":"Money v5.6.0 - Changelog","doc":"This is the changelog for Money v5.6.0 released on August 31st, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-6-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds `Money.to_currency_code/1` to return the currency code part of a `t:Money`. Thanks to @Adzz for the proposal. Closes #130.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.5.5 - Changelog","doc":"This is the changelog for Money v5.5.5 released on August 15th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-5-5"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Allow either `phoenix_html` version 2.x or 3.x. Thanks to @seantanly for the PR. Closes #129.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.5.4 - Changelog","doc":"This is the changelog for Money v5.5.4 released on June 17th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-5-4"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Support `t:Cldr.Number.Format.Options` as an argument to `Money.to_string/2`.  Thanks to @jeroenvisser101 for the PR. Closes #127.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.5.3 - Changelog","doc":"This is the changelog for Money v5.5.3 released on May 7th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-5-3"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixes parsing money when a currency string has a \".\" in it such as \"kr.\". Thanks for the report to @Doerge. Closes #125.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.5.2 - Changelog","doc":"This is the changelog for Money v5.5.2 released on April 14th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-5-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix exception message when describing the requirement for a default backend configuration. Thanks to @holandes22 for the report. Closes #124.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.5.1 - Changelog","doc":"This is the changelog for Money v5.5.1 released on February 18th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-5-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix formatting a `t:Money` that has no `:format_options` key. That can happen if re-hydrating a `t:Money` using `:erlang.binary_to_term/1` from an older version of `ex_money` that doesn't have the `:format_options` key in the struct.  Thanks to @coladarci. Fixes #123.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.5.0 - Changelog","doc":"This is the changelog for Money v5.5.0 released on February 10th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-5-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds format options to `t:Money` to allow per-money formatting options to be applied with the `String.Chars` protocol. Thanks to @morinap for the feature request.\n\n* Adds `Money.put_format_options/2`","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.4.1 - Changelog","doc":"This is the changelog for Money v5.4.1 released on January 7th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-4-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Update `stream_data` to remove stacktrace warning\n\n* Use `Cldr.default_backend!/0` instead of deprecated `Cldr.default_backend/0` in tests. Closes #120. Thanks to @darwintantuco.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.4.0 - Changelog","doc":"This is the changelog for Money v5.4.0 released on November 1st, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-4-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add support for [CLDR 38](http://cldr.unicode.org/index/downloads/cldr-38)","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.3.2 - Changelog","doc":"This is the changelog for Money v5.3.2 released on September 30th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-3-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix docs for `Money.div!/2`\n\n* Update `cldr_utils` which implements a shim for `Decimal` to support both version `1.9` and `2.0`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.3.1 - Changelog","doc":"This is the changelog for Money v5.3.1 released on September 26th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-3-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Support `nimble_parsec` versions that match `~> 0.5 or ~> 1.0`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.3.0 - Changelog","doc":"This is the changelog for Money v5.3.0 released on September 5th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-3-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix parsing money amounts to use Unicode definition of whitespace (set `[:Zs:]`). Thanks to @Sanjibukai for the report.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `Money.sum/2` to sum a list of `Money`, converting them if required.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.2.1 - Changelog","doc":"This is the changelog for Money v5.2.1 released on June 23rd, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-2-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Configure the `Money.Application` supervisor via the arguments to `Money.Application.start/2` and configure defaults in `mix.exs`. This permits different restart strategies and names.\n\n* Add `Money.ExchangeRates.Supervisor.default_supervisor/0` to return the name of the default supervisor which is `Money.Supervisor`\n\n* Change `Money.ExchangeRates.Supervisor.stop/0` to become `Money.ExchangeRates.Supervisor.stop/{0, 1}` allowing the supervisor name to be passed in. The default is `Money.ExchangeRates.Supervisor.default_supervisor/0`","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Add back the name of the Application supervisor, `Money.Supervisor`. Thanks for the report of the regression to @jeroenvisser101. Fixes #117.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.2.0 - Changelog","doc":"This is the changelog for Money v5.2.0 released on May 30th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-2-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds a configuration option `:verify_peer` which is a boolean that determines whether to verify the client certificate for any exchange rate service API call. The default is `true`. This option should not be changed without a very clear understanding of the security implications. This option will remain undocumented but supported for now.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Handle expired certificate errors on the exchange rates API service and log them. Thanks to @coladarci. Fixes #116","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.1.0 - Changelog","doc":"This is the changelog for Money v5.1.0 released on May 26th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-1-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Extract default currency from locale when calling `Money.parse/2` on a money string. The updated docs now say:\n\n  * `:default_currency` is any valid currency code or `false`\n    that will used if no currency code, symbol or description is\n    indentified in the parsed string. The default is `nil`\n    which means that the default currency associated with\n    the `:locale` option will be used. If `false` then the\n    currency assocated with the `:locale` option will not be\n    used and an error will be returned if there is no currency\n    in the string being parsed.\n\n* Add certificate verification for exchange rate retrieval","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v5.0.2 - Changelog","doc":"This is the changelog for Money v5.0.2 released on April 29th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-0-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Update the application supervisor spec","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.0.1 - Changelog","doc":"This is the changelog for Money v5.0.1 released on January 28th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-0-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Make `nimble_parsec` a required dependency since it is required for parsing money amounts. Thanks to @jonnystoten for the report.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v5.0.0 - Changelog","doc":"This is the changelog for Money v5.0.0 released on January 21st, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v5-0-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* Elixir 1.10 introduces semantic sorting for stucts that depends on the availability of a `compare/2` function that returns `:lt`, `:eq` or `:gt`.  Therefore in this release of `ex_money` the functions `compare/2` and `compare!/2` are swapped with `cmp/2` and `cmp!/2` in order to conform with this expectation. Now `compare/2` will return `:eq`, `:lt` or `:gt`. And `cmp/2` return `-1`, `0` or `1`.\n\n* Deprecate `Money.reduce/1` in favour of `Money.normalize/1` to be consistent with `Decimal` versions `1.9` and later.\n\nIt is believed and tested that `Money` version `5.0.0` is compatible with all versions of `Decimal` from `1.6` up to the as-yet-unreleased `2.0`.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Support of Elixir 1.10 Enum sorting - Changelog","doc":"From Elixir verison `1.10.0`, several functions in the `Enum` module can use the `Money.compare/2` function to simplify sorting. For example:\n\n```\niex> list = [Money.new(:USD, 100), Money.new(:USD, 200)]\n[#Money<:USD, 100>, #Money<:USD, 200>]\niex> Enum.sort list, Money\n[#Money<:USD, 100>, #Money<:USD, 200>]\niex> Enum.sort list, {:asc, Money}\n[#Money<:USD, 100>, #Money<:USD, 200>]\niex> Enum.sort list, {:desc, Money}\n[#Money<:USD, 200>, #Money<:USD, 100>]\n```\n**Note that `Enum.sort/2` will sort money amounts even when the currencies are incompatible. In this case the order of the result is not predictable. It is the developers responsibility to filter the list to compatible currencies prior to sorting. This is a limitation of the `Enum.sort/2` implementation.**","ref":"changelog.html#support-of-elixir-1-10-enum-sorting"},{"type":"extras","title":"Notes on Decimal version support - Changelog","doc":"* `ex_money` version `5.0.0` is compatible with `Decimal` versions from `1.6` onwards. In `Decimal` version `2.0` the same changes to `compare/2` and `cmp/2` will occur and in `Decimal` version `1.9`, `Decimal.cmp/2` is deprecated.  `ex_money` version `5.0.0` detects these different versions of `Decimal` and therefore remains compatability with `Decimal` back to version `1.6`.","ref":"changelog.html#notes-on-decimal-version-support"},{"type":"extras","title":"Money v4.4.2 - Changelog","doc":"This is the changelog for Money v4.4.2 released on January 2nd, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v4-4-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Remove calls to `Code.ensure_compiled?/1` since it is deprecated in Elixir 1.10. Use instead `Cldr.Config.ensure_compiled?/1` which is added as a private API in `Cldr` version 2.12.0. This version of `Cldr` now becomes the minimum version required.\n\n* Remove spurious entries in `.dialyzer_ignore_warnings` - no entries are required and dialyzer is happy.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v4.4.1 - Changelog","doc":"This is the changelog for Money v4.4.1 released on November 10th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v4-4-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixes money parsing error. Thanks to @Doerge. Closes #112.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v4.4.0 - Changelog","doc":"This is the changelog for Money v4.4.0 released on November 6th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v4-4-0"},{"type":"extras","title":"Breaking Change - Changelog","doc":"* `Money.parse/2` until this release supported the `:currency_filter` option. It allowed for currencies to be filtered based upon their attributes (`:all`, `:current`, `:historic`, `:tender`, `:annotated`). When multiple attributes were passed in a list, a currency had to meet all of these attributes. From this release onwards, multiple attributes items are `or`ed, not `and`ed. It is expected this option is used extremely rarely and therefore of limited impact.","ref":"changelog.html#breaking-change"},{"type":"extras","title":"Enhancements - Changelog","doc":"* `Money.parse/2` now includes the option `:default_currency` which allows for parsing a number only (without a currency code) and it will be tagged with the `:default_currency`.\n```elixir\n  iex> Money.parse(\"100\")\n  {:error,\n   {Money.Invalid,\n    \"A currency code, symbol or description must be specified but was not found in \\\"100\\\"\"}}\n  iex> Money.parse(\"100\", default_currency: :USD)\n  #Money<:USD, 100>\n```\n\n* Add `:only` and `:except` options to `Money.parse/2` to specify which currency codes or currency attributes are permitted.  `:only` and `:except` replace the option `:currency_filter` which is now deprecated. If provided, `:currency_filter` is interpreted as `:only`. An example:\n```elixir\n  iex> Money.parse(\"100 usd\", only: :current, except: :USD)\n  {:error,\n   {Money.UnknownCurrencyError,\n    \"The currency \\\"usd\\\" is unknown or not supported\"}}\n```\n\n* `Money.parse/2` now supports negative money amounts.\n```elixir\n  iex> Money.parse(\"chf -100\")\n  #Money<:CHF, -100>\n\n  iex> Money.parse(\"(chf 100)\")\n  #Money<:CHF, -100>\n```\n* The money parser has been rewritten using [nimble_parsec](https://hex,pm.packages/nimble_parsec)","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v4.3.0 - Changelog","doc":"This is the changelog for Money v4.3.0 released on September 8th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v4-3-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds a `Money` backend in the same spirit as other libraries that leverge [ex_cldr](https://hex,pm/packages/ex_cldr). Thanks to @Lostkobrakai. Closes #108. All of the functions in the `Money` module may also be called on a backend module ` .Money.fun` without having to specify a backend module since this is implicit.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* `Money.new!/3` replaces `Money.new!/2` to accept options. Thanks to @Lostkobrakai. Closes #109.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v4.2.2 - Changelog","doc":"This is the changelog for Money v4.2.2 released on September 7th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v4-2-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Use `Keyword.get_lazy` when the default is `Cldr.default_backend/0` to avoid exceptions when no default backend is configured. Thanks to @Lostkobrakai. Closes #108.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v4.2.1 - Changelog","doc":"This is the changelog for Money v4.2.1 released on September 2nd, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v4-2-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixes parsing of money amount that have a single digit amount. Closes #107.  Thanks to @njwest","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v4.2.0 - Changelog","doc":"This is the changelog for Money v4.2.0 released on 21 August, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v4-2-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Move the `Money.Migration` module to [ex_money_ecto](https://hex.pm/packages/ex_money_sql) where it belongs","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* `Money.default_backend/0` will now either use the backend configured under the `:default_cldr_backend` key of `ex_money` or `Cldr.default_backend/0`. In either case an exeption will be raised if no default backend is configured.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v4.1.0 - Changelog","doc":"This is the changelog for Money v4.1.0 released on July 13th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v4-1-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds `Money.abs/1`. Thanks to @jeremyjh.\n\n* Improve `@doc` consistency using `","ref":"changelog.html#enhancements"},{"type":"extras","title":"Arguments` not `## Options`. - Changelog","doc":"","ref":"changelog.html#arguments-not-options"},{"type":"extras","title":"Money v4.0.0 - Changelog","doc":"This is the changelog for Money v4.0.0 released on July 8th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v4-0-0"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* Functions related to the serialization of money types have been extracted to the library [ex_money_sql](https://hex.pm/packages/ex_money_sql).  For applications using the dependency `ex_money` that *do not* require serialization no changes are required.  For applications using serialization, the dependency should be changed to `ex_money_sql` (which in turn depends on `ex_money`).\n\n* Supports Elixir 1.6 and later only","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Money v3.4.4 - Changelog","doc":"This is the changelog for Money v3.4.4 released on June 2nd, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-4-4"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Supports passing an `Cldr.Number.Formation.Options.t` as alternative to a `Keyword.t` for options to `Money.to_string/2`.  Performance is doubled when using pre-validated options which is useful if formatting is being executed in a tight loop.\n\nAn example of this usage is:\n\n```elixir\n  iex> money = Money.new(:USD, 100)\n\n  # Apply any options required as a keyword list\n  # Money will take care of managing the `:currency` option\n  iex> options = []\n\n  iex> {:ok, options} = Cldr.Number.Format.Options.validate_options(0, backend, options)\n  iex> Money.to_string(money, options)\n```\n\nThe `0` in `validate_options` is used to determine the sign of the amount because that can influence formatting - for example the accounting format often uses `(1234)` as its format.  If you know your amounts are always positive, just use `0`.\n\nIf the use case may have both positive and negative amounts, generate two option sets (one with the positive number and one with the negative).  Then use the appropriate option set.  For example:\n\n```elixir\n  iex> money = Money.new(:USD, 1234)\n\n  # Add options as required\n  # Money will take care of managing the `:currency` option\n  iex> options = []\n\n  iex> {:ok, positive_options} = Cldr.Number.Format.Options.validate_options(0, backend, options)\n  iex> {:ok, negative_options} = Cldr.Number.Format.Options.validate_options(-1, backend, options)\n\n  iex> if Money.cmp(money, Money.zero(:USD)) == :gt do\n  ...>   Money.to_string(money, positive_options)\n  ...> else\n  ...>   Money.to_string(money, negative_options)\n  ...> end\n```","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v3.4.3 - Changelog","doc":"This is the changelog for Money v3.4.3 released on June 2nd, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-4-3"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Ensure `Money.to_string!/2` properly raises\n\n* Add specs for `Money.to_string/2` and `Money.to_string!/2`\n\nThanks to @rodrigues for the report and PR.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v3.4.2 - Changelog","doc":"This is the changelog for Money v3.4.2 released on April 16th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-4-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* `Money.put_fraction/2` now correctly allows setting the fraction to 0.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* `Money.round/2` allows setting `:currency_digits` to an integer number of digits in addition to the options `:iso`, `:cash` and `:accounting`.  The default remains `:iso`.\n\n* Improves the documentation for `Money.to_string/2`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v3.4.1 - Changelog","doc":"This is the changelog for Money v3.4.1 released on April 5th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-4-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix `README.md` markdown formatting error.  Thanks to @fireproofsocks for the report and @lostkobrakai for the fix.  Closes #99.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v3.4.0 - Changelog","doc":"This is the changelog for Money v3.4.0 released on March 28th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-4-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Updates to [CLDR version 35.0.0](http://cldr.unicode.org/index/downloads/cldr-35) released on March 27th 2019 through `ex_cldr` version 2.6.0.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v3.3.1 - Changelog","doc":"This is the changelog for Money v3.3.1 released on March 8th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-3-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix or silence dialyzer warnings","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v3.3.0 - Changelog","doc":"This is the changelog for Money v3.3.0 released on February 24th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-3-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds `Money.put_fraction/2`. This will set the fractional part of a money to the specified integer amount.  Examples:\n```\n  iex> Money.put_fraction Money.new(:USD, \"2.49\"), 99\n  #Money<:USD, 2.99>\n\n  iex> Money.put_fraction Money.new(:USD, \"2.49\"), 999\n  {:error,\n   {Money.InvalidAmountError, \"Rounding up to 999 is invalid for currency :USD\"}}\n```","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Parsing money strings now uses a more complete set of character definitions for decimal and grouping separators based upon the `characters.json` file of the \"en\" locale.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v3.2.4 - Changelog","doc":"This is the changelog for Money v3.2.4 released on February 13th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-2-4"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Updates to [ex_cldr_currencies version 2.1.2](https://hex.pm/packages/ex_cldr_currencies/2.1.2) which correctly removes duplicate currency strings when the same string referred to different currency codes. See the [changelog](https://github.com/kipcole9/cldr_currencies/blob/v2.1.2/CHANGELOG.md) for further detail.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds a `:fuzzy` option to `Money.parse/2` that uses `String.jaro_distance/2` to help determine if the provided currency text can be resolved as a currency code.  For example:\n```\n  iex> Money.parse(\"100 eurosports\", fuzzy: 0.8)\n  #Money<:EUR, 100>\n\n  iex> Money.parse(\"100 eurosports\", fuzzy: 0.9)\n  {:error,\n   {Money.Invalid, \"Unable to create money from \\\\\"eurosports\\\\\" and \\\\\"100\\\\\"\"}}\n```","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v3.2.3 - Changelog","doc":"This is the changelog for Money v3.2.3 released on February 12th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-2-3"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Correctly parse money strings with unicode currency symbols like \"€\". Closes #95.  Thanks to @crbelaus.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v3.2.2 - Changelog","doc":"This is the changelog for Money v3.2.2 released on February 10th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-2-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Improves parsing of money strings. Parsing now uses various strings that [CLDR](https://cldr.unicode.org) knows about.  Some examples:\n\n```elixir\n iex> Money.parse \"$au 12 346\", locale: \"fr\"\n #Money<:AUD, 12346>\n iex> Money.parse \"12 346 dollar australien\", locale: \"fr\"\n #Money<:AUD, 12346>\n iex> Money.parse \"A$ 12346\", locale: \"en\"\n #Money<:AUD, 12346>\n iex> Money.parse \"australian dollar 12346.45\", locale: \"en\"\n #Money<:AUD, 12346.45>\n iex> Money.parse \"AU$ 12346,45\", locale: \"de\"\n #Money<:AUD, 12346.45>\n\n # Can also return the strings available for a given currency\n # and locale\n iex> Cldr.Currency.strings_for_currency :AUD, \"de\"\n [\"aud\", \"au$\", \"australischer dollar\", \"australische dollar\"]\n\n # Round trip formatting also seems to be ok\n iex> {:ok, string} = Cldr.Number.to_string 1234, Money.Cldr, currency: :AUD\n iex> Money.parse string\n #Money<:AUD, 1234.00>\n```","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v3.2.1 - Changelog","doc":"This is the changelog for Money v3.2.1 released on February 2nd, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-2-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Added `Money.Ecto.Composite.Type.cast/1` and `Money.Ecto.Map.Type.cast/1` for a `String.t` parameter. When a `String.t` is provided, `cast/1` will call `Money.parse/2` to create the `Money.t`.\n\n* `Money.new/3` now uses the current locale on the default backend if no locale or backend is specified. This means that `Money.Ecto.Composite.Type.cast/1` and `Money.Ecto.Map.Type.cast/1` will be parsed using the locale that has been set for the current process in the default backend. As a result, a simple `type=text` form field can be used to input a money type (currency code and amount in a single string) that can then be cast to a `Money.t`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v3.2.0 - Changelog","doc":"This is the changelog for Money v3.2.0 released on February 1st, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-2-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Correctly generate `migrations_path/1` function based upon whether `Ecto` is configured and which version","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds `Money.parse/2` which will parse a string comprising a currency code and an amount. It will return a `Money.t` or an error.  This function may be helpful in supporting money input in HTML forms.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Money v3.1.0 - Changelog","doc":"This is the changelog for Money v3.1.0 released on December 30th, 2018.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)","ref":"changelog.html#money-v3-1-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix typo in `exchange_rates_retriever.ex`.  Thanks to @lostkobrakai.  Closes #91.\n\n* Remove obsolete `cldr` compiler\n\n* Changes the `sum` aggregate function for `money_with_currency` to be `STRICT` which means it handles `NULL` columns in the same way as the standard `SUM` function.  Thanks to @lostkobrakai.  Closes #88.\n\n* Fixes documentation link errors\n\n* Fix unhandled terminate typo error in exchange rates server. Thanks to @xavier. Closes #90.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Money v3.0.0 - Changelog","doc":"This is the changelog for Money v3.0.0 released on November 23rd, 2018.  For older changelogs please consult the release tag on [GitHub](https://github.com/kipcole9/money/tags)\n\nThe primary purpose of this release is to support ex_cldr version 2.0","ref":"changelog.html#money-v3-0-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* `Money.from_tuple/1` has been removed\n* Uses [ex_cldr](https://hex.pm/packages/ex_cldr/2.0.0) version 2.  Please see [the changelog](https://github.com/kipcole9/cldr/blob/v2.0.1/CHANGELOG.md#migrating-from-cldr-1x-to-cldr-version-2x) for configuration changes that are required.\n* Requires a default_cldr_backend to be configured in `config.exs`.  For example:\n```\n  config :ex_money,\n    ...\n    default_cldr_backend: MyApp.Cldr\n  end\n```","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"LICENSE","doc":"","ref":"license.html"},{"type":"extras","title":"License - LICENSE","doc":"Copyright 2017-2021 Kip Cole\n\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use this file except in\ncompliance with the License. You may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software distributed under the License\nis distributed on an \"AS IS\" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\nimplied. See the License for the specific language governing permissions and limitations under the\nLicense.","ref":"license.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}